//! This file has been automatically generated by `objc2`'s `header-translator`.
//! DO NOT EDIT
use core::ffi::*;
use core::ptr::NonNull;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserialbsdservicevalue?language=objc)
pub const kIOSerialBSDServiceValue: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOSerialBSDClient\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserialbsdtypekey?language=objc)
pub const kIOSerialBSDTypeKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOSerialBSDClientType\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserialbsdalltypes?language=objc)
pub const kIOSerialBSDAllTypes: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOSerialStream\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserialbsdmodemtype?language=objc)
pub const kIOSerialBSDModemType: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOSerialStream\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserialbsdrs232type?language=objc)
pub const kIOSerialBSDRS232Type: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOSerialStream\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiottydevicekey?language=objc)
pub const kIOTTYDeviceKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOTTYDevice\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiottybasenamekey?language=objc)
pub const kIOTTYBaseNameKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOTTYBaseName\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiottysuffixkey?language=objc)
pub const kIOTTYSuffixKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOTTYSuffix\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocalloutdevicekey?language=objc)
pub const kIOCalloutDeviceKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOCalloutDevice\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodialindevicekey?language=objc)
pub const kIODialinDeviceKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IODialinDevice\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiottywaitforidlekey?language=objc)
pub const kIOTTYWaitForIdleKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOTTYWaitForIdle\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/user_ul_t?language=objc)
pub type user_ul_t = u64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/user_speed_t?language=objc)
pub type user_speed_t = u64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/user_us_t?language=objc)
pub type user_us_t = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/user_shspeed_t?language=objc)
pub type user_shspeed_t = u32;
