// This file has been automatically generated by `objc2`'s `header-translator`.
// DO NOT EDIT

#![allow(unused_imports)]
#![allow(deprecated)]
#![allow(non_snake_case)]
#![allow(non_camel_case_types)]
#![allow(non_upper_case_globals)]
#![allow(missing_docs)]
#![allow(clippy::too_many_arguments)]
#![allow(clippy::type_complexity)]
#![allow(clippy::upper_case_acronyms)]
#![allow(clippy::identity_op)]
#![allow(clippy::missing_safety_doc)]
#![allow(clippy::doc_lazy_continuation)]
#![allow(rustdoc::broken_intra_doc_links)]
#![allow(rustdoc::bare_urls)]
#![allow(rustdoc::invalid_html_tags)]

#[link(name = "IOKit", kind = "framework")]
extern "C" {}

#[cfg(feature = "graphics")]
#[path = "graphics.rs"]
mod __graphics;
#[cfg(feature = "hid")]
#[path = "hid.rs"]
mod __hid;
#[cfg(feature = "hidsystem")]
#[path = "hidsystem.rs"]
mod __hidsystem;
#[cfg(feature = "network")]
#[path = "network.rs"]
mod __network;
#[cfg(feature = "ps")]
#[path = "ps.rs"]
mod __ps;
#[cfg(feature = "pwr_mgt")]
#[path = "pwr_mgt.rs"]
mod __pwr_mgt;
#[cfg(feature = "serial")]
#[path = "serial.rs"]
mod __serial;
#[cfg(feature = "usb")]
#[path = "usb/mod.rs"]
mod __usb;

#[cfg(feature = "graphics")]
pub(crate) use self::__graphics::_IOBlitMemory;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAccelSurfaceLockBits;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAccelSurfaceMemoryTypes;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAccelSurfaceMethods;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAccelSurfaceModeBits;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAccelSurfaceScaleBits;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAccelSurfaceShapeBits;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAccelSurfaceStateBits;
#[cfg(feature = "graphics")]
pub use self::__graphics::eIOAcceleratorClientTypes;
#[cfg(feature = "graphics")]
pub use self::__graphics::kAndConnections;
#[cfg(feature = "graphics")]
pub use self::__graphics::kAppleDisplayTypeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kAppleSenseKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionAudioStreaming;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionBlueGammaScale;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionChanged;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionCheckEnable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionColorDepthsSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionColorMode;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionColorModesSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionControllerColorDepth;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionControllerDepthsSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionControllerDitherControl;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionDisplayFlags;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionDisplayParameterCount;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionDisplayParameters;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionEnable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionEnableAudio;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionFlags;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionFlushParameters;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionGammaScale;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionGreenGammaScale;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionHandleDisplayPortEvent;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionIgnore;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionOverscan;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionPanelTimingDisable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionPostWake;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionPower;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionProbe;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionRedGammaScale;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionStartOfFrameTime;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionSupportsAppleSense;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionSupportsHLDDCSense;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionSupportsLLDDCSense;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionSyncEnable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionSyncFlags;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionVBLMultiplier;
#[cfg(feature = "graphics")]
pub use self::__graphics::kConnectionVideoBest;
#[cfg(feature = "graphics")]
pub use self::__graphics::kCurrentGraphicsInterfaceRevision;
#[cfg(feature = "graphics")]
pub use self::__graphics::kCurrentGraphicsInterfaceVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayBlueGamma;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayBluePointX;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayBluePointY;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayBrightnessAffectsGamma;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayBundleKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayCSProfile;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayFixedPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayGammaChannels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayGammaEntryCount;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayGammaEntrySize;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayGammaTable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayGreenGamma;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayGreenPointX;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayGreenPointY;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayHorizontalImageSize;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeAcceleratorBackedFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeAlwaysShowFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeBuiltInFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeDefaultFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeInterlacedFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeNativeFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeNeverShowFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeNotGraphicsQualityFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeNotPresetFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeNotResizeFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeRequiresPanFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeSafeFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeSafetyFlags;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeSimulscanFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeStretchedFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeTelevisionFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeValidFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeValidForAirPlayFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeValidForHiResFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeValidForMirroringFlag;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayModeValidateAgainstDisplay;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayProductID;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayProductIDGeneric;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayProductName;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayRedGamma;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayRedPointX;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayRedPointY;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySerialNumber;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySerialString;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelConfiguration;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelConfigurationDelta;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelConfigurationQuad;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelConfigurationStripe;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelConfigurationStripeOffset;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelConfigurationUndefined;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelLayout;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelLayoutBGR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelLayoutQuadGBL;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelLayoutQuadGBR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelLayoutRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelLayoutUndefined;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelShape;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelShapeElliptical;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelShapeOval;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelShapeRectangular;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelShapeRound;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelShapeSquare;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplaySubPixelShapeUndefined;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayVendorID;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayVendorIDUnknown;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayVerticalImageSize;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayViewAngleAffectsGamma;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayWeekOfManufacture;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayWhiteGamma;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayWhitePointX;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayWhitePointY;
#[cfg(feature = "graphics")]
pub use self::__graphics::kDisplayYearOfManufacture;
#[cfg(feature = "graphics")]
pub use self::__graphics::kFramebufferDisableAltivecAccess;
#[cfg(feature = "graphics")]
pub use self::__graphics::kFramebufferSupportsCopybackCache;
#[cfg(feature = "graphics")]
pub use self::__graphics::kFramebufferSupportsGammaCorrection;
#[cfg(feature = "graphics")]
pub use self::__graphics::kFramebufferSupportsWritethruCache;
#[cfg(feature = "graphics")]
pub use self::__graphics::kHardwareCursorDescriptorMajorVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kHardwareCursorDescriptorMinorVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kHardwareCursorInfoMajorVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kHardwareCursorInfoMinorVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO128RGBAFloatPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16BE4444PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16BE555PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16BE565PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16BitFloatPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16LE4444PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16LE5551PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16LE555PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO16LE565PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO1IndexedGrayPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO1MonochromePixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO24BGRPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO24RGBPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO2IndexedGrayPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO2IndexedPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO2vuyPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO30BitDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO32ABGRPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO32ARGBPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO32BGRAPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO32BitFloatPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO32RGBAPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO4IndexedGrayPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO4IndexedPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO64BGRAPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO64BitDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO64RGBAFloatPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO8IndexedGrayPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIO8IndexedPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAccelIndexKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAccelPrivateID;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAccelRevisionKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAccelTypesKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAccelVolatileSurface;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAcceleratorClassName;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAnalogSetupExpected;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAnalogSignalLevel_0700_0000;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAnalogSignalLevel_0700_0300;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAnalogSignalLevel_0714_0286;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOAnalogSignalLevel_1000_0400;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBitsPerColorComponent10;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBitsPerColorComponent12;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBitsPerColorComponent16;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBitsPerColorComponent6;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBitsPerColorComponent8;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBitsPerColorComponentNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitAllOptions;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitBeamSync;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitBeamSyncAlways;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitBeamSyncSpin;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitBeamSyncSwaps;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitBlendOperation;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitColorSpaceTypes;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitCopyOperation;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitDestFramebuffer;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitFixedSource;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitFlushWithSwap;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitFramebufferDestination;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitHasCGSSurface;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitHighlightOperation;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitMemoryRequiresHostFlush;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitOrOperation;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitReferenceSource;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceCGSSurface;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceDefault;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceFramebuffer;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceIsSame;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceMemory;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceOOLMemory;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceOOLPattern;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourcePattern;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSourceSolid;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSurfaceDestination;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSynchronizeFlushHostWrites;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitSynchronizeWaitBeamExit;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeColorSpaceConvert;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeCopyRects;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeCopyRegion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeDestKeyColorEqual;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeDestKeyColorModeMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeDestKeyColorNotEqual;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeHideCursor;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeLines;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeMonoExpand;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeMoveCursor;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeOperationMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeOperationShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeOperationType0;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeOperationTypeMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeRects;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeScale;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeScanlines;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeShowCursor;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeSourceKeyColorEqual;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeSourceKeyColorModeMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeSourceKeyColorNotEqual;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitTypeVerbMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitUnlockWithSwap;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitWaitAll;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitWaitAll2D;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitWaitCheck;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitWaitContext;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitWaitGlobal;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBlitXorOperation;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOBuiltinPanelPowerAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOCLUTPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOCSyncDisable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOCapturedAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOClamshellStateAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryAdobeRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryBT2020;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryBT2100;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryBT601;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryBT709;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryDCIP3;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryNativeRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryWGRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetrysRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOColorimetryxvYCC;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOConnectionBuiltIn;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOConnectionStereoSync;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOCursorControlAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventAutomatedTestRequest;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventContentProtection;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventForceRetrain;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventIdle;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventMCCS;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventRemoteControlCommandPending;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventSinkSpecific;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODPEventStart;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODSCBlockPredEnable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODeferCLUTSetAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODetailedTimingValid;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODigitalSignal;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAmbientLightSensorKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAssertionConnectType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAttributesKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAudioBalanceLRKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAudioBassKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAudioMuteAndScreenBlankKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAudioProcessorModeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayAudioTrebleKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayBlueGammaScaleKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayBrightnessFadeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayBrightnessKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayBrightnessProbeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayCapabilityStringKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayColorMode;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayConnectFlagsKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayContrastKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayControllerIDKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherAll;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherDefault;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherDisable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherFrameRateControl;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherRGBShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherSpatial;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherTemporal;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherYCbCr422Shift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayDitherYCbCr444Shift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayEDIDKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayEDIDOriginalKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayFirmwareLevelKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayGUIDKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayGammaScaleKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayGreenGammaScaleKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayHasBacklightKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayHorizontalPositionKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayHorizontalSizeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayIsDigitalKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayLinearBrightnessKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayLinearBrightnessProbeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayLocationKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayMCCSVersionKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayManufacturerSpecificKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayMatchingInfo;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayMaxValueKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayMicrophoneVolumeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayMinValueKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayNeedsCEAUnderscan;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayNoProductName;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayOnlyPreferredName;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayOverrideMatchingKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayOverscanKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayParallelogramKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayParametersCommitKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayParametersDefaultKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayParametersFlushKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayParametersKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayPincushionKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayPowerModeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayPowerStateKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayPowerStateMinUsable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayPowerStateOff;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayPowerStateOn;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRGBColorComponentBits10;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRGBColorComponentBits12;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRGBColorComponentBits14;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRGBColorComponentBits16;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRGBColorComponentBits6;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRGBColorComponentBits8;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRGBColorComponentBitsUnknown;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRedGammaScaleKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayRotationKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplaySelectedColorModeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplaySpeakerSelectKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplaySpeakerVolumeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplaySupportsBasicAudioKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplaySupportsUnderscanKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplaySupportsYCbCr422Key;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplaySupportsYCbCr444Key;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayTechnologyTypeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayTheatreModeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayTheatreModeWindowKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayTrapezoidKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayUsableLinearBrightnessKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayUsageTimeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayValueKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayVerticalPositionKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayVerticalSizeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayVideoBestKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr422ColorComponentBits10;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr422ColorComponentBits12;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr422ColorComponentBits14;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr422ColorComponentBits16;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr422ColorComponentBits6;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr422ColorComponentBits8;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr422ColorComponentBitsUnknown;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr444ColorComponentBits10;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr444ColorComponentBits12;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr444ColorComponentBits14;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr444ColorComponentBits16;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr444ColorComponentBits6;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr444ColorComponentBits8;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODisplayYCbCr444ColorComponentBitsUnknown;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODriverPowerAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODynamicRangeDolbyNormalMode;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODynamicRangeDolbyTunnelMode;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODynamicRangeHDR10;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODynamicRangeNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODynamicRangeSDR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODynamicRangeTraditionalGammaHDR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIODynamicRangeTraditionalGammaSDR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBAVSignalTypeDP;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBAVSignalTypeDVI;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBAVSignalTypeHDMI;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBAVSignalTypeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBAVSignalTypeUnknown;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBAVSignalTypeVGA;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBitRateHBR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBitRateHBR2;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBitRateRBR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBitsPerComponentKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBitsPerPixelKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBlueGammaScaleAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBytesPerPlaneKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBBytesPerRowKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBCLUTDeferKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBChangedInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBComponentCountKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBConfigKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBConnectInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBCurrentShmemVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBCursorHWCapable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBCursorImageNew;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBCursorInfoKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBCursorMemory;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDetailedTimingsKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayPortConfigurationDataKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayPortInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayPortLinkChangeInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayPortTrainingAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayState;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayState_AlreadyActive;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayState_Mask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayState_PipelineBlack;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBDisplayState_RestoredProfile;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBFlagsKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBFrameInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBGammaCountKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBGammaWidthKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBGreenGammaScaleAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHBLInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHDCPLimit_AllowAll;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHDCPLimit_NoHDCP1x;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHDCPLimit_NoHDCP20Type0;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHDCPLimit_NoHDCP20Type1;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHDMIDongleROMKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHDRMetaDataAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHardwareCursorActive;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHardwareCursorInVRAM;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHeightKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBHostAccessFlagsKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLimitHDCPAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLimitHDCPStateAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkDownspreadMax;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkDownspreadNone;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkPreEmphasisLevel0;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkPreEmphasisLevel1;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkPreEmphasisLevel2;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkPreEmphasisLevel3;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkScramblerAlternate;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkScramblerNormal;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkVoltageLevel0;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkVoltageLevel1;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkVoltageLevel2;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBLinkVoltageLevel3;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBMCCSInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBMaxCursorDepth;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBMaxCursorFrames;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBMaxCursorWidth;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBMemorySizeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBModeAIDKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBModeDFKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBModeDMKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBModeIDKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBModePIKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBModeTMKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBModesKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_Dim;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_DisplayStateMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_DisplayStateShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_Doze;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_GenerationMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_GenerationShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_MessageMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_Rendezvous;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_Sleep;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_UnDim;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNS_Wake;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNeedsRefreshKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNumCursorFrames;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBNumCursorFramesShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBOfflineInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBOnlineInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBProbeOptionsKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBRedGammaScaleAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBRefreshRateKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBScalerInfoKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBServerConnectType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBSetGammaSyncNoSync;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBSetGammaSyncNotSpecified;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBSetGammaSyncVerticalBlankSync;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBSharedConnectType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBShmemCursorNumFramesMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBShmemCursorNumFramesShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBShmemVersionMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBStop;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBSystemAperture;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBTenPtOneShmemVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBTenPtTwoShmemVersion;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBTimingRangeKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBUserRequestProbe;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBVBLInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBVariableRefreshRate;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBWakeInterruptType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFBWidthKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFixedCLUTPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOFramebufferInfoKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOGDiagnoseConnectType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOGDiagnoseGTraceType;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOHSyncDisable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOHardwareCursorAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOHibernateEFIGfxStatusKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOHibernatePreviewActive;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOHibernatePreviewActiveKey;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOHibernatePreviewUpdates;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOInterlacedCEATiming;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMaxPixelBits;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMirrorAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMirrorDefault;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMirrorDefaultAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMirrorForced;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMirrorHWClipped;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMirrorIsMirrored;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMirrorIsPrimary;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMonoDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMonoInverseDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOMultiAlignedTiming;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIONTSCTiming;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIONoSeparateSyncControl;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPALTiming;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPixelEncodingNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPixelEncodingRGB444;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPixelEncodingYCbCr420;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPixelEncodingYCbCr422;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPixelEncodingYCbCr444;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPowerAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOPowerStateAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORGBDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORGBSignedDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORGBSignedFloatingPointPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeBitsPerColorComponent10;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeBitsPerColorComponent12;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeBitsPerColorComponent16;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeBitsPerColorComponent6;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeBitsPerColorComponent8;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeBitsPerColorComponentNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryAdobeRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryBT2020;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryBT2100;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryBT601;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryBT709;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryDCIP3;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryNativeRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryWGRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetrysRGB;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeColorimetryxvYCC;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeDynamicRangeDolbyNormalMode;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeDynamicRangeDolbyTunnelMode;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeDynamicRangeHDR10;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeDynamicRangeNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeDynamicRangeSDR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeDynamicRangeTraditionalGammaHDR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeDynamicRangeTraditionalGammaSDR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangePixelEncodingNotSupported;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangePixelEncodingRGB444;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangePixelEncodingYCbCr420;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangePixelEncodingYCbCr422;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangePixelEncodingYCbCr444;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsCompositeSync;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsInterlacedCEATiming;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsInterlacedCEATimingWithConfirm;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsMultiAlignedTiming;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsSeparateSyncs;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsSignal_0700_0000;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsSignal_0700_0300;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsSignal_0714_0286;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsSignal_1000_0400;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsSyncOnGreen;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsVRR;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIORangeSupportsVSyncSerration;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleCanBorderInsetOnly;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleCanDownSamplePixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleCanRotate;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleCanScaleInterlaced;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleCanSupportInset;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleCanUpSamplePixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleInvertX;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleInvertY;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleRotate0;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleRotate180;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleRotate270;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleRotate90;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleRotateFlags;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleStretchOnly;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleStretchToFit;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScaleSwapAxes;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOScalingInfoValid;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOSyncOnBlue;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOSyncOnGreen;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOSyncOnRed;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOSyncPositivePolarity;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOSystemPowerAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDAppleNTSC_FF;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDAppleNTSC_FFconv;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDAppleNTSC_ST;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDAppleNTSC_STconv;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApplePAL_FF;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApplePAL_FFconv;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApplePAL_ST;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApplePAL_STconv;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_0x0_0hz_Offline;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_1024x768_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_1152x870_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_512x384_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_560x384_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_640x400_67hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_640x480_67hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_640x818_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_640x870_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_832x624_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDApple_FixedRateLCD;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDFilmRate_48hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDGTF_640x480_120hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDInvalid;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDSMPTE240M_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDSony_1600x1024_76hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDSony_1920x1080_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDSony_1920x1080_72hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDSony_1920x1200_76hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1024x768_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1024x768_70hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1024x768_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1024x768_85hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1152x864_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1280x1024_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1280x1024_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1280x1024_85hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1280x960_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1280x960_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1280x960_85hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1360x768_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1600x1200_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1600x1200_65hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1600x1200_70hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1600x1200_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1600x1200_80hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1600x1200_85hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1792x1344_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1792x1344_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1856x1392_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1856x1392_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1920x1440_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_1920x1440_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_640x480_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_640x480_72hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_640x480_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_640x480_85hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_800x600_56hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_800x600_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_800x600_72hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_800x600_75hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_800x600_85hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingIDVESA_848x480_60hz;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingRangeV1;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTimingRangeV2;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOTriStateSyncs;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOUYVY422PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOVRAMSaveAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOVSyncDisable;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_Accelerated;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_DeferEnd;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_DeferStart;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_DriverOpen;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_From_Accelerated;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_Hibernate;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_NonConsoleDevice;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_Reserved;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_Sleep;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_StateMask;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_To_Accelerated;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_Transactional;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWSAA_Unaccelerated;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOWindowServerActiveAttribute;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOYUV211PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOYUV411PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOYUVSPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOYUVUPixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOYVU9PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kIOYVYU422PixelFormat;
#[cfg(feature = "graphics")]
pub use self::__graphics::kInvertingEncodedPixel;
#[cfg(feature = "graphics")]
pub use self::__graphics::kInvertingEncoding;
#[cfg(feature = "graphics")]
pub use self::__graphics::kInvertingEncodingShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::kOrConnections;
#[cfg(feature = "graphics")]
pub use self::__graphics::kSetCLUTByValue;
#[cfg(feature = "graphics")]
pub use self::__graphics::kSetCLUTImmediately;
#[cfg(feature = "graphics")]
pub use self::__graphics::kSetCLUTWithLuminance;
#[cfg(feature = "graphics")]
pub use self::__graphics::kTransparentEncodedPixel;
#[cfg(feature = "graphics")]
pub use self::__graphics::kTransparentEncoding;
#[cfg(feature = "graphics")]
pub use self::__graphics::kTransparentEncodingShift;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO16BitDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO1BitIndexedPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO2BitIndexedPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO32BitDirectPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO4BitIndexedPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO8BitIndexedPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO8BitOverlayPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOAccelBounds;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOAccelDeviceRegion;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IOAccelFindAccelerator;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOAccelID;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOAccelSize;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IOAccelSurfaceInformation;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IOAccelSurfaceReadData;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOAccelSurfaceScaling;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOAppleTimingID;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitAccumulatePtr;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCompletionToken;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCopyRectangle;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCopyRectangleStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCopyRectangles;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCopyRectanglesStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCopyRegion;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCopyRegionStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCursor;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitCursorStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitMemory;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitMemoryRef;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitOperation;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitOperationStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitProcPtr;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitRectangle;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitRectangleStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitRectangles;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitRectanglesStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitScanlines;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitScanlinesStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitSourceDestType;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitSourceType;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitSurface;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitSurfaceStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitSurfaceStruct_memory;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitType;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitVertex;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitVertexStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitVertices;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitVerticesStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOBlitterPtr;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOColorComponent;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOColorEntry;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODetailedTimingInformation;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODetailedTimingInformationV1;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODetailedTimingInformationV2;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplayCommitParameters;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplayCopyFloatParameters;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplayCopyParameters;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplayCreateInfoDictionary;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplayForFramebuffer;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplayGetFloatParameter;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplayGetIntegerRangeParameter;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayMatchDictionaries;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayModeID;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayModeInformation;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayProductID;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayScalerInformation;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplaySetFloatParameter;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplaySetIntegerParameter;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IODisplaySetParameters;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayTimingRange;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayTimingRangeV1;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayTimingRangeV2;
#[cfg(feature = "graphics")]
pub use self::__graphics::IODisplayVendorID;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOFBDPLinkConfig;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOFBDisplayModeDescription;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOFBHDRMetaData;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOFBHDRMetaDataV1;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOFixed1616;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOFourCharCode;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOFramebufferInformation;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IOFramebufferOpen;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOGBounds;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOGPoint;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOGSize;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IOGraphicsAcceleratorInterface;
#[cfg(all(feature = "graphics", feature = "libc"))]
pub use self::__graphics::IOGraphicsAcceleratorInterfaceStruct;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOHardwareCursorDescriptor;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOHardwareCursorInfo;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOIndex;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOPagedPixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOPixelAperture;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOSelect;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOTimingInformation;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOTimingInformation_detailedInfo;
#[cfg(feature = "graphics")]
pub use self::__graphics::IOYUV422Pixels;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO_SampleTypeAlpha;
#[cfg(feature = "graphics")]
pub use self::__graphics::IO_SampleTypeSkip;
#[cfg(feature = "hid")]
pub use self::__hid::kFnFunctionUsageMapKey;
#[cfg(feature = "hid")]
pub use self::__hid::kFnKeyboardUsageMapKey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDAccelParametricCurvesKey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_AlphanumericDisplay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Arcade;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_BarCodeScanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_BatterySystem;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_BrailleDisplay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Button;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_CameraControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Consumer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Digitizer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_FIDO;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Game;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_GenericDesktop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_GenericDeviceControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Haptics;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_KeyboardOrKeypad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_LEDs;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_MagneticStripeReader;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Monitor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_MonitorEnumerated;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_MonitorReserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_MonitorVirtual;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Ordinal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_PID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_PowerDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_PowerReserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_PowerReserved2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Scale;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Sensor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Simulation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Sport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Telephony;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_Unicode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_VR;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_VendorDefinedStart;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPage_WeighingDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDPointerReportRateKey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDScrollAccelParametricCurvesKey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_ASCIICharacterSet;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_AlphanumericDisplay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CharacterHeight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CharacterReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CharacterSpacingHorizontal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CharacterSpacingVertical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CharacterWidth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_ClearDisplay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_Column;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_Columns;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CursorBlink;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CursorEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CursorMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CursorPixelPositioning;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_CursorPositionReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_DataReadBack;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_DisplayAttributesReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_DisplayControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_DisplayData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_DisplayEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_DisplayStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_ErrFontdatacannotberead;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_ErrNotaloadablecharacter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_FontData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_FontReadBack;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_FontReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_HorizontalScroll;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_Row;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_Rows;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_ScreenSaverDelay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_ScreenSaverEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_StatNotReady;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_StatReady;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_UnicodeCharacterSet;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_AD_VerticalScroll;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_2DControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ActiveTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_AddEAN2_3LabelDefinition;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_AimDuration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_AimingLaserPattern;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Aiming_PointerMide;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_AttributeReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_AztecCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BC412;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BadgeReader;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BarCodePresent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BarCodePresentSensor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BarCodeScanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BarCodeScannerCradle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BarSpaceData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BeeperState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_BooklandEAN;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ChannelCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Check;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitCodabarEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitCode99Enable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitDisable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitEnableInterleaved2of5OPCC;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitEnableInterleaved2of5USS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitEnableOneMSIPlessey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitEnableStandard2of5OPCC;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitEnableStandard2of5USS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDigitEnableTwoMSIPlessey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckDisablePrice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckEnable4DigitPrice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckEnable5DigitPrice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckEnableEuropean4DigitPrice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CheckEnableEuropean5DigitPrice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Class1ALaser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Class2Laser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ClearAllEAN2_3LabelDefinitions;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Codabar;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CodabarControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code128;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code128ControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code16;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code32;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code39;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code39ControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code49;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Code93;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CodeOne;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Colorcode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CommitParametersToNVM;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ConstantElectronicArticleSurveillance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ContactScanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ConvertEAN8To13Type;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ConvertUPCAToEAN_13;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ConvertUPC_EToA;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_CordlessScannerBase;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DLMethodCheckForDiscrete;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DLMethodCheckInRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DLMethodReadAny;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DataLengthMethod;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DataMatrix;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DataPrefix;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DecodeDataContinued;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DecodedData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DisableCheckDigitTransmit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_DumbBarCodeScanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN13FlagDigit1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN13FlagDigit2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN13FlagDigit3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN2_3LabelControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN8FlagDigit1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN8FlagDigit2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN8FlagDigit3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EANThreeLabel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EANTwoLabel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN_13;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN_8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN_99_128_Mandatory;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EAN_99_P5_128_Optional;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ElectronicArticleSurveillanceNotification;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_EnableCheckDigitTransmit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ErrorIndication;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_FirstDiscreteLengthToDecode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_FixedBeeper;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_FragmentDecoding;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_FullASCIIConversion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GRWTIAfterDecode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GRWTIBeep_LampAfterTransmit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GRWTINoBeep_LampUseAtAll;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodDecodeIndication;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodReadLED;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodReadLampDuration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodReadLampIntensity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodReadToneFrequency;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodReadToneLength;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodReadToneVolume;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_GoodReadWhenToWrite;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_HandsFreeScanning;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_HeaterPresent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_InitiateBarcodeRead;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Interleaved2of5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Interleaved2of5ControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_IntrinsicallySafe;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ItalianPharmacyCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_KlasseEinsLaser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_LaserOnTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_LaserState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_LockoutTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_LongRangeScanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MSIPlesseyControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MSI_Plessey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MaxiCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MaximumLengthToDecode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MicroPDF;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MinimumLengthToDecode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MirrorSpeedControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Misc1DControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MotorState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MotorTimeout;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_MultiRangeScanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_NoReadMessage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_NotOnFileIndication;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_NotOnFileVolume;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PDF_417;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ParameterScanning;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ParametersChanged;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Periodical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PeriodicalAutoDiscriminatePlus2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PeriodicalAutoDiscriminatePlus5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PeriodicalIgnorePlus2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PeriodicalIgnorePlus5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PeriodicalOnlyDecodeWithPlus2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PeriodicalOnlyDecodeWithPlus5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PolarityInvertedBarCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PolarityNormalBarCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PosiCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PowerOnResetScanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PowerupBeep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PrefixAIMI;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PrefixNone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PrefixProprietary;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_PreventReadOfBarcodes;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ProgrammableBeeper;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ProximitySensor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_QRCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_RawDataPolarity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_RawScannedDataReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ScannedDataReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ScannerDataAccuracy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ScannerInCradle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ScannerInRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_ScannerReadConfidence;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SecondDiscreteLengthToDecode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SetParameterDefaultValues;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SettingsReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SoundErrorBeep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SoundGoodReadBeep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SoundNotOnFileBeep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Standard2of5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Standard2of5ControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Standard2of5IATA;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_StatusReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SuperCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SymbologyIdentifier1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SymbologyIdentifier2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_SymbologyIdentifier3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TransmitCheckDigit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TransmitStart_Stop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriOptic;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriggerMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriggerModeBlinkingLaserOn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriggerModeContinuousLaserOn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriggerModeLaserOnWhilePulled;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriggerModeLaserStaysOnAfterTriggerRelease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriggerReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_TriggerState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Triggerless;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UCC_EAN_128;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_A;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_AWith128Mandatory;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_AWith128Optical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_AWithP5Optional;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_E;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_E1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_EAN;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_EANControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_EANCouponCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UPC_EANPeriodicals;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_USB_5_SlugCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_UltraCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_VeriCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_Wand;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BCS_WaterResistant;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_6DotBrailleCell;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_8DotBrailleCell;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleButtons;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleDPadCenter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleDPadDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleDPadLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleDPadRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleDPadUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleDisplay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleFaceControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleJoystickCenter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleJoystickDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleJoystickLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleJoystickRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleJoystickUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardDot8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardLeftSpace;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardRightSpace;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleKeyboardSpace;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleLeftControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BraillePanLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BraillePanRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleRightControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleRockerDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleRockerPress;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleRockerUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleRow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_BrailleTopControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_NumberOfBrailleCells;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_RouterKey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_RouterSet1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_RouterSet2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_RouterSet3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_RowRouterKey;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_ScreenReaderControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_ScreenReaderIdentifier;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BD_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ACPresent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AbsoluteStateOfCharge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AlarmInhibited;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AtRate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AtRateOK;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AtRateTimeToEmpty;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AtRateTimeToFull;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AverageCurrent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AverageTimeToEmpty;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_AverageTimeToFull;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_BattPackModelLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_BatteryInsertion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_BatteryPresent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_BatterySupported;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_BelowRemainingCapacityLimit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_BroadcastToCharger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_CapacityGranularity1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_CapacityGranularity2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_CapacityMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ChargeController;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ChargerConnection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ChargerSelectorSupport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ChargerSpec;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Charging;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ChargingIndicator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ConditioningFlag;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ConnectionToSMBus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_CurrentNotRegulated;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_CurrentOutOfRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_CycleCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_DesignCapacity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Discharging;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_EnablePolling;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_FullChargeCapacity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_FullyCharged;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_FullyDischarged;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_InhibitCharge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_InternalChargeController;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Level2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Level3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ManufacturerAccess;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ManufacturerData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ManufacturerDate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_MasterMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Maxerror;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_NeedReplacement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_OKToUse;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_OptionalMfgFunction1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_OptionalMfgFunction2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_OptionalMfgFunction3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_OptionalMfgFunction4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_OptionalMfgFunction5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_OutputConnection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_PowerFail;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_PrimaryBattery;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_PrimaryBatterySupport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Rechargable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_RelativeStateOfCharge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_RemainingCapacity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_RemainingCapacityLimit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_RemainingTimeLimit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_RemainingTimeLimitExpired;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ResetToZero;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_RunTimeToEmpty;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBAlarmWarning;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBBatteryMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBBatteryStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBChargerMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBChargerSpecInfo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBChargerStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBErrorCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBSelectorInfo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBSelectorPresets;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SMBSelectorState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SelectorRevision;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SerialNumber;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_SpecificationInfo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_TerminateCharge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_TerminateDischarge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ThermistorCold;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ThermistorHot;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ThermistorOverRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_ThermistorUnderRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_Usenext;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_VoltageNotRegulated;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_VoltageOutOfRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_WarningCapacityLimit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_iDeviceChemistry;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_iDevicename;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_iManufacturerName;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_BS_iOEMInformation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_10;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_100;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_101;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_102;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_103;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_104;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_105;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_106;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_107;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_108;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_109;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_11;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_110;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_111;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_112;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_113;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_114;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_115;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_116;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_117;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_118;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_119;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_12;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_120;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_121;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_122;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_123;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_124;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_125;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_126;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_127;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_128;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_129;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_13;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_130;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_131;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_132;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_133;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_134;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_135;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_136;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_137;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_138;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_139;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_14;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_140;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_141;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_142;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_143;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_144;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_145;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_146;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_147;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_148;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_149;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_15;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_150;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_151;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_152;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_153;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_154;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_155;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_156;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_157;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_158;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_159;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_16;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_160;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_161;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_162;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_163;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_164;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_165;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_166;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_167;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_168;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_169;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_17;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_170;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_171;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_172;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_173;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_174;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_175;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_176;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_177;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_178;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_179;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_18;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_180;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_181;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_182;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_183;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_184;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_185;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_186;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_187;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_188;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_189;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_19;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_190;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_191;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_192;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_193;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_194;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_195;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_196;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_197;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_198;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_199;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_20;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_200;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_201;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_202;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_203;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_204;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_205;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_206;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_207;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_208;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_209;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_21;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_210;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_211;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_212;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_213;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_214;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_215;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_216;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_217;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_218;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_219;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_22;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_220;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_221;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_222;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_223;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_224;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_225;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_226;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_227;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_228;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_229;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_23;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_230;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_231;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_232;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_233;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_234;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_235;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_236;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_237;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_238;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_239;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_24;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_240;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_241;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_242;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_243;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_244;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_245;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_246;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_247;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_248;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_249;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_25;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_250;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_251;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_252;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_253;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_254;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_255;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_26;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_27;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_28;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_29;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_30;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_31;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_32;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_33;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_34;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_35;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_36;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_37;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_38;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_39;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_40;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_41;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_42;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_43;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_44;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_45;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_46;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_47;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_48;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_49;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_50;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_51;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_52;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_53;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_54;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_55;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_56;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_57;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_58;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_59;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_60;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_61;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_62;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_63;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_64;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_65;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_65535;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_66;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_67;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_68;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_69;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_70;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_71;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_72;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_73;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_74;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_75;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_76;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_77;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_78;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_79;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_80;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_81;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_82;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_83;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_84;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_85;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_86;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_87;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_88;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_89;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_9;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_90;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_91;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_92;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_93;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_94;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_95;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_96;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_97;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_98;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Button_99;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_CC_Autofocus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_CC_Shutter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_CC_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_3DModeSelect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_AC;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACAddToCart;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACAllCaps;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACAttachComment;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACAttachFile;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACBack;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACBold;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACBookmarks;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACBulletedList;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACBuyOrCheckout;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACCancel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACCatalog;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACClearAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACClose;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACCollapse;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACCollapseAll;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACCopy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACCut;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDelete;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDemote;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDesktopShowAllApplications;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDesktopShowAllWindows;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDetachComment;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDistributeH;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDistributeV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACDownload;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACEdit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACEditTimeZones;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACExit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACExpand;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACExpandAll;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFilter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFind;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFindandReplace;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFlipHorizontal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFlipVertical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFontColor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFontSelect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFontSize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFormat;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACForward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACForwardMessage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACFullScreenView;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACGoTo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACHistory;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACHome;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACIdleKeepAlive;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACIndentyDecrease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACIndentyIncrease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACInsertColumn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACInsertFile;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACInsertMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACInsertObject;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACInsertPicture;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACInsertRow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACInsertSymbol;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACItalics;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyBlockH;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyBlockV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyBottom;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyCenterH;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyCenterV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACJustifyTop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACKeyboardLayoutSelect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACMaximize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACMerge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACMinimize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACMirrorHorizontal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACMirrorVertical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACNavigationGuidance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACNew;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACNewWindow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACNextLink;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACNo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACNormalView;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACNumberedList;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACOpen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPan;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPanLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPanRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPaste;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPasteSpecial;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPreviousLink;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPrint;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPrintPreview;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACPromote;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACProperties;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACProtect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACRedoOrRepeat;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACRefresh;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACRename;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACReply;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACReplyAll;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACResetAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACResize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACRestartNumbering;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACRotate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSave;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSaveAndClose;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACScroll;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACScrollDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACScrollUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSearch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectColumn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectObject;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectParagraph;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectRow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectSentence;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectTable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectTimeZone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSelectWord;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSend;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSendOrReceive;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSendTo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSetAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSetBorders;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSetClock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSnoozeAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSoftKeyLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSoftKeyRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSort;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSortAscending;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSortDescending;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSplit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACStop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACStrikethrough;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSubscript;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSubscriptions;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSuperscript;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACSynchronize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACTileHorizontally;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACTileVertically;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACUnderline;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACUndo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACUnlock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACUnprotect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACUpload;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACViewClock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACViewComment;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACViewToggle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACYes;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACZoom;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACZoomIn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ACZoomOut;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_AL;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALAOrVCaptureOrPlayback;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALAlarms;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALAudioBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALAudioPlayer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALCalculator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALCalendarOrSchedule;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALCheckbookOrFinance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALClock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALCommandLineProcessorOrRun;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALConsumerControlConfiguration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALContactSync;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALContactsOrAddressBook;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALContextawareDesktopAssistant;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALControlPanel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALCustomizedCorporateNewsBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALDatabaseApp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALDesktop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALDictionary;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALDigitalRightsManager;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALDigitalWallet;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALDocuments;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALEmailReader;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALEncryption;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALEntertainmentContentBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALFileBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALGrammerCheck;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALGraphicsEditor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALImageBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALInstantMessaging;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALIntegratedHelpCenter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALInternetBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALKeyboardLayout;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALLANOrWANBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALLaunchButtonConfigurationTool;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALLocalMachineBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALLogOrJournalOrTimecard;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALLogoff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALLogon;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALLogonOrLogoff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALMarketMonitorOrFinanceBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALMessageStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALMovieBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALNavigation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALNetworkChat;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALNetworkConference;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALNewsreader;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALNextTaskOrApplication;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALOEMFeatureBrowser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALOEMHelp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALOnlineActivityBrowswer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALOnlineCommunity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALOnlineShoppingBrowswer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALPowerStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALPreemptiveHaltTaskOrApplication;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALPresentationApp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALPreviousTaskOrApplication;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALProcessOrTaskManager;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALProgrammableButtonConfiguration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALRemoteNetworkingOrISPConnect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALResearchOrSearchBrowswer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALScreenSaver;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALSmartCardInformationOrHelp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALSpellCheck;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALSpreadsheet;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALTaskOrProjectManager;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALTelephonyOrDialer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALTerminalLockOrScreensaver;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALTextEditor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALThesaurus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALVirusProtection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALVoicemail;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALWirelessStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ALWordProcessor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_AMOrPM;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_AlternateAudioDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_AlternateAudioIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ApplicationLaunchButtons;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Aspect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Assign;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Balance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_BalanceLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_BalanceRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Bass;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_BassBoost;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_BassDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_BassIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_BlueMenuButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_BroadcastMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Channel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelCenter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelCenterFront;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelFront;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelLowFrequencyEnhancement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelSide;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelSurround;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelTop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ChannelUnknown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ClearMark;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ClimateControlEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ClosedCaption;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ClosedCaptionSelect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ConsumerControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactAdded;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactEdited;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactEmailBusiness;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactEmailMain;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactEmailOther;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactEmailPersonal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactFirstName;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactFullName;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactIndex;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactLastName;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactMisc;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactNickname;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactPhoneNumberBusiness;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactPhoneNumberFax;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactPhoneNumberMobile;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactPhoneNumberOther;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactPhoneNumberPager;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactPhoneNumberPersonal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactRecordActive;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactSpeedDialNumber;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ContactStatusFlag;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_CounterReset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Daily;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DataOnScreen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DisplayBacklightToggle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DisplayBrightness;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DisplayBrightnessDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DisplayBrightnessIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DisplayBrightnessMaximum;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DisplayBrightnessMinimum;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DisplayBrightnessSetAutoBrightness;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_DuressAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Eject;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_EnterChannel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_EnterDisc;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ExtendedKeyboardAttributesCollection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ExtendedPlay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_FanEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_FanSpeed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_FastForward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_FireAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_FrameBack;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_FrameForward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_FunctionButtons;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_GenericGUIApplicationControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_GraphicEqualizer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_GreenMenuButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Headphone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Help;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_HoldupAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Illumination;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ImplementedKeyboardInputAssistControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardBrightnessDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardBrightnessIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardFormFactor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardIETFLanguageTagIndex;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardInputAssistAccept;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardInputAssistCancel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardInputAssistNext;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardInputAssistNextGroup;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardInputAssistPrevious;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardInputAssistPreviousGroup;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardKeyType;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_KeyboardPhysicalLayout;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_LightEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_LightIlluminationLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_LongPlay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Loudness;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MPX;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Mark;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Media;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectCD;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectCable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectCall;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectComputer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectDVD;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectGames;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectHome;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectMessages;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectProgramGuide;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectSatellite;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectSecurity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectTV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectTape;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectTelephone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectTuner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectVCR;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectVideoPhone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelectWWW;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MediaSelection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MedicalAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Menu;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuEscape;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuPick;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuValueDecrease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_MenuValueIncrease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Microphone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ModeStep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Monthly;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Motion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Mute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_NumericKeyPad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Once;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_OrderMovie;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Pause;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_PictureInPictureSwap;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_PictureInPictureToggle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Play;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_PlayOrPause;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_PlayOrSkip;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_PlaybackSpeed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Plus10;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Plus100;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_PoliceAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Power;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ProgrammableButtons;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Proximity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Quit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_RandomPlay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_RecallLast;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Record;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_RedMenuButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Repeat;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_RepeatFromMark;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Reset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ReturnToMark;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Rewind;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_RoomTemperature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ScanNextTrack;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ScanPreviousTrack;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SearchMarkBackwards;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SearchMarkForward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SecurityEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SelectDisc;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Selection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_ShowCounter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Sleep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SleepAfter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SleepMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Slow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SlowTracking;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Snapshot;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SpeakerSystem;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Speed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_StandardPlay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Still;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Stop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_StopOrEject;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SubChannel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SubChannelDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SubChannelIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_SurroundMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_TrackNormal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Tracking;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_TrackingDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_TrackingIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Treble;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_TrebleDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_TrebleIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_VCROrTV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_VCRPlus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_VendorSpecificKeyboardPhysicalLayout;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_VoiceCommand;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Volume;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_VolumeDecrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_VolumeIncrement;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_Weekly;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Csmr_YellowMenuButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_3DDigitizer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Altitude;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Armature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_ArticulatedArm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Azimuth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_BarrelPressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_BarrelSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_BatteryStrength;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_CapacitiveHeatMapDigitizer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_CapacitiveHeatMapFrameData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_CapacitiveHeatMapProtocolVendorID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_CapacitiveHeatMapProtocolVersion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_ContactCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_ContactCountMaximum;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_ContactIdentifier;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_CoordinateMeasuringMachine;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_DataValid;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_DeviceConfiguration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_DeviceIdentifier;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_DeviceMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_DeviceSettings;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Digitizer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Eraser;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Finger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_FreeSpaceWand;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterDataLength;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterEncoding;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterEncodingUTF16BE;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterEncodingUTF16LE;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterEncodingUTF32BE;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterEncodingUTF32LE;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterEncodingUTF8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_GestureCharacterQuality;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Height;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_InRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Invert;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_LightPen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_MultiplePointDigitizer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Pen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_ProgramChangeKeys;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Puck;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Quality;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_RelativeScanTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_ReportRate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_SecondaryTipSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_StereoPlotter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Stylus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_SurfaceSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TabletFunctionKeys;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TabletPick;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Tap;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TipPressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TipSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Touch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TouchPad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TouchScreen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TouchValid;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_TransducerIndex;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Twist;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Untouch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_WhiteBoard;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_Width;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_XTilt;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Dig_YTilt;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_FIDO_InputData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_FIDO_OutputData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_FIDO_U2FDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_FIDO_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_ApplicationBreak;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_ApplicationDebuggerBreak;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_AssistiveControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_AssistiveControlCompatible;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_ByteCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_CallActiveLED;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_CallMuteLED;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_CallMuteToggle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_ChassisEnclosure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_ControlEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_CoolantCriticalLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_CoolantLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_CoolantPump;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_CountedBuffer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DPadDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DPadLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DPadRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DPadUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Dial;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DoNotDisturb;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DockableDeviceDisplayOcclusion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DockableDeviceDockingState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DockableDeviceObjectType;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DockableDevicePrimaryUsageID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DockableDevicePrimaryUsagePage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DockableDeviceUniqueID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_DockableDeviceVendorID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_FeatureNotification;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_GamePad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Hatswitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_IndexTrigger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Joystick;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Keyboard;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Keypad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_MotionWakeup;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Mouse;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_MultiAxisController;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_PalmTrigger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Pointer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Qw;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Qx;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Qy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Qz;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_RPM;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_ResolutionMultiplier;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Rx;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Ry;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Rz;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SFShift;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SFShiftLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SFShiftLockIndicator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Select;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SensorZone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Slider;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SpatialController;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Start;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemAppMenu;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemBreak;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemColdRestart;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemContextMenu;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDebuggerBreak;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDismissNotification;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayBoth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayDual;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayExternal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayInternal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayInvert;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayRotationLockButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayRotationLockSliderSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplaySwap;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayToggleLCDAutoscale;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDisplayToggleMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemDock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemHibernate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMainMenu;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenu;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenuDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenuExit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenuHelp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenuLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenuRight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenuSelect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMenuUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMicrophoneMute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemMultiAxisController;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemPowerDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemSetup;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemSleep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemSpeakerMute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemUndock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemWakeUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_SystemWarmRestart;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_TabletPCSystemControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Thumbstick;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Vbrx;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Vbry;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Vbrz;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Vno;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Vx;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Vy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Vz;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Wheel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_WirelessRadioButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_WirelessRadioLED;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_WirelessRadioSliderSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_X;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Y;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GD_Z;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_3DGameController;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_Bump;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_Flipper;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GamepadFireOrJump;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GamepadFormFitting;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GamepadFormFitting_Compatibility;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GamepadTrigger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_Gun;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GunAutomatic;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GunBolt;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GunBurst;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GunClip;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GunDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GunSafety;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_GunSingleShot;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_HeightOfPOV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_LeanForwardOrBackward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_LeanRightOrLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_MoveForwardOrBackward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_MoveRightOrLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_MoveUpOrDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_NewGame;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_PinballDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_PitchUpOrDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_Player;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_PointofView;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_RollRightOrLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_SecondaryFlipper;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_ShootBall;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Game_TurnRightOrLeft;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GenDevControls_BackgroundControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_GenDevControls_BatteryStrength;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_AutoTrigger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_AutoTriggerAssociatedControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_DurationList;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_Intensity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_ManualTrigger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_RepeatCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_RetriggerPeriod;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_SimpleHapticController;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_VendorWaveformFirst;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_VendorWaveformLast;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformBuzzContinuous;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformClick;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformCutoffTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformList;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformNone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformPress;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformRelease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformRumbleContinuous;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformStop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformVendorID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Haptics_WaveformVendorPage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard0;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard9;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardA;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardAgain;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardAlternateErase;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardApplication;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardB;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardBackslash;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardC;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardCancel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardCapsLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardClear;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardClearOrAgain;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardCloseBracket;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardComma;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardCopy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardCrSelOrProps;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardCut;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardD;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardDeleteForward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardDeleteOrBackspace;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardDownArrow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardE;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardEnd;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardEqualSign;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardErrorRollOver;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardErrorUndefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardEscape;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardExSel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardExecute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF10;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF11;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF12;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF13;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF14;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF15;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF16;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF17;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF18;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF19;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF20;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF21;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF22;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF23;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF24;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardF9;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardFind;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardG;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardGraveAccentAndTilde;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardH;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardHelp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardHome;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardHyphen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardI;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInsert;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardInternational9;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardJ;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardK;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardL;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLANG9;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLeftAlt;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLeftArrow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLeftControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLeftGUI;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLeftShift;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLockingCapsLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLockingNumLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardLockingScrollLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardM;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardMenu;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardMute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardN;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardNonUSBackslash;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardNonUSPound;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardO;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardOpenBracket;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardOper;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardOut;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardP;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPOSTFail;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPageDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPageUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPaste;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPause;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPeriod;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPrintScreen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardPrior;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardQ;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardQuote;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardR;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardReturn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardReturnOrEnter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardRightAlt;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardRightArrow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardRightControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardRightGUI;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardRightShift;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardScrollLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardSelect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardSemicolon;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardSeparator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardSlash;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardSpacebar;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardStop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardSysReqOrAttention;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardT;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardTab;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardU;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardUndo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardUpArrow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardVolumeDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardVolumeUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardW;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardX;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardY;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeyboardZ;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keyboard_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad0;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Keypad9;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadAsterisk;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadComma;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadEnter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadEqualSign;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadEqualSignAS400;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadHyphen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadNumLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadPeriod;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadPlus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_KeypadSlash;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_BatteryLow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_BatteryOK;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_BatteryOperation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_BlueLEDChannel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Busy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_CAV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_CLV;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_CallPickup;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_CameraOff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_CameraOn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_CapsLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Compose;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Conference;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Coverage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_DataMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_DoNotDisturb;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_EqualizerEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Error;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_ExternalPowerConnected;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_FastBlinkOffTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_FastBlinkOnTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_FastForward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_FlashOnTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Forward;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_GenericIndicator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_GoodStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_GreenLEDChannel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_HeadSet;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_HighCutFilter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Hold;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorAmber;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorBlue;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorFastBlink;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorFlash;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorGreen;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorOff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorOn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorOrange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorRed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_IndicatorSlowBlink;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Kana;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_LEDIntensity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_LowCutFilter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_MessageWaiting;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Microphone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Mute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_NightMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_NumLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_OffHook;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_OffLine;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_OnLine;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_PaperJam;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_PaperOut;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Pause;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Play;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Player8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_PlayerIndicator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Power;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_RGB_LED;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Ready;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Record;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_RecordingFormatDetect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_RedLEDChannel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Remote;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Repeat;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Reverse;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Rewind;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Ring;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SamplingRateDetect;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_ScrollLock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SendCalls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Shift;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SlowBlinkOffTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SlowBlinkOnTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SoundFieldOn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Speaker;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Spinning;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_StandBy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Stereo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Stop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SurroundOn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SystemMicrophoneMute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_SystemSuspend;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_ToneEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_Usage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_UsageInUseIndicator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_UsageIndicatorColor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_UsageMultiModeIndicator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_LED_WarningStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_DeviceReadOnly;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_Track1Data;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_Track1Length;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_Track2Data;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_Track2Length;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_Track3Data;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_Track3Length;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_TrackData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_TrackJISData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_TrackJISLength;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_MSR_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Ord_Instance1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Ord_Instance2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Ord_Instance3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Ord_Instance4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Ord_Instance65535;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ActivePower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ApparentPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_AudibleAlarmControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_AwaitingPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_BadCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Battery;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_BatteryID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_BatterySystem;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_BatterySystemID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Boost;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Buck;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ChangedStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Charger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ChargerID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_CommunicationLost;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigActivePower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigApparentPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigCurrent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigFrequency;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigHumidity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigPercentLoad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigTemperature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ConfigVoltage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Current;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_DelayBeforeReboot;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_DelayBeforeShutdown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_DelayBeforeStartup;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Flow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_FlowID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Frequency;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_FrequencyOutOfRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Gang;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_GangID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Good;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_HighVoltageTransfer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Humidity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Initialized;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Input;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_InputID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_InternalFailure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_LowVoltageTransfer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ModuleReset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Outlet;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_OutletID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_OutletSystem;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_OutletSystemID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Output;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_OutputID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_OverCharged;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_OverTemperature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Overload;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PercentLoad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PeripheralDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PowerConverter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PowerConverterID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PowerSummary;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PowerSummaryID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PowerSupply;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Present;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_PresentStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ShutdownImminent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ShutdownRequested;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_SwitchOffControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_SwitchOnControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_SwitchOnOff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Switchable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Temperature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Test;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Tested;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_ToggleControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_UPS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Used;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_Voltage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_VoltageOutOfRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_iManufacturer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_iName;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_iProduct;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PD_iserialNumber;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ActuatorOverrideSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ActuatorPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ActuatorsEnabled;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_AttackLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_AttackTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_AxesEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockFreeReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockHandle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockLoadError;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockLoadFull;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockLoadReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockLoadStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockLoadSuccess;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_BlockType;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_CP_Offset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_CreateNewEffectReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_CustomForceData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_CustomForceDataOffset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_CustomForceDataReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_CustomForceVendorDefinedData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DC_DeviceContinue;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DC_DevicePause;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DC_DeviceReset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DC_DisableActuators;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DC_EnableActuators;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DC_StopAllEffects;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DeadBand;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DeviceControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DeviceControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DeviceGain;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DeviceGainReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DeviceManagedPool;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DevicePaused;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Direction;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DirectionEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_DownloadForceSample;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Duration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_ConstantForce;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_CustomForceData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Damper;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Friction;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Inertia;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Ramp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_SawtoothDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_SawtoothUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Sine;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Spring;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Square;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ET_Triangle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_EffectBlockIndex;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_EffectOperation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_EffectOperationReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_EffectPlaying;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_EffectType;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_FadeLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_FadeTime;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Gain;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_IsochCustomForceEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_LoopCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Magnitude;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_MoveDestination;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_MoveLength;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_MoveSource;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_NegativeCoefficient;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_NegativeSaturation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Normal;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Offset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_OpEffectStart;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_OpEffectStartSolo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_OpEffectStop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ParamBlockOffset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ParameterBlockSize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Period;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Phase;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_PhysicalInterfaceDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_PoolAlignment;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_PoolMoveReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_PoolReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_PositiveCoefficient;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_PositiveSaturation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_RAM_PoolAvailable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_RAM_PoolSize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ROM_EffectBlockCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ROM_Flag;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_ROM_PoolSize;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_RampEnd;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_RampStart;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SafetySwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SampleCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SamplePeriod;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SetConditionReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SetConstantForceReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SetCustomForceReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SetEffectReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SetEnvelopeReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SetPeriodicReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SetRampForceReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SharedParameterBlocks;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_SimultaneousEffectsMax;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_StartDelay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_StateReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_TriggerButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_TriggerRepeatInterval;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_TypeSpecificBlockHandle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_PID_TypeSpecificBlockOffset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Accelerator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Aileron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_AileronTrim;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_AirplaneSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_AntiTorqueControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_AutomobileSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_AutopilotEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Ballast;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_BarrelElevation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_BicycleCrank;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_BicycleSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Brake;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_ChaffRelease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Clutch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_CollectiveControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_CyclicControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_CyclicTrim;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_DiveBrake;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_DivePlane;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_ElectronicCountermeasures;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Elevator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_ElevatorTrim;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_FlareRelease;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_FlightCommunications;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_FlightControlStick;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_FlightSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_FlightStick;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_FlightYoke;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_FrontBrake;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_HandleBars;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_HelicopterSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_LandingGear;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_MagicCarpetSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_MotorcycleSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_RearBrake;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Rudder;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_SailingSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Shifter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_SpaceshipSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_SportsSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Steering;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_SubmarineSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_TankSimulationDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Throttle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_ToeBrake;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_TrackControl;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Trigger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_TurretDirection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_Weapons;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_WeaponsArm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sim_WingFlaps;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Biometric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Biometric_HeartRate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Biometric_HumanPresence;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Biometric_HumanProximity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Biometric_HumanTouch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Biometric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Biometric_HeartRate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Biometric_HumanPresence;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Biometric_HumanProximityOutOfRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Biometric_HumanProximityRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Biometric_HumanTouchState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_BooleanArray;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Usage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Value;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Value1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Value2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Value3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Value4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Value5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Custom_Value6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_Capacitance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_Current;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_ElectricalPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_Frequency;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_Inductance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_PercentOfRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_Period;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_Resistance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Electrical_Voltage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Environmental;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Environmental_AtmosphericPressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Environmental_RelativeHumidity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Environmental_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Environmental_Temperature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Environmental_WindDirection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Environmental_WindSpeed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Light;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Light_Chromaticity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Light_ChromaticityX;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Light_ChromaticityY;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Light_ColorTemperature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Light_ConsumerIRSentenceReceive;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Light_Illuminance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_AddressLine1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_AddressLine2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_AltitudeAntennaSeaLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_AltitudeEllipsoid;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_AltitudeEllipsoidError;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_AltitudeSeaLevel;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_AltitudeSeaLevelError;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_City;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_CountryOrRegion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_DifferentialGPSDataAge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_DifferentialReferenceStationID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_ErrorRadius;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixQuality;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixQualityDGPS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixQualityGPS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixQualityNoFix;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixType;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeDGPSSPSMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeEstimated;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeFloatRTK;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeGPSPPSMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeGPSSPSMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeManualInputMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeNoFix;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeRealTimeKinematic;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_FixTypeSimulatorMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSOperationMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSOperationModeAutomatic;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSOperationModeManual;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSSelectionMode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSSelectionModeAutonomous;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSSelectionModeDGPS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSSelectionModeDataNotValid;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSSelectionModeEstimated;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSSelectionModeManualInput;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSSelectionModeSimulator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSStatusDataNotValid;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GPSStatusDataValid;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_GeoidalSeparation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_HorizontalDilutionOfPrecision;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_Latitude;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_Longitude;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_MagneticHeading;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_MagneticVariation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_NMEASentence;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_PositionDilutionOfPrecision;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_PostalCode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesInView;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesInViewAzimuth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesInViewElevation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesInViewIDs;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesInViewPRNs;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesInViewSNRatios;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesUsedCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_SatellitesUsedPRNs;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_Speed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_StateOrProvince;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_TrueHeading;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Location_VerticalDilutionOfPrecision;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_AbsolutePressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_BooleanSwitchArrayStates;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_BooleanSwitchState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_Force;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_GaugePressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_MultivalueSwitchValue;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_Strain;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Mechanical_Weight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_Acceleration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AccelerationAxisX;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AccelerationAxisY;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AccelerationAxisZ;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularPosition;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularPositionXAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularPositionYAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularPositionZAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularVelocity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularVelocityXAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularVelocityYAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_AngularVelocityZAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_Intensity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_Speed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Motion_State;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_Distance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_DistanceOutOfRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_DistanceXAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_DistanceYAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_DistanceZAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_Heading;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_HeadingCompensatedMagneticNorth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_HeadingCompensatedTrueNorth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_HeadingMagneticNorth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_HeadingTrueNorth;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_HeadingXAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_HeadingYAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_HeadingZAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_MagneticFlux;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_MagneticFluxXAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_MagneticFluxYAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_MagneticFluxZAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_Quaternion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_RotationMatrix;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_Tilt;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_TiltXAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_TiltYAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Orientation_TiltZAxis;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Scanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Scanner_NFCSentenceReceive;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Scanner_RFIDTag40Bit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Day;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeek;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeekFriday;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeekMonday;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeekSaturday;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeekSunday;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeekThursday;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeekTuesday;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_DayOfWeekWednesday;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Hour;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_JulianDayOfYear;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Millisecond;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Minute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Month;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Second;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Timestamp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Data_Time_Year;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Capacitance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Current;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Frequency;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Inductance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Period;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Potentiometer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Power;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Resistance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Electrical_Voltage;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Environmental;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Environmental_AtmosphericPressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Environmental_Humidity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Environmental_Temperature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Environmental_WindDirection;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Environmental_WindSpeed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_ChangeSensitivity;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_ComplexTrigger;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_DataUpdated;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_FrequencyExceeded;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_HighThresholdCrossDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_HighThresholdCrossUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_LowThresholdCrossDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_LowThresholdCrossUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_PeriodExceeded;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_PollResponse;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_PropertyChanged;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_RangeMaxReached;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_RangeMinReached;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_StateChanged;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_Unknown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_ZeroThresholdCrossDown;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorEvent_ZeroThresholdCrossUp;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState_AccessDenied;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState_Error;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState_Initializing;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState_NoData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState_NotAvailable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState_Ready;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Event_SensorState_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Light;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Light_AmbientLight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Light_ConsumerInfrared;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Light_Illuminance;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location_Broadcast;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location_DeadReckoning;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location_GPS;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location_Lookup;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location_Other;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location_Static;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Location_Triangulation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_BooleanSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_BooleanSwitchArray;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_Force;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_HallEffectSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_HapticVibrator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_MultivalueSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_Pressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_Strain;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Mechanical_Weight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_Accuracy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_CalibrationMultiplier;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_CalibrationOffset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_ChangeSensitivityAbsolute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_ChangeSensitivityPercentRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_ChangeSensitivityPercentRelative;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_FrequencyMax;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_Max;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_Min;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_None;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_PeriodMax;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_ReportInterval;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_Resolution;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_ThresholdHigh;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_ThresholdLow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Modifier_VendorDefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Accelerometer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Accelerometer1D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Accelerometer2D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Accelerometer3D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_GravityVector;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Gyrometer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Gyrometer1D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Gyrometer2D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Gyrometer3D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_LinearAccelerometer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_MotionDetector;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Motion_Speedometer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Compass1D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Compass2D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Compass3D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_CompassD;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_DeviceOrientation;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Distance1D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Distance2D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Distance3D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_DistanceD;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Inclinometer1D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Inclinometer2D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_Inclinometer3D;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Orientation_InclinometerD;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Other;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Other_Custom;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Other_Generic;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Other_GenericEnumerator;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Accuracy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ChangeSensitivityAbsolute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ChangeSensitivityPercentRange;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ChangeSensitivityPercentRelative;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ConnectionType;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ConnectionType_Attached;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ConnectionType_External;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ConnectionType_Integrated;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Description;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_DevicePath;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Environmental;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Environmental_ReferencePressure;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_FirmwareVersion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_FriendlyName;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_HardwareRevision;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Light;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Light_ConsumerIRSentenceSend;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Location;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Location_AccuracyDefault;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Location_AccuracyHigh;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Location_AccuracyLow;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Location_AccuracyMedium;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Location_DesiredAccuracy;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Manufacturer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_MaxFIFOEvents;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Maximum;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Mechanical;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Mechanical_BackwardVibrationSpeed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Mechanical_ForwardVibrationSpeed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Mechanical_VibrationState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Minimum;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_MinimumReportInterval;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Model;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PersistentUniqueID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PowerState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PowerState_D0_FullPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PowerState_D1_LowPower;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PowerState_D2_Standby;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PowerState_D3_Sleep;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PowerState_D4_PowerOff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_PowerState_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReleaseData;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportInterval;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportLatency;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportingState;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportingState_AllEvents;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportingState_NoEvents;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportingState_ThresholdEvents;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportingState_WakeAllEvents;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportingState_WakeNoEvents;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ReportingState_WakeThresholdEvents;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Resolution;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_ResponseCurve;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_SamplingRate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Scanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Scanner_NFCSentenceSend;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_SensorStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_SerialNumber;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Time;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Time_ArmAlarm;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Time_DaylightSavingsTimeObserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Time_TimeTrimAdjustment;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Time_TimeZoneName;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Property_Time_TimeZoneOffsetFromUTC;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Scanner;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Scanner_Barcode;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Scanner_NFC;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Scanner_RFID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Sensor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Time;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Time_AlarmTimer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Time_RealTimeClock;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Snsr_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_10Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_11Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_1Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_1Wood;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_2Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_3Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_3Wood;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_4Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_5Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_5Wood;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_6Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_7Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_7Wood;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_8Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_9Iron;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_9Wood;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_BaseballBat;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_GolfClub;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_LoftWedge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_Oar;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_PowerWedge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_Putter;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_Rate;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_RowingMachine;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_SandWedge;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_Slope;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_StickFaceAngle;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_StickFollowThrough;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_StickHeelOrToe;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_StickHeight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_StickSpeed;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_StickTempo;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_StickType;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Sprt_Treadmill;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_TFon_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_AlternateFunction;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_AnswerOnOrOff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_AnsweringMachine;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_CallWaitingTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_CallerID;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Conference;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_ConfirmationTone1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_ConfirmationTone2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_DoNotDisturb;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Drop;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Feature;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Flash;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_ForwardCalls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Handset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Headset;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Hold;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_HookSwitch;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_InsideDialTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_InsideRingTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_InsideRingback;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Line;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_LineBusyTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Message;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_MessageControls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_OutsideDialTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_OutsideRingTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_OutsideRingback;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Park;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Phone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneDirectory;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey0;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey1;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey2;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey3;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey4;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey5;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey6;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey7;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey8;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKey9;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKeyA;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKeyB;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKeyC;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKeyD;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKeyPound;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneKeyStar;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PhoneMute;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PriorityRingTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_PriorityRingback;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_ProgrammableButton;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_RecallNumber;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Redial;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_ReorderTone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Ring;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_RingEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_ScreenCalls;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_SpeakerPhone;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_SpeedDial;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_StoreNumber;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_TelephonyKeyPad;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_TonesOff;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_Transfer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Tfon_VoiceMail;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_AnimatronicDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_Belt;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_BodySuit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_DisplayEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_Flexor;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_Glove;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_HandTracker;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_HeadMountedDisplay;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_HeadTracker;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_Oculometer;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_Reserved;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_StereoEnable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_VR_Vest;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_CalibrationCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_DataScaling;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_DataWeight;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_EnforcedZeroReturn;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_RezeroCount;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleAtrributeReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleControlReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleDataReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassGeneric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIIIEnglish;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIIILEnglish;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIIILMetric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIIIMetric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIIMetric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIMetric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIMetricCL;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIVEnglish;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleClassIVMetric;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleScaleDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatisticsReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatus;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusFault;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusInMotion;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusOverWeightLimit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusRequiresCalibration;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusRequiresRezeroing;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusStableAtZero;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusUnderZero;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleStatusWeightStable;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ScaleWeightLimitReport;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_Undefined;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeighingDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnit;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitAvoirTon;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitCarats;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitGrains;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitGram;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitKilogram;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitMetricTon;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitMilligram;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitOunce;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitPennyweights;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitPound;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitTaels;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_WeightUnitTroyOunce;
#[cfg(feature = "hid")]
pub use self::__hid::kHIDUsage_WD_ZeroScale;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDAccelerationAlgorithmTypeDefault;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDAccelerationAlgorithmTypeParametric;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDAccelerationAlgorithmTypeTable;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDAltHandlerIdKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDBatchIntervalKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDBuiltInKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDCategoryAutomotiveValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDCategoryKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDCountryCodeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceAccessEntitlementKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceDefaultAsyncRequestTimeout;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceMaxAsyncRequestTimeout;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceMinAsyncRequestTimeout;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceOpenedByEventSystemKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceSuspendKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceUsageKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceUsagePageKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDeviceUsagePairsKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDigitizerGestureCharacterStateKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDigitizerSurfaceSwitchKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDigitizerTipThresholdKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDisplayIntegratedKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDDropAccelPropertyEventsKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCalibrationDeadZoneMaxKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCalibrationDeadZoneMinKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCalibrationGranularityKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCalibrationMaxKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCalibrationMinKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCalibrationSaturationMaxKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCalibrationSaturationMinKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCollectionTypeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCookieKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCookieMaxKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementCookieMinKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementDuplicateIndexKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsBufferedByteMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsConstantMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsNoPreferredMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsNonLinearMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsNullStateMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsRelativeMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsVariableMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsVolativeMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementFlagsWrapMask;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementHasNullStateKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementHasPreferredStateKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementIsArrayKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementIsNonLinearKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementIsRelativeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementIsWrappingKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementMaxKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementMinKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementNameKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementParentCollectionKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementReportCountKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementReportIDKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementReportSizeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementScaledMaxKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementScaledMinKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementSizeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementTypeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementUnitExponentKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementUnitKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementUsageKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementUsageMaxKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementUsageMinKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementUsagePageKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementValueLocationKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementVariableSizeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDElementVendorSpecificKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDEventDriverHandlesReport;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDEventServiceSensorControlOptionsKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDHeightKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDIdleTimeMicrosecondsKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardCapsLockDelay;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardCapsLockDelayOverride;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardCapsLockDelayOverrideKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardEjectDelay;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardFunctionKeyCountKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardLanguageKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardLayoutValueKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardPhysicalLayoutType101;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardPhysicalLayoutType102;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardPhysicalLayoutType103;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardPhysicalLayoutType104;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardPhysicalLayoutType106;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardPhysicalLayoutTypeUnknown;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardPhysicalLayoutTypeVendor;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardSupportsDoNotDisturbKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDKeyboardSupportsEscKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDLocationIDKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDManufacturerKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMaxFeatureReportSizeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMaxInputReportSizeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMaxOutputReportSizeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMaxReportBufferCountKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMaxResponseLatencyKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDModelNumberKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMouseAccelerationType;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMouseAccelerationTypeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDMouseScrollAccelerationKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDOptionsTypeMaskPrivate;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDOptionsTypeNone;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDOptionsTypeSeizeDevice;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPhysicalDeviceUniqueIDKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerAccelerationAlgorithmKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerAccelerationKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerAccelerationMinimumKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerAccelerationMultiplierKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerAccelerationSupportKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerAccelerationTypeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerButtonMode;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPointerButtonModeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPowerOnDelayNSKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPrimaryTrackpadCanBeDisabledKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPrimaryUsageKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDPrimaryUsagePageKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDProductIDArrayKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDProductIDKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDProductIDMaskKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDProductKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDQueueOptionsTypeEnqueueAll;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDQueueOptionsTypeNone;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDReportBufferEntrySizeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDReportDescriptorKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDReportIntervalKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDReportOptionNotInterrupt;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDReportOptionVariableSize;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDRequestTimeoutKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDResetKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSampleIntervalKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollAccelerationAlgorithmKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollAccelerationKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollAccelerationSupportKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollAccelerationTypeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollReportRateKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollResolutionKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollResolutionXKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollResolutionYKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDScrollResolutionZKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSensorPropertyBatchIntervalKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSensorPropertyMaxFIFOEventsKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSensorPropertyReportIntervalKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSensorPropertyReportLatencyKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSensorPropertySampleIntervalKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSerialNumberKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDServiceAccelerationProperties;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDServiceCapsLockStateKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDServiceEjectDelayKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDServiceInitialKeyRepeatDelayKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDServiceKeyRepeatDelayKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDServiceLockKeyDelayKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDStandardTypeANSI;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDStandardTypeISO;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDStandardTypeJIS;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDStandardTypeKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDStandardTypeUnspecified;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSurfaceDimensionsKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDSystemButtonPressedDuringDarkBoot;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTrackpadAccelerationType;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTrackpadScrollAccelerationKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransactionOptionDefaultOutputValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportAIDBValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportAirPlayValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportBTAACPValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportBluetoothLowEnergyValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportBluetoothValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportFIFOValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportI2CValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportIAPValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportInductiveInBandValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportSPIValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportSPUValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportSerialValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportUSBValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDTransportVirtualValue;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDUniqueIDKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDUseLinearScalingMouseAccelerationKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDUserKeyUsageMapKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDValueOptionsFlagPrevious;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDValueOptionsFlagRelativeSimple;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDValueOptionsUpdateElementValues;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDValueScaleTypeCalibrated;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDValueScaleTypeExponent;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDValueScaleTypePhysical;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDVendorIDKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDVendorIDSourceKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDVersionNumberKey;
#[cfg(feature = "hid")]
pub use self::__hid::kIOHIDWidthKey;
#[cfg(feature = "hid")]
pub use self::__hid::kKeyboardUsageMapKey;
#[cfg(feature = "hid")]
pub use self::__hid::kNumLockKeyboardUsageMapKey;
#[cfg(feature = "hid")]
pub use self::__hid::HIDReportCommandType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDAccelerationAlgorithmType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDCallbackFunction;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDCompletion;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDCompletionAction;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDevice;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceActivate;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceCancel;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceClose;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceConformsTo;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceCopyMatchingElements;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceCopyValueMultiple;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceCopyValueMultipleWithCallback;
#[cfg(all(feature = "hid", feature = "libc"))]
pub use self::__hid::IOHIDDeviceCreate;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceDeviceInterface;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceGetProperty;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceGetReport;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceGetReportWithCallback;
#[cfg(all(feature = "hid", feature = "libc"))]
pub use self::__hid::IOHIDDeviceGetService;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceGetValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceGetValueOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceGetValueWithCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceGetValueWithOptions;
#[cfg(all(feature = "hid", feature = "libc"))]
pub use self::__hid::IOHIDDeviceInterface;
#[cfg(all(feature = "hid", feature = "libc"))]
pub use self::__hid::IOHIDDeviceInterface121;
#[cfg(all(feature = "hid", feature = "libc"))]
pub use self::__hid::IOHIDDeviceInterface122;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceOpen;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceQueueInterface;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceRegisterInputReportCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceRegisterInputReportWithTimeStampCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceRegisterInputValueCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceRegisterRemovalCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceScheduleWithRunLoop;
#[cfg(all(feature = "dispatch2", feature = "hid"))]
pub use self::__hid::IOHIDDeviceSetCancelHandler;
#[cfg(all(feature = "dispatch2", feature = "hid"))]
pub use self::__hid::IOHIDDeviceSetDispatchQueue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetInputValueMatching;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetInputValueMatchingMultiple;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetProperty;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetReport;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetReportWithCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetValueMultiple;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetValueMultipleWithCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceSetValueWithCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceTimeStampedDeviceInterface;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceTransactionInterface;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDDeviceUnscheduleFromRunLoop;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementAttach;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementCallbackFunction;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementCollectionType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementCommitDirection;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementCookie;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementCopyAttached;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementCreateWithDictionary;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementDetach;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementFlags;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetChildren;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetCollectionType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetCookie;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetDevice;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetLogicalMax;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetLogicalMin;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetName;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetParent;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetPhysicalMax;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetPhysicalMin;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetProperty;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetReportCount;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetReportID;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetReportSize;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetUnit;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetUnitExponent;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetUsage;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementGetUsagePage;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementHasNullState;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementHasPreferredState;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementIsArray;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementIsNonLinear;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementIsRelative;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementIsVirtual;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementIsWrapping;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementSetProperty;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDElementType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDEventStruct;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDKeyboardEventOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDKeyboardPhysicalLayoutType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManager;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerActivate;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerCancel;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerClose;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerCopyDevices;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerCreate;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerGetProperty;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerOpen;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerRegisterDeviceMatchingCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerRegisterDeviceRemovalCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerRegisterInputReportCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerRegisterInputReportWithTimeStampCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerRegisterInputValueCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerSaveToPropertyDomain;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerScheduleWithRunLoop;
#[cfg(all(feature = "dispatch2", feature = "hid"))]
pub use self::__hid::IOHIDManagerSetCancelHandler;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerSetDeviceMatching;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerSetDeviceMatchingMultiple;
#[cfg(all(feature = "dispatch2", feature = "hid"))]
pub use self::__hid::IOHIDManagerSetDispatchQueue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerSetInputValueMatching;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerSetInputValueMatchingMultiple;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerSetProperty;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDManagerUnscheduleFromRunLoop;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDOptionsType;
#[cfg(all(feature = "hid", feature = "libc"))]
pub use self::__hid::IOHIDOutputTransactionInterface;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDPointerEventOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueActivate;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueAddElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueCancel;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueContainsElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueCopyNextValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueCopyNextValueWithTimeout;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueCreate;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueGetDepth;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueGetDevice;
#[cfg(all(feature = "hid", feature = "libc"))]
pub use self::__hid::IOHIDQueueInterface;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueOptionsType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueRegisterValueAvailableCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueRemoveElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueScheduleWithRunLoop;
#[cfg(all(feature = "dispatch2", feature = "hid"))]
pub use self::__hid::IOHIDQueueSetCancelHandler;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueSetDepth;
#[cfg(all(feature = "dispatch2", feature = "hid"))]
pub use self::__hid::IOHIDQueueSetDispatchQueue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueStart;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueStop;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDQueueUnscheduleFromRunLoop;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDReportCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDReportCallbackFunction;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDReportType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDReportWithTimeStampCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDScrollEventOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDServiceSensorControlOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDStandardType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransaction;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionAddElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionClear;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionCommit;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionCommitWithCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionContainsElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionCreate;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionDirectionType;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionGetDevice;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionGetDirection;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionGetValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionRemoveElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionScheduleWithRunLoop;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionSetDirection;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionSetValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDTransactionUnscheduleFromRunLoop;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueCreateWithBytes;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueCreateWithBytesNoCopy;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueCreateWithIntegerValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueGetBytePtr;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueGetElement;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueGetIntegerValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueGetLength;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueGetScaledValue;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueGetTimeStamp;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueMultipleCallback;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueOptions;
#[cfg(feature = "hid")]
pub use self::__hid::IOHIDValueScaleType;
#[cfg(all(feature = "graphics", feature = "hidsystem"))]
pub use self::__hidsystem::evioLLEvent;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::evsioEVSIOCCSIndices;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::evsioEVSIOSCSIndices;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::evsioKeymapping;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::evsioMouseScaling;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelGainCubicKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelGainLinearKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelGainParabolicKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelGainQuarticKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelIndexKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelParametricCurvesDebugKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelTangentSpeedCubicRootKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelTangentSpeedLinearKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelTangentSpeedParabolicRootKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDAccelTangentSpeedQuarticRootKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDScrollAccelParametricCurvesDebugKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kHIDTrackingAccelParametricCurvesKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDActivityDisplayOn;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDActivityUserIdle;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDActivityUserIdleKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDAutoDimBrightnessKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDAutoDimStateKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDAutoDimThresholdKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDAutoDimTimeKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDBrightnessKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDCapsLockState;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDClickSpaceKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDClickTimeKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDCurrentShmemVersion;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDDisallowRemappingOfPrimaryClickKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDEventNotification;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDEventQueueTypeKernel;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDEventQueueTypeUser;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDEventSystemConnectType;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDF12EjectDelayKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDFKeyModeKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDGlobalMemory;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDIdleTimeKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDInitialKeyRepeatKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDInterfaceIDKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyMappingKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyRepeatKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyboardCapsLockDoesLockKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyboardGlobalModifiersKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyboardModifierMappingDstKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyboardModifierMappingPairsKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyboardModifierMappingSrcKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyboardSupportedModifiersKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKeyboardSupportsF12EjectKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDKindKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDLastCompatibleShmemVersion;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDMessageRelayServiceInterfaceActive;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDMouseKeysEnablesVirtualNumPadKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDMouseKeysOnKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDMouseKeysOptionTogglesKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDNumLockState;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDOpenedByEventSystem;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDOpenedByFastPathClient;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDOriginalSubinterfaceIDKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDParamConnectType;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDParametersKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPointerAccelerationSettingsKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPointerAccelerationTableKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPointerButtonCountKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPointerContactToMoveKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPointerConvertAbsoluteKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPointerPressureToClickKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPointerResolutionKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDPostHIDManagerEvent;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDResetKeyboardKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDResetLEDsKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDResetPointerKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDResetStickyKeyNotification;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollAccelerationTableKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollAccelerationTableXKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollAccelerationTableYKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollAccelerationTableZKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountAccelerationFactorKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountBootDefaultKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountIgnoreMomentumScrollsKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountMaxKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountMaxTimeDeltaBetweenKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountMaxTimeDeltaToSustainKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountMinDeltaToStartKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountMinDeltaToSustainKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountMouseCanResetKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountResetKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollCountZeroKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollMouseButtonKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollResetKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDScrollZoomModifierMaskKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDServerConnectType;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDServiceGlobalModifiersUsageKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDSetCursorPosition;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDSetGlobalEventFlags;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDSetRelativeCursorPosition;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDSlowKeysDelayKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDStickyKeysDisabledKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDStickyKeysOnKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDStickyKeysShiftTogglesKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDSubinterfaceIDKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDSystemClass;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDUseKeyswitchKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDVirtualHIDevice;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIDWaitCursorFrameIntervalKey;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIKeyboardClass;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kIOHIPointingClass;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kNXEventDataVersion;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::kNXEventVersion;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::EvCmd;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::EvOffsets;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOFixedPoint32;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDAccessType;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDButtonModes;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDCheckAccess;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDCopyCFTypeParameter;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDCreateSharedMemory;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDEventSystemClient;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDEventSystemClientCopyProperty;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDEventSystemClientCopyServices;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDEventSystemClientCreateSimpleClient;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDEventSystemClientSetProperty;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetAccelerationWithKey;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetActivityState;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetButtonEventNum;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetModifierLockState;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetMouseAcceleration;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetMouseButtonMode;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetParameter;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetScrollAcceleration;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDGetStateForSelector;
#[cfg(all(feature = "graphics", feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDPostEvent;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDRegisterVirtualDisplay;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDRequestAccess;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDRequestType;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDServiceClient;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDServiceClientConformsTo;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDServiceClientCopyProperty;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDServiceClientGetRegistryID;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDServiceClientSetProperty;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetAccelerationWithKey;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetCFTypeParameter;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetCursorEnable;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetEventsEnable;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetModifierLockState;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetMouseAcceleration;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetMouseButtonMode;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetMouseLocation;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetParameter;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetScrollAcceleration;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetStateForSelector;
#[cfg(all(feature = "graphics", feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDSetVirtualDisplayBounds;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::IOHIDUnregisterVirtualDisplay;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDevice;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDeviceActivate;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDeviceCancel;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDeviceCopyProperty;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDeviceCreateWithProperties;
#[cfg(all(feature = "block2", feature = "hid", feature = "hidsystem"))]
pub use self::__hidsystem::IOHIDUserDeviceGetReportBlock;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDeviceHandleReportWithTimeStamp;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDeviceOptions;
#[cfg(all(feature = "block2", feature = "hid", feature = "hidsystem"))]
pub use self::__hidsystem::IOHIDUserDeviceRegisterGetReportBlock;
#[cfg(all(feature = "block2", feature = "hid", feature = "hidsystem"))]
pub use self::__hidsystem::IOHIDUserDeviceRegisterSetReportBlock;
#[cfg(all(feature = "dispatch2", feature = "hidsystem"))]
pub use self::__hidsystem::IOHIDUserDeviceSetCancelHandler;
#[cfg(all(feature = "dispatch2", feature = "hidsystem"))]
pub use self::__hidsystem::IOHIDUserDeviceSetDispatchQueue;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::IOHIDUserDeviceSetProperty;
#[cfg(all(feature = "block2", feature = "hid", feature = "hidsystem"))]
pub use self::__hidsystem::IOHIDUserDeviceSetReportBlock;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXClickTime;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXCloseEventStatus;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXCoord;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEvent;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_compound;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_compound_misc;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_key;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_mouse;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_mouseMove;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_mouseMove_tablet;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_mouse_tablet;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_proximity;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_scrollWheel;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_tablet;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_tablet_tilt;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventData_tracking;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXEventHandle;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventSystemDevice;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventSystemDeviceList;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXEventSystemInfo;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXEventSystemInfoType;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXGetClickSpace;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXKeyMapping;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXKeyRepeatInterval;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXKeyRepeatThreshold;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXMouseButton;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXMouseScaling;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXOpenEventStatus;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXParsedKeyMapping;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXPoint;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXResetKeyboard;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXResetMouse;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXSetClickSpace;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXSetClickTime;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXSetKeyRepeatInterval;
#[cfg(all(feature = "hidsystem", feature = "libc"))]
pub use self::__hidsystem::NXSetKeyRepeatThreshold;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXSize;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXTabletPointData;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::NXTabletProximityData;
#[cfg(feature = "hidsystem")]
pub(crate) use self::__hidsystem::_NXEvent_location;
#[cfg(feature = "hidsystem")]
pub(crate) use self::__hidsystem::_NXParsedKeyMapping_;
#[cfg(feature = "hidsystem")]
pub(crate) use self::__hidsystem::_NXTabletPointData_tilt;
#[cfg(feature = "hidsystem")]
pub(crate) use self::__hidsystem::_evOffsets;
#[cfg(feature = "hidsystem")]
pub use self::__hidsystem::EVENT_SYSTEM_VERSION;
#[cfg(feature = "network")]
pub use self::__network::kIOActiveMedium;
#[cfg(feature = "network")]
pub use self::__network::kIOActivePacketFilters;
#[cfg(feature = "network")]
pub use self::__network::kIOBuiltin;
#[cfg(feature = "network")]
pub use self::__network::kIODefaultMedium;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetAddressSize;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetCRCSize;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetControllerClass;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetDisabledWakeOnLANFilterGroup;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetInterfaceClass;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetMaxPacketSize;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetMinPacketSize;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetStatsKey;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetWakeOnLANFilterGroup;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetWakeOnMagicPacket;
#[cfg(feature = "network")]
pub use self::__network::kIOEthernetWakeOnPacketAddressMatch;
#[cfg(feature = "network")]
pub use self::__network::kIOFeatures;
#[cfg(feature = "network")]
pub use self::__network::kIOInterfaceExtraFlags;
#[cfg(feature = "network")]
pub use self::__network::kIOInterfaceFlags;
#[cfg(feature = "network")]
pub use self::__network::kIOInterfaceNamePrefix;
#[cfg(feature = "network")]
pub use self::__network::kIOInterfaceState;
#[cfg(feature = "network")]
pub use self::__network::kIOInterfaceType;
#[cfg(feature = "network")]
pub use self::__network::kIOInterfaceUnit;
#[cfg(feature = "network")]
pub use self::__network::kIOLinkData;
#[cfg(feature = "network")]
pub use self::__network::kIOLinkSpeed;
#[cfg(feature = "network")]
pub use self::__network::kIOLinkStatus;
#[cfg(feature = "network")]
pub use self::__network::kIOLocation;
#[cfg(feature = "network")]
pub use self::__network::kIOMACAddress;
#[cfg(feature = "network")]
pub use self::__network::kIOMaxPacketSize;
#[cfg(feature = "network")]
pub use self::__network::kIOMaxTransferUnit;
#[cfg(feature = "network")]
pub use self::__network::kIOMediaAddressLength;
#[cfg(feature = "network")]
pub use self::__network::kIOMediaHeaderLength;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumDictionary;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet1000BaseCX;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet1000BaseLX;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet1000BaseSX;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet1000BaseT;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet1000BaseTX;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet100BaseFX;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet100BaseT2;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet100BaseT4;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet100BaseTX;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet100BaseVG;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10Base2;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10Base5;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10BaseFL;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10BaseSTP;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10BaseT;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10GBaseCX4;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10GBaseLR;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10GBaseSR;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet10GBaseT;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet2500BaseT;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernet5000BaseT;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernetAuto;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernetHomePNA1;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernetManual;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumEthernetNone;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumFlags;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211Auto;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211DS1;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211DS11;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211DS2;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211DS5;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211FH1;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211FH2;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211Manual;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211None;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIEEE80211OptionAdhoc;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumIndex;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionEEE;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionFlag0;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionFlag1;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionFlag2;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionFlowControl;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionFullDuplex;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionHalfDuplex;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumOptionLoopback;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumSpeed;
#[cfg(feature = "network")]
pub use self::__network::kIOMediumType;
#[cfg(feature = "network")]
pub use self::__network::kIOMinPacketSize;
#[cfg(feature = "network")]
pub use self::__network::kIOModel;
#[cfg(feature = "network")]
pub use self::__network::kIOMulticastAddressList;
#[cfg(feature = "network")]
pub use self::__network::kIOMulticastFilterData;
#[cfg(feature = "network")]
pub use self::__network::kIONUCGetNetworkDataCapacityIndex;
#[cfg(feature = "network")]
pub use self::__network::kIONUCGetNetworkDataHandleIndex;
#[cfg(feature = "network")]
pub use self::__network::kIONUCLastIndex;
#[cfg(feature = "network")]
pub use self::__network::kIONUCReadNetworkDataIndex;
#[cfg(feature = "network")]
pub use self::__network::kIONUCResetNetworkDataIndex;
#[cfg(feature = "network")]
pub use self::__network::kIONUCType;
#[cfg(feature = "network")]
pub use self::__network::kIONUCWriteNetworkDataIndex;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkControllerClass;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkData;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataAccessTypeMask;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataAccessTypeRead;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataAccessTypeReset;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataAccessTypeSerialize;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataAccessTypeWrite;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataAccessTypes;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataBasicAccessTypes;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataBufferTypeExternal;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataBufferTypeInternal;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataBufferTypeNone;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataBytes;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkDataSize;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureHWTimeStamp;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureHardwareVlan;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureLRO;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureMultiPages;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureNoBSDWait;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureSWTimeStamp;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureSoftwareVlan;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureTSOIPv4;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureTSOIPv6;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFeatureTransmitCompletionStatus;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkFilterGroup;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkInterfaceClass;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkInterfaceDisabledState;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkInterfaceOpenedState;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkInterfaceRegisteredState;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkLinkActive;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkLinkNoNetworkChange;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkLinkValid;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkNoBSDAttachKey;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkStackRegisterInterfaceAll;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkStackRegisterInterfaceWithLowestUnit;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkStackRegisterInterfaceWithUnit;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkStackUserCommand;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkStackUserCommandKey;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkStatsKey;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkSupportedPacketFilters;
#[cfg(feature = "network")]
pub use self::__network::kIONetworkUserClientTypeID;
#[cfg(feature = "network")]
pub use self::__network::kIOOutputQueueStatsKey;
#[cfg(feature = "network")]
pub use self::__network::kIOPacketFilterBroadcast;
#[cfg(feature = "network")]
pub use self::__network::kIOPacketFilterMulticast;
#[cfg(feature = "network")]
pub use self::__network::kIOPacketFilterMulticastAll;
#[cfg(feature = "network")]
pub use self::__network::kIOPacketFilterPromiscuous;
#[cfg(feature = "network")]
pub use self::__network::kIOPacketFilterPromiscuousAll;
#[cfg(feature = "network")]
pub use self::__network::kIOPacketFilterUnicast;
#[cfg(feature = "network")]
pub use self::__network::kIOPacketFilters;
#[cfg(feature = "network")]
pub use self::__network::kIOPrimaryInterface;
#[cfg(feature = "network")]
pub use self::__network::kIORequiredPacketFilters;
#[cfg(feature = "network")]
pub use self::__network::kIORevision;
#[cfg(feature = "network")]
pub use self::__network::kIOSelectedMedium;
#[cfg(feature = "network")]
pub use self::__network::kIOVendor;
#[cfg(feature = "network")]
pub use self::__network::IODot3CollEntry;
#[cfg(feature = "network")]
pub use self::__network::IODot3RxExtraEntry;
#[cfg(feature = "network")]
pub use self::__network::IODot3StatsEntry;
#[cfg(feature = "network")]
pub use self::__network::IODot3TxExtraEntry;
#[cfg(feature = "network")]
pub use self::__network::IOEthernetStats;
#[cfg(feature = "network")]
pub use self::__network::IOMediumType;
#[cfg(feature = "network")]
pub use self::__network::IONDHandle;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkClose;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkGetDataCapacity;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkGetDataHandle;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkGetPacketFiltersMask;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkOpen;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkReadData;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkResetData;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkSetPacketFiltersMask;
#[cfg(feature = "network")]
pub use self::__network::IONetworkStats;
#[cfg(all(feature = "libc", feature = "network"))]
pub use self::__network::IONetworkWriteData;
#[cfg(feature = "network")]
pub use self::__network::IOOutputQueueStats;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPMACPowerKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPMBatteryPowerKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPMUPSPowerKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSACPowerValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSBatteryFailureModesKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSBatteryHealthConditionKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSBatteryHealthKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSBatteryPowerValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCheckBatteryValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCommandDelayedRemovePowerKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCommandEnableAudibleAlarmKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCommandSendCurrentStateOfCharge;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCommandSendCurrentTemperature;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCommandSetCurrentLimitKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCommandSetRequiredVoltageKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCommandStartupDelayKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCurrentCapacityKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSCurrentKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSDeadWarnLevelKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSDesignCapacityKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSDynamicStorePath;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureCellImbalance;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureChargeFET;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureChargeOverCurrent;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureChargeOverTemp;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureDataFlushFault;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureDischargeFET;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureDischargeOverCurrent;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureDischargeOverTemp;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureExternalInput;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureFuseBlown;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureOpenThermistor;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailurePeriodicAFEComms;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailurePermanentAFEComms;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFailureSafetyOverVoltage;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSFairValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSGoodValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSHardwareSerialNumberKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSHealthConfidenceKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSInternalBatteryType;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSInternalFailureKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSInternalType;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSIsChargedKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSIsChargingKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSIsFinishingChargeKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSIsPresentKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSLowWarnLevelKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSMaxCapacityKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSMaxErrKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNameKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNetworkTransportType;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNominalCapacityKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNotifyAnyPowerSource;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNotifyAttach;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNotifyLowBattery;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNotifyPowerSource;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSNotifyTimeRemaining;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSOffLineValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPermanentFailureValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPoorValue;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerAdapterCurrentKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerAdapterFamilyKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerAdapterIDKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerAdapterRevisionKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerAdapterSerialNumberKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerAdapterSourceKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerAdapterWattsKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerSourceIDKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSPowerSourceStateKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSProductIDKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSSerialTransportType;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTemperatureKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTimeRemainingNotificationKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTimeRemainingUnknown;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTimeRemainingUnlimited;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTimeToEmptyKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTimeToFullChargeKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTransportTypeKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSTypeKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSUPSManagementClaimed;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSUPSType;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSUSBTransportType;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSVendorDataKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSVendorIDKey;
#[cfg(feature = "ps")]
pub use self::__ps::kIOPSVoltageKey;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSCopyExternalPowerAdapterDetails;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSCopyPowerSourcesInfo;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSCopyPowerSourcesList;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSCreateLimitedPowerNotification;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSGetBatteryWarningLevel;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSGetPowerSourceDescription;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSGetProvidingPowerSourceType;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSGetTimeRemainingEstimate;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSLowBatteryWarningLevel;
#[cfg(feature = "ps")]
pub use self::__ps::IOPSNotificationCreateRunLoopSource;
#[cfg(feature = "ps")]
pub use self::__ps::IOPowerSourceCallbackType;
#[cfg(feature = "ps")]
pub use self::__ps::IOUPSEventCallbackFunction;
#[cfg(feature = "ps")]
pub use self::__ps::IOUPSPlugInInterface;
#[cfg(feature = "ps")]
pub use self::__ps::IOUPSPlugInInterface_v140;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kAppleClamshellCausesSleepKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kAppleClamshellStateKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kClamshellSleepBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kClamshellStateBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryAmperageKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryCapacityKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryCharge;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryChargerConnect;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryCurrentChargeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryCycleCountKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryFlagsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryInfoKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryInstalled;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOBatteryVoltageKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMACInstalled;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMACnoChargeCapability;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAckImplied;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAllowSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAssertionLevelOff;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAssertionLevelOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAssertionsDriverDetailedKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAssertionsDriverKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAutoPowerOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAutoRestart;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAutoShutdown;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAutoSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAutoWake;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAutoWakeOrPowerOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMAuxPowerOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBadSpecification;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryAtWarn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryChargeStatusGradient;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryChargeStatusTooCold;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryChargeStatusTooHot;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryChargeStatusTooHotOrCold;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryCharging;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryDepleted;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBatteryInstalled;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBootSessionUUIDKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMBroadcastAggressiveness;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMCPUPowerLimitProcessorCountKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMCPUPowerLimitProcessorSpeedKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMCPUPowerLimitSchedulerTimeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMCPUPowerLimitsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMCPUPowerNotificationKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMCannotRaisePower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMCapabilitiesMask;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMChildClamp;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMChildClamp2;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMClamshellClosed;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMClamshellOpened;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMClamshellStateOnWake;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMClockNormal;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMClockRunning;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMClosedClamshell;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMConfigRetained;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMContextRetained;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDWOverTemp;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDeepSleepDelayHighKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDeepSleepDelayKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDeepSleepEnabledKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDestroyFVKeyOnStandbyKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDeviceNameKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDeviceUsable;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDisableClamshell;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDoze;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionAssertedKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionBluetoothHIDDevicePairedBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionCPUBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionCreatedTimeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionExternalMediaMountedBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionForceFullWakeupBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionForceWakeupBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionIDKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionLevelKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionMagicPacketWakeEnabledBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionModifiedTimeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionNetworkKeepAliveActiveBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionOwnerServiceKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionOwnerStringKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionPreventDisplaySleepBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionPreventSystemIdleSleepBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionRegistryEntryIDKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionReservedBit5;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionReservedBit7;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMDriverAssertionUSBExternalDeviceBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMEnableClamshell;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMExternalPower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMFairValue;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMForceLowSpeed;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMFullyChargedKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMGoodValue;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMGraphicsPowerLimitPerformanceKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMGraphicsPowerLimitsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMHighestState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMInitialDeviceState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMInternalPower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMLowPower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMLowestState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMaxPerformance;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMaxPowerStates;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageBatteryStatusHasChanged;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageClamshellStateChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageDarkWakeThermalEmergency;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageDriverAssertionsChanged;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageFeatureChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageInternalBatteryFullyDischarged;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageSleepWakeUUIDChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMMessageSystemPowerEventOccurred;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNextHigherState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNextLowerState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNoErr;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNoSuchState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNotAttainable;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNotPowerManaged;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNotYetInitialized;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMNullAssertionID;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMOverTemp;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsAmperageKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsCloakedKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsDescriptionKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsErrorFlagsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsFamilyKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsIDKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsPMUConfigurationKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsRevisionKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsSerialNumberKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsSharedSourceKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsSourceIDKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsVoltage;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterDetailsWattsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAdapterInfoKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAmperageKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAtCriticalLevelKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSAtWarnLevelKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSBatteryChargeStatusKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSBatteryHealthKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSBatteryInstalledKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSBatteryTemperatureKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSCapacityEstimatedKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSChargerConfigurationKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSCurrentCapacityKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSCycleCountKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSDesignCapacityKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSErrorConditionKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSExternalChargeCapableKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSExternalConnectedKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSHealthConfidenceKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSInvalidWakeSecondsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSIsChargingKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSLegacyBatteryInfoKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSLocationKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSLocationLeft;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSLocationRight;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSManufactureDateKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSManufacturerKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSMaxCapacityKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSMaxErrKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSModelKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSPostChargeWaitSecondsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSPostDishargeWaitSecondsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSSerialKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSTimeRemainingKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPSVoltageKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMParameterError;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPassThrough;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPoorValue;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerButton;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerButtonUp;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerEmergency;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerEventAppNameKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerEventAppPIDKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerEventTimeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerEventTypeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPowerOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPreventIdleSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPreventSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMPreventSystemSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMProModeDisengaged;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMProModeEngaged;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMProcessorSpeedChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMRawLowBattery;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMResetPowerStateOnWakeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMRestart;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMRestartCapability;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMRootDomainState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingAutoPowerCalendarKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingAutoPowerSecondsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingAutoWakeCalendarKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingAutoWakeSecondsKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingDebugPowerRelativeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingDebugWakeRelativeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingDisplaySleepUsesDimKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingGraphicsSwitchKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingMaintenanceWakeCalendarKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingMobileMotionModuleKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingProModeControl;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingProModeDefer;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingReduceBrightnessKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingRestartOnPowerLossKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingSleepOnPowerButtonKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingTimeZoneOffsetKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingWakeOnACChangeKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingWakeOnClamshellKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSettingWakeOnRingKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSleepCapability;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSleepNow;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSleepWakeUUIDKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSoftSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMStandbyBatteryThresholdKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMStaticPowerValid;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSystemCapabilityAOT;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSystemCapabilityAudio;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSystemCapabilityCPU;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSystemCapabilityDidChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSystemCapabilityGraphics;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSystemCapabilityNetwork;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMSystemCapabilityWillChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalLevelCritical;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalLevelDanger;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalLevelNormal;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalLevelTrap;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalLevelUnknown;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalLevelWarning;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalLevelWarningKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalWarningLevelCrisis;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalWarningLevelDanger;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalWarningLevelNormal;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMThermalWarningNotificationKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMUMessageLegacyAutoPower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMUMessageLegacyAutoWake;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMUPSInstalled;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMUndefinedValue;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMUnidleDevice;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMUnknown;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPMWillAckLater;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSAdapterErrorFlagDeviceNeedsToBeRepositioned;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSAdapterErrorFlagForeignObjectDetected;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSAdapterErrorFlagInsufficientAvailablePower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSAdapterErrorFlagNoErrors;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeAC;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeDisconnected;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal2;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal3;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal4;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal5;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal6;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal7;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeExternal8;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeFirewire;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBAdapter;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBCBrick;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBCPD;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBCTypeC;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBChargingPort;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBChargingPortDedicated;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBChargingPortDownstream;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBDevice;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBHost;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBHostSuspended;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUSBUnknown;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOPSFamilyCodeUnsupportedRegion;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOREMSleepEnabledKey;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOSystemLoadAdvisoryLevelBad;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOSystemLoadAdvisoryLevelGreat;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOSystemLoadAdvisoryLevelOK;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kIOSystemLoadAdvisoryNotifyName;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kInflowForciblyEnabledBit;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kMaxType;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kNumPMMethods;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMActivityTickle;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMAllowPowerChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMCancelPowerChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMEthernetWakeOnLANSettings;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMGeneralAggressiveness;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMGetAggressiveness;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMGetSystemSleepType;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMLastAggressivenessType;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMMinutesToDim;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMMinutesToSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMMinutesToSpinDown;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMMotionSensor;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMPowerSource;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMRequestIdleSleepRevert;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMRestartSystem;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetAggressiveness;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetClamshellSleepState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetDisplayPowerOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetDisplayState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetLDMHibernationDisable;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetMaintenanceWakeCalendar;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetProcessorSpeed;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSetUserAssertionLevels;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMShutdownSystem;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSleepSystem;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSleepSystemOptions;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSleepWakeDebugTrig;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::kPMSleepWakeWatchdogEnable;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::sleepWakeNote;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOAllowPowerChange;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOCancelPowerChange;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOCopySystemLoadAdvisoryDetailed;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IODeregisterApp;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IODeregisterForSystemPower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOGetSystemLoadAdvisory;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAckImplied;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionCopyProperties;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionCreate;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionCreateWithDescription;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionCreateWithName;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionCreateWithProperties;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionDeclareUserActivity;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionID;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionLevel;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionRelease;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionRetain;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAssertionSetProperty;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMAuxPowerOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMBadSpecification;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMCalendarStruct;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMCancelScheduledPowerEvent;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMCannotRaisePower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMClockNormal;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMClockRunning;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMConfigRetained;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMContextRetained;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMCopyAssertionsByProcess;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMCopyAssertionsStatus;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOPMCopyBatteryInfo;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMCopyCPUPowerStatus;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMCopyScheduledPowerEvents;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMDeclareNetworkClientActivity;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMDeviceUsable;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOPMFindPowerManagement;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOPMGetAggressiveness;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMGetThermalWarningLevel;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMHighestState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMLowestState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMMaxPerformance;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMMaxPowerStates;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMNextHigherState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMNextLowerState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMNoErr;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMNoSuchState;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMNotAttainable;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMNotPowerManaged;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMNotYetInitialized;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMParameterError;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMPowerFlags;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMPowerOn;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMSchedulePowerEvent;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOPMSetAggressiveness;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOPMSleepEnabled;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IOPMSleepSystem;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMSoftSleep;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMSystemCapabilityChangeParameters;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMUserActiveType;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPMWillAckLater;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOPowerStateChangeNotification;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IORegisterApp;
#[cfg(all(feature = "libc", feature = "pwr_mgt"))]
pub use self::__pwr_mgt::IORegisterForSystemPower;
#[cfg(feature = "pwr_mgt")]
pub use self::__pwr_mgt::IOSystemLoadAdvisoryLevel;
#[cfg(feature = "serial")]
pub use self::__serial::kIOCalloutDeviceKey;
#[cfg(feature = "serial")]
pub use self::__serial::kIODialinDeviceKey;
#[cfg(feature = "serial")]
pub use self::__serial::kIOSerialBSDAllTypes;
#[cfg(feature = "serial")]
pub use self::__serial::kIOSerialBSDModemType;
#[cfg(feature = "serial")]
pub use self::__serial::kIOSerialBSDRS232Type;
#[cfg(feature = "serial")]
pub use self::__serial::kIOSerialBSDServiceValue;
#[cfg(feature = "serial")]
pub use self::__serial::kIOSerialBSDTypeKey;
#[cfg(feature = "serial")]
pub use self::__serial::kIOTTYBaseNameKey;
#[cfg(feature = "serial")]
pub use self::__serial::kIOTTYDeviceKey;
#[cfg(feature = "serial")]
pub use self::__serial::kIOTTYSuffixKey;
#[cfg(feature = "serial")]
pub use self::__serial::kIOTTYWaitForIdleKey;
#[cfg(feature = "serial")]
pub use self::__serial::user_shspeed_t;
#[cfg(feature = "serial")]
pub use self::__serial::user_speed_t;
#[cfg(feature = "serial")]
pub use self::__serial::user_ul_t;
#[cfg(feature = "serial")]
pub use self::__serial::user_us_t;
#[cfg(feature = "usb")]
pub use self::__usb::*;
use core::cell::UnsafeCell;
use core::ffi::*;
use core::marker::{PhantomData, PhantomPinned};
use core::ptr::NonNull;
#[cfg(feature = "dispatch2")]
use dispatch2::*;
#[cfg(feature = "objc2")]
use objc2::__framework_prelude::*;
use objc2_core_foundation::*;

use crate::*;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_common?language=objc)
pub const sub_iokit_common: c_uint = err_sub!(0);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_usb?language=objc)
pub const sub_iokit_usb: c_uint = err_sub!(1);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_firewire?language=objc)
pub const sub_iokit_firewire: c_uint = err_sub!(2);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_block_storage?language=objc)
pub const sub_iokit_block_storage: c_uint = err_sub!(4);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_graphics?language=objc)
pub const sub_iokit_graphics: c_uint = err_sub!(5);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_networking?language=objc)
pub const sub_iokit_networking: c_uint = err_sub!(6);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_bluetooth?language=objc)
pub const sub_iokit_bluetooth: c_uint = err_sub!(8);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_pmu?language=objc)
pub const sub_iokit_pmu: c_uint = err_sub!(9);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_acpi?language=objc)
pub const sub_iokit_acpi: c_uint = err_sub!(10);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_smbus?language=objc)
pub const sub_iokit_smbus: c_uint = err_sub!(11);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_ahci?language=objc)
pub const sub_iokit_ahci: c_uint = err_sub!(12);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_powermanagement?language=objc)
pub const sub_iokit_powermanagement: c_uint = err_sub!(13);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_hidsystem?language=objc)
pub const sub_iokit_hidsystem: c_uint = err_sub!(14);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_scsi?language=objc)
pub const sub_iokit_scsi: c_uint = err_sub!(16);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_usbaudio?language=objc)
pub const sub_iokit_usbaudio: c_uint = err_sub!(17);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_wirelesscharging?language=objc)
pub const sub_iokit_wirelesscharging: c_uint = err_sub!(18);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_graphics_acceleration?language=objc)
pub const sub_iokit_graphics_acceleration: c_uint = err_sub!(30);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_keystore?language=objc)
pub const sub_iokit_keystore: c_uint = err_sub!(31);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_apfs?language=objc)
pub const sub_iokit_apfs: c_uint = err_sub!(33);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_acpiec?language=objc)
pub const sub_iokit_acpiec: c_uint = err_sub!(34);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/sub_iokit_timesync_avb?language=objc)
pub const sub_iokit_timesync_avb: c_uint = err_sub!(35);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnerror?language=objc)
pub const kIOReturnError: c_uint = iokit_common_err!(0x2bc);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnomemory?language=objc)
pub const kIOReturnNoMemory: c_uint = iokit_common_err!(0x2bd);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnoresources?language=objc)
pub const kIOReturnNoResources: c_uint = iokit_common_err!(0x2be);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnipcerror?language=objc)
pub const kIOReturnIPCError: c_uint = iokit_common_err!(0x2bf);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnodevice?language=objc)
pub const kIOReturnNoDevice: c_uint = iokit_common_err!(0x2c0);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotprivileged?language=objc)
pub const kIOReturnNotPrivileged: c_uint = iokit_common_err!(0x2c1);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnbadargument?language=objc)
pub const kIOReturnBadArgument: c_uint = iokit_common_err!(0x2c2);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnlockedread?language=objc)
pub const kIOReturnLockedRead: c_uint = iokit_common_err!(0x2c3);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnlockedwrite?language=objc)
pub const kIOReturnLockedWrite: c_uint = iokit_common_err!(0x2c4);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnexclusiveaccess?language=objc)
pub const kIOReturnExclusiveAccess: c_uint = iokit_common_err!(0x2c5);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnbadmessageid?language=objc)
pub const kIOReturnBadMessageID: c_uint = iokit_common_err!(0x2c6);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnunsupported?language=objc)
pub const kIOReturnUnsupported: c_uint = iokit_common_err!(0x2c7);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnvmerror?language=objc)
pub const kIOReturnVMError: c_uint = iokit_common_err!(0x2c8);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturninternalerror?language=objc)
pub const kIOReturnInternalError: c_uint = iokit_common_err!(0x2c9);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnioerror?language=objc)
pub const kIOReturnIOError: c_uint = iokit_common_err!(0x2ca);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturncannotlock?language=objc)
pub const kIOReturnCannotLock: c_uint = iokit_common_err!(0x2cc);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotopen?language=objc)
pub const kIOReturnNotOpen: c_uint = iokit_common_err!(0x2cd);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotreadable?language=objc)
pub const kIOReturnNotReadable: c_uint = iokit_common_err!(0x2ce);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotwritable?language=objc)
pub const kIOReturnNotWritable: c_uint = iokit_common_err!(0x2cf);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotaligned?language=objc)
pub const kIOReturnNotAligned: c_uint = iokit_common_err!(0x2d0);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnbadmedia?language=objc)
pub const kIOReturnBadMedia: c_uint = iokit_common_err!(0x2d1);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnstillopen?language=objc)
pub const kIOReturnStillOpen: c_uint = iokit_common_err!(0x2d2);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnrlderror?language=objc)
pub const kIOReturnRLDError: c_uint = iokit_common_err!(0x2d3);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturndmaerror?language=objc)
pub const kIOReturnDMAError: c_uint = iokit_common_err!(0x2d4);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnbusy?language=objc)
pub const kIOReturnBusy: c_uint = iokit_common_err!(0x2d5);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturntimeout?language=objc)
pub const kIOReturnTimeout: c_uint = iokit_common_err!(0x2d6);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnoffline?language=objc)
pub const kIOReturnOffline: c_uint = iokit_common_err!(0x2d7);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotready?language=objc)
pub const kIOReturnNotReady: c_uint = iokit_common_err!(0x2d8);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotattached?language=objc)
pub const kIOReturnNotAttached: c_uint = iokit_common_err!(0x2d9);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnochannels?language=objc)
pub const kIOReturnNoChannels: c_uint = iokit_common_err!(0x2da);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnospace?language=objc)
pub const kIOReturnNoSpace: c_uint = iokit_common_err!(0x2db);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnportexists?language=objc)
pub const kIOReturnPortExists: c_uint = iokit_common_err!(0x2dd);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturncannotwire?language=objc)
pub const kIOReturnCannotWire: c_uint = iokit_common_err!(0x2de);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnointerrupt?language=objc)
pub const kIOReturnNoInterrupt: c_uint = iokit_common_err!(0x2df);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnoframes?language=objc)
pub const kIOReturnNoFrames: c_uint = iokit_common_err!(0x2e0);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnmessagetoolarge?language=objc)
pub const kIOReturnMessageTooLarge: c_uint = iokit_common_err!(0x2e1);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotpermitted?language=objc)
pub const kIOReturnNotPermitted: c_uint = iokit_common_err!(0x2e2);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnopower?language=objc)
pub const kIOReturnNoPower: c_uint = iokit_common_err!(0x2e3);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnomedia?language=objc)
pub const kIOReturnNoMedia: c_uint = iokit_common_err!(0x2e4);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnunformattedmedia?language=objc)
pub const kIOReturnUnformattedMedia: c_uint = iokit_common_err!(0x2e5);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnunsupportedmode?language=objc)
pub const kIOReturnUnsupportedMode: c_uint = iokit_common_err!(0x2e6);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnunderrun?language=objc)
pub const kIOReturnUnderrun: c_uint = iokit_common_err!(0x2e7);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnoverrun?language=objc)
pub const kIOReturnOverrun: c_uint = iokit_common_err!(0x2e8);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturndeviceerror?language=objc)
pub const kIOReturnDeviceError: c_uint = iokit_common_err!(0x2e9);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnocompletion?language=objc)
pub const kIOReturnNoCompletion: c_uint = iokit_common_err!(0x2ea);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnaborted?language=objc)
pub const kIOReturnAborted: c_uint = iokit_common_err!(0x2eb);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnobandwidth?language=objc)
pub const kIOReturnNoBandwidth: c_uint = iokit_common_err!(0x2ec);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotresponding?language=objc)
pub const kIOReturnNotResponding: c_uint = iokit_common_err!(0x2ed);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnisotooold?language=objc)
pub const kIOReturnIsoTooOld: c_uint = iokit_common_err!(0x2ee);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnisotoonew?language=objc)
pub const kIOReturnIsoTooNew: c_uint = iokit_common_err!(0x2ef);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturnnotfound?language=objc)
pub const kIOReturnNotFound: c_uint = iokit_common_err!(0x2f0);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioreturninvalid?language=objc)
pub const kIOReturnInvalid: c_uint = iokit_common_err!(0x1);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiokitbuildversionkey?language=objc)
pub const kIOKitBuildVersionKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOKitBuildVersion\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiokitdiagnosticskey?language=objc)
pub const kIOKitDiagnosticsKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOKitDiagnostics\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioregistryplaneskey?language=objc)
pub const kIORegistryPlanesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IORegistryPlanes\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocataloguekey?language=objc)
pub const kIOCatalogueKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOCatalogue\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserviceplane?language=objc)
pub const kIOServicePlane: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOService\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopowerplane?language=objc)
pub const kIOPowerPlane: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPower\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodevicetreeplane?language=objc)
pub const kIODeviceTreePlane: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IODeviceTree\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioaudioplane?language=objc)
pub const kIOAudioPlane: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOAudio\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiofirewireplane?language=objc)
pub const kIOFireWirePlane: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOFireWire\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiousbplane?language=objc)
pub const kIOUSBPlane: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUSB\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioregistryentryidkey?language=objc)
pub const kIORegistryEntryIDKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IORegistryEntryID\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioregistryentrypropertykeyskey?language=objc)
pub const kIORegistryEntryPropertyKeysKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IORegistryEntryPropertyKeys\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioregistryentryallowablesetpropertieskey?language=objc)
pub const kIORegistryEntryAllowableSetPropertiesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IORegistryEntryAllowableSetProperties\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioregistryentrydefaultlockingsetpropertieskey?language=objc)
pub const kIORegistryEntryDefaultLockingSetPropertiesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IORegistryEntryDefaultLockingSetProperties\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserviceclass?language=objc)
pub const kIOServiceClass: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOService\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioresourcesclass?language=objc)
pub const kIOResourcesClass: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOResources\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioclasskey?language=objc)
pub const kIOClassKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOClass\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioprobescorekey?language=objc)
pub const kIOProbeScoreKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOProbeScore\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiokitdebugkey?language=objc)
pub const kIOKitDebugKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOKitDebug\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclasskey?language=objc)
pub const kIOUserClassKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClass\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclasseskey?language=objc)
pub const kIOUserClassesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClasses\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosupportedpropertieskey?language=objc)
pub const kIOSupportedPropertiesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOSupportedProperties\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserservicepropertieskey?language=objc)
pub const kIOUserServicePropertiesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserServiceProperties\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioproviderclasskey?language=objc)
pub const kIOProviderClassKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOProviderClass\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionamematchkey?language=objc)
pub const kIONameMatchKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IONameMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopropertymatchkey?language=objc)
pub const kIOPropertyMatchKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPropertyMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopropertyexistsmatchkey?language=objc)
pub const kIOPropertyExistsMatchKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPropertyExistsMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopathmatchkey?language=objc)
pub const kIOPathMatchKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPathMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiolocationmatchkey?language=objc)
pub const kIOLocationMatchKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOLocationMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioparentmatchkey?language=objc)
pub const kIOParentMatchKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOParentMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioresourcematchkey?language=objc)
pub const kIOResourceMatchKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOResourceMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioresourcematchedkey?language=objc)
pub const kIOResourceMatchedKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOResourceMatched\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchedservicecountkey?language=objc)
pub const kIOMatchedServiceCountKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMatchedServiceCountMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionamematchedkey?language=objc)
pub const kIONameMatchedKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IONameMatched\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchcategorykey?language=objc)
pub const kIOMatchCategoryKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMatchCategory\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodefaultmatchcategorykey?language=objc)
pub const kIODefaultMatchCategoryKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IODefaultMatchCategory\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchedpersonalitykey?language=objc)
pub const kIOMatchedPersonalityKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMatchedPersonality\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorematchpersonalitykey?language=objc)
pub const kIORematchPersonalityKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IORematchPersonality\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorematchcountkey?language=objc)
pub const kIORematchCountKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IORematchCount\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodextmatchcountkey?language=objc)
pub const kIODEXTMatchCountKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IODEXTMatchCount\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiousercliententitlementskey?language=objc)
pub const kIOUserClientEntitlementsKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientEntitlements\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioservicedextentitlementskey?language=objc)
pub const kIOServiceDEXTEntitlementsKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOServiceDEXTEntitlements\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitentitlementkey?language=objc)
pub const kIODriverKitEntitlementKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitusercliententitlementskey?language=objc)
pub const kIODriverKitUserClientEntitlementsKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit.userclient-access\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitrequiredentitlementskey?language=objc)
pub const kIODriverKitRequiredEntitlementsKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.private.driverkit.driver-access\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkittestdriverentitlementkey?language=objc)
pub const kIODriverKitTestDriverEntitlementKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.private.driverkit.test-driver\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitusercliententitlementallowanykey?language=objc)
pub const kIODriverKitUserClientEntitlementAllowAnyKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(
        b"com.apple.developer.driverkit.allow-any-userclient-access\0",
    )
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitusercliententitlementadministratorkey?language=objc)
pub const kIODriverKitUserClientEntitlementAdministratorKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit.administrator\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitusercliententitlementcommunicateswithdriverskey?language=objc)
pub const kIODriverKitUserClientEntitlementCommunicatesWithDriversKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(
        b"com.apple.developer.driverkit.communicates-with-drivers\0",
    )
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitusercliententitlementallowthirdpartyuserclientskey?language=objc)
pub const kIODriverKitUserClientEntitlementAllowThirdPartyUserClientsKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(
        b"com.apple.developer.driverkit.allow-third-party-userclients\0",
    )
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkitusbtransportentitlementkey?language=objc)
pub const kIODriverKitUSBTransportEntitlementKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit.transport.usb\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkithidtransportentitlementkey?language=objc)
pub const kIODriverKitHIDTransportEntitlementKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit.transport.hid\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkithidfamilydeviceentitlementkey?language=objc)
pub const kIODriverKitHIDFamilyDeviceEntitlementKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit.family.hid.device\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkithidfamilyeventserviceentitlementkey?language=objc)
pub const kIODriverKitHIDFamilyEventServiceEntitlementKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit.family.hid.eventservice\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodriverkittransportbuiltinentitlementkey?language=objc)
pub const kIODriverKitTransportBuiltinEntitlementKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.developer.driverkit.builtin\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvramreadaccesskey?language=objc)
pub const kIONVRAMReadAccessKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.private.iokit.nvram-read-access\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvramwriteaccesskey?language=objc)
pub const kIONVRAMWriteAccessKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.private.iokit.nvram-write-access\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioresourcessetpropertykey?language=objc)
pub const kIOResourcesSetPropertyKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.private.iokit.ioresources.setproperty\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvramsystemallowkey?language=objc)
pub const kIONVRAMSystemAllowKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.private.iokit.system-nvram-allow\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchdeferkey?language=objc)
pub const kIOMatchDeferKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMatchDefer\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioallcpuinitializedkey?language=objc)
pub const kIOAllCPUInitializedKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOAllCPUInitialized\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientclasskey?language=objc)
pub const kIOUserClientClassKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientClass\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapperidkey?language=objc)
pub const kIOMapperIDKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMapperID\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientcrossendiankey?language=objc)
pub const kIOUserClientCrossEndianKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientCrossEndian\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientcrossendiancompatiblekey?language=objc)
pub const kIOUserClientCrossEndianCompatibleKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientCrossEndianCompatible\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientsharedinstancekey?language=objc)
pub const kIOUserClientSharedInstanceKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientSharedInstance\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientdefaultlockingkey?language=objc)
pub const kIOUserClientDefaultLockingKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientDefaultLocking\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientdefaultlockingsetpropertieskey?language=objc)
pub const kIOUserClientDefaultLockingSetPropertiesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientDefaultLockingSetProperties\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientdefaultlockingsinglethreadexternalmethodkey?language=objc)
pub const kIOUserClientDefaultLockingSingleThreadExternalMethodKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"IOUserClientDefaultLockingSingleThreadExternalMethod\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserclientcreatorkey?language=objc)
pub const kIOUserClientCreatorKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserClientCreator\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouseruserclientkey?language=objc)
pub const kIOUserUserClientKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserUserClient\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserserveroneprocesskey?language=objc)
pub const kIOUserServerOneProcessKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserServerOneProcess\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserserverpreserveuserspacerebootkey?language=objc)
pub const kIOUserServerPreserveUserspaceRebootKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserServerPreserveUserspaceReboot\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopublishnotification?language=objc)
pub const kIOPublishNotification: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOServicePublish\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiofirstpublishnotification?language=objc)
pub const kIOFirstPublishNotification: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOServiceFirstPublish\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchednotification?language=objc)
pub const kIOMatchedNotification: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOServiceMatched\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiofirstmatchnotification?language=objc)
pub const kIOFirstMatchNotification: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOServiceFirstMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioterminatednotification?language=objc)
pub const kIOTerminatedNotification: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOServiceTerminate\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiowillterminatenotification?language=objc)
pub const kIOWillTerminateNotification: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOServiceWillTerminate\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiogeneralinterest?language=objc)
pub const kIOGeneralInterest: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOGeneralInterest\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobusyinterest?language=objc)
pub const kIOBusyInterest: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOBusyInterest\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioapppowerstateinterest?language=objc)
pub const kIOAppPowerStateInterest: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOAppPowerStateInterest\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioprioritypowerstateinterest?language=objc)
pub const kIOPriorityPowerStateInterest: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPriorityPowerStateInterest\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioplatformdevicemessagekey?language=objc)
pub const kIOPlatformDeviceMessageKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPlatformDeviceMessage\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocfplugintypeskey?language=objc)
pub const kIOCFPlugInTypesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOCFPlugInTypes\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocompatibilitymatchkey?language=objc)
pub const kIOCompatibilityMatchKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOCompatibilityMatch\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocompatibilitypropertieskey?language=objc)
pub const kIOCompatibilityPropertiesKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOCompatibilityProperties\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopathkey?language=objc)
pub const kIOPathKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPath\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocommandpoolsizekey?language=objc)
pub const kIOCommandPoolSizeKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOCommandPoolSize\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumprioritycountkey?language=objc)
pub const kIOMaximumPriorityCountKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumPriorityCount\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumblockcountreadkey?language=objc)
pub const kIOMaximumBlockCountReadKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumBlockCountRead\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumblockcountwritekey?language=objc)
pub const kIOMaximumBlockCountWriteKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumBlockCountWrite\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumbytecountreadkey?language=objc)
pub const kIOMaximumByteCountReadKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumByteCountRead\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumbytecountwritekey?language=objc)
pub const kIOMaximumByteCountWriteKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumByteCountWrite\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumsegmentcountreadkey?language=objc)
pub const kIOMaximumSegmentCountReadKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumSegmentCountRead\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumsegmentcountwritekey?language=objc)
pub const kIOMaximumSegmentCountWriteKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumSegmentCountWrite\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumsegmentbytecountreadkey?language=objc)
pub const kIOMaximumSegmentByteCountReadKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumSegmentByteCountRead\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumsegmentbytecountwritekey?language=objc)
pub const kIOMaximumSegmentByteCountWriteKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumSegmentByteCountWrite\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiominimumsegmentalignmentbytecountkey?language=objc)
pub const kIOMinimumSegmentAlignmentByteCountKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMinimumSegmentAlignmentByteCount\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumsegmentaddressablebitcountkey?language=objc)
pub const kIOMaximumSegmentAddressableBitCountKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumSegmentAddressableBitCount\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiominimumsaturationbytecountkey?language=objc)
pub const kIOMinimumSaturationByteCountKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMinimumSaturationByteCount\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaximumswapwritekey?language=objc)
pub const kIOMaximumSwapWriteKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOMaximumSwapWrite\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioiconkey?language=objc)
pub const kIOIconKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOIcon\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobundleresourcefilekey?language=objc)
pub const kIOBundleResourceFileKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOBundleResourceFile\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobusbadgekey?language=objc)
pub const kIOBusBadgeKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOBusBadge\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodeviceiconkey?language=objc)
pub const kIODeviceIconKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IODeviceIcon\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioplatformserialnumberkey?language=objc)
pub const kIOPlatformSerialNumberKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPlatformSerialNumber\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioplatformuuidkey?language=objc)
pub const kIOPlatformUUIDKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOPlatformUUID\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvrambootargskey?language=objc)
pub const kIONVRAMBootArgsKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"boot-args\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvramdeletepropertykey?language=objc)
pub const kIONVRAMDeletePropertyKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IONVRAM-DELETE-PROPERTY\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvramsyncnowpropertykey?language=objc)
pub const kIONVRAMSyncNowPropertyKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IONVRAM-SYNCNOW-PROPERTY\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvramactivatecsrconfigpropertykey?language=objc)
pub const kIONVRAMActivateCSRConfigPropertyKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IONVRAM-ARMCSR-PROPERTY\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodtnvrampanicinfokey?language=objc)
pub const kIODTNVRAMPanicInfoKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"aapl,panic-info\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kionvramdeletepropertykeywret?language=objc)
pub const kIONVRAMDeletePropertyKeyWRet: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IONVRAM-DELETEWRET-PROPERTY\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobootdevicekey?language=objc)
pub const kIOBootDeviceKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOBootDevice\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobootdevicepathkey?language=objc)
pub const kIOBootDevicePathKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOBootDevicePath\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobootdevicesizekey?language=objc)
pub const kIOBootDeviceSizeKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOBootDeviceSize\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kosbuildversionkey?language=objc)
pub const kOSBuildVersionKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"OS Build Version\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiostatenotificationitemcreatekey?language=objc)
pub const kIOStateNotificationItemCreateKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.statenotification.create\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiostatenotificationitemsetkey?language=objc)
pub const kIOStateNotificationItemSetKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.statenotification.set\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiostatenotificationitemcopykey?language=objc)
pub const kIOStateNotificationItemCopyKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.statenotification.copy\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiostatenotificationnamekey?language=objc)
pub const kIOStateNotificationNameKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.statenotification.name\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiostatenotificationentitlementsetkey?language=objc)
pub const kIOStateNotificationEntitlementSetKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.statenotification.entitlement-set\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiostatenotificationentitlementgetkey?language=objc)
pub const kIOStateNotificationEntitlementGetKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.statenotification.entitlement-get\0")
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstateclamshellkey?language=objc)
pub const kIOSystemStateClamshellKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.clamshell\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatesleepdescriptionkey?language=objc)
pub const kIOSystemStateSleepDescriptionKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.sleepdescription\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatesleepdescriptionreasonkey?language=objc)
pub const kIOSystemStateSleepDescriptionReasonKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.sleepreason\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatesleepdescriptionhibernatestatekey?language=objc)
pub const kIOSystemStateSleepDescriptionHibernateStateKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.hibernatestate\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatewakedescriptionkey?language=objc)
pub const kIOSystemStateWakeDescriptionKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.wakedescription\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatewakedescriptionwakereasonkey?language=objc)
pub const kIOSystemStateWakeDescriptionWakeReasonKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.wakereason\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatewakedescriptioncontinuoustimeoffsetkey?language=objc)
pub const kIOSystemStateWakeDescriptionContinuousTimeOffsetKey: &CStr = unsafe {
    CStr::from_bytes_with_nul_unchecked(
        b"com.apple.iokit.pm.wakedescription.continuous-time-offset\0",
    )
};
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatehaltdescriptionkey?language=objc)
pub const kIOSystemStateHaltDescriptionKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.haltdescription\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatehaltdescriptionhaltstatekey?language=objc)
pub const kIOSystemStateHaltDescriptionHaltStateKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.haltstate\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatepowersourcedescriptionkey?language=objc)
pub const kIOSystemStatePowerSourceDescriptionKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.powersourcedescription\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatepowersourcedescriptionacattachedkey?language=objc)
pub const kIOSystemStatePowerSourceDescriptionACAttachedKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"com.apple.iokit.pm.acattached\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobsdkey?language=objc)
pub const kIOBSDKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"IOBSD\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobsdnamekey?language=objc)
pub const kIOBSDNameKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"BSD Name\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobsdnameskey?language=objc)
pub const kIOBSDNamesKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"BSD Names\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobsdmajorkey?language=objc)
pub const kIOBSDMajorKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"BSD Major\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobsdminorkey?language=objc)
pub const kIOBSDMinorKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"BSD Minor\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiobsdunitkey?language=objc)
pub const kIOBSDUnitKey: &CStr = unsafe { CStr::from_bytes_with_nul_unchecked(b"BSD Unit\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageserviceisterminated?language=objc)
pub const kIOMessageServiceIsTerminated: c_uint = iokit_common_msg!(0x010);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageserviceissuspended?language=objc)
pub const kIOMessageServiceIsSuspended: c_uint = iokit_common_msg!(0x020);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageserviceisresumed?language=objc)
pub const kIOMessageServiceIsResumed: c_uint = iokit_common_msg!(0x030);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageserviceisrequestingclose?language=objc)
pub const kIOMessageServiceIsRequestingClose: c_uint = iokit_common_msg!(0x100);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageserviceisattemptingopen?language=objc)
pub const kIOMessageServiceIsAttemptingOpen: c_uint = iokit_common_msg!(0x101);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageservicewasclosed?language=objc)
pub const kIOMessageServiceWasClosed: c_uint = iokit_common_msg!(0x110);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageservicebusystatechange?language=objc)
pub const kIOMessageServiceBusyStateChange: c_uint = iokit_common_msg!(0x120);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageconsolesecuritychange?language=objc)
pub const kIOMessageConsoleSecurityChange: c_uint = iokit_common_msg!(0x128);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessageservicepropertychange?language=objc)
pub const kIOMessageServicePropertyChange: c_uint = iokit_common_msg!(0x130);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagecopyclientid?language=objc)
pub const kIOMessageCopyClientID: c_uint = iokit_common_msg!(0x330);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemcapabilitychange?language=objc)
pub const kIOMessageSystemCapabilityChange: c_uint = iokit_common_msg!(0x340);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagedevicesignaledwakeup?language=objc)
pub const kIOMessageDeviceSignaledWakeup: c_uint = iokit_common_msg!(0x350);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagedevicewillpoweroff?language=objc)
pub const kIOMessageDeviceWillPowerOff: c_uint = iokit_common_msg!(0x210);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagedevicehaspoweredon?language=objc)
pub const kIOMessageDeviceHasPoweredOn: c_uint = iokit_common_msg!(0x230);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemwillpoweroff?language=objc)
pub const kIOMessageSystemWillPowerOff: c_uint = iokit_common_msg!(0x250);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemwillrestart?language=objc)
pub const kIOMessageSystemWillRestart: c_uint = iokit_common_msg!(0x310);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystempagingoff?language=objc)
pub const kIOMessageSystemPagingOff: c_uint = iokit_common_msg!(0x255);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagecansystemsleep?language=objc)
pub const kIOMessageCanSystemSleep: c_uint = iokit_common_msg!(0x270);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemwillnotsleep?language=objc)
pub const kIOMessageSystemWillNotSleep: c_uint = iokit_common_msg!(0x290);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemwillsleep?language=objc)
pub const kIOMessageSystemWillSleep: c_uint = iokit_common_msg!(0x280);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemwillpoweron?language=objc)
pub const kIOMessageSystemWillPowerOn: c_uint = iokit_common_msg!(0x320);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemhaspoweredon?language=objc)
pub const kIOMessageSystemHasPoweredOn: c_uint = iokit_common_msg!(0x300);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagecandevicepoweroff?language=objc)
pub const kIOMessageCanDevicePowerOff: c_uint = iokit_common_msg!(0x200);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagedevicewillnotpoweroff?language=objc)
pub const kIOMessageDeviceWillNotPowerOff: c_uint = iokit_common_msg!(0x220);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagesystemwillnotpoweroff?language=objc)
pub const kIOMessageSystemWillNotPowerOff: c_uint = iokit_common_msg!(0x260);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagecansystempoweroff?language=objc)
pub const kIOMessageCanSystemPowerOff: c_uint = iokit_common_msg!(0x240);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagedevicewillpoweron?language=objc)
pub const kIOMessageDeviceWillPowerOn: c_uint = iokit_common_msg!(0x215);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomessagedevicehaspoweredoff?language=objc)
pub const kIOMessageDeviceHasPoweredOff: c_uint = iokit_common_msg!(0x225);
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserserverclasskey?language=objc)
pub const kIOUserServerClassKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserServer\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserservernamekey?language=objc)
pub const kIOUserServerNameKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserServerName\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiouserservertagkey?language=objc)
pub const kIOUserServerTagKey: &CStr =
    unsafe { CStr::from_bytes_with_nul_unchecked(b"IOUserServerTag\0") };
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodefaultmemorytype?language=objc)
pub const kIODefaultMemoryType: c_uint = 0;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiodefaultcache?language=objc)
pub const kIODefaultCache: c_uint = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioinhibitcache?language=objc)
pub const kIOInhibitCache: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiowritethrucache?language=objc)
pub const kIOWriteThruCache: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocopybackcache?language=objc)
pub const kIOCopybackCache: c_uint = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiowritecombinecache?language=objc)
pub const kIOWriteCombineCache: c_uint = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocopybackinnercache?language=objc)
pub const kIOCopybackInnerCache: c_uint = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopostedwrite?language=objc)
pub const kIOPostedWrite: c_uint = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorealtimecache?language=objc)
pub const kIORealTimeCache: c_uint = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopostedreordered?language=objc)
pub const kIOPostedReordered: c_uint = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiopostedcombinedreordered?language=objc)
pub const kIOPostedCombinedReordered: c_uint = 9;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapanywhere?language=objc)
pub const kIOMapAnywhere: c_uint = 0x00000001;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapcachemask?language=objc)
pub const kIOMapCacheMask: c_uint = 0x00000f00;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapcacheshift?language=objc)
pub const kIOMapCacheShift: c_uint = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapdefaultcache?language=objc)
pub const kIOMapDefaultCache: c_uint = kIODefaultCache << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapinhibitcache?language=objc)
pub const kIOMapInhibitCache: c_uint = kIOInhibitCache << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapwritethrucache?language=objc)
pub const kIOMapWriteThruCache: c_uint = kIOWriteThruCache << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapcopybackcache?language=objc)
pub const kIOMapCopybackCache: c_uint = kIOCopybackCache << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapwritecombinecache?language=objc)
pub const kIOMapWriteCombineCache: c_uint = kIOWriteCombineCache << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapcopybackinnercache?language=objc)
pub const kIOMapCopybackInnerCache: c_uint = kIOCopybackInnerCache << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomappostedwrite?language=objc)
pub const kIOMapPostedWrite: c_uint = kIOPostedWrite << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaprealtimecache?language=objc)
pub const kIOMapRealTimeCache: c_uint = kIORealTimeCache << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomappostedreordered?language=objc)
pub const kIOMapPostedReordered: c_uint = kIOPostedReordered << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomappostedcombinedreordered?language=objc)
pub const kIOMapPostedCombinedReordered: c_uint = kIOPostedCombinedReordered << kIOMapCacheShift;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapuseroptionsmask?language=objc)
pub const kIOMapUserOptionsMask: c_uint = 0x00000fff;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapreadonly?language=objc)
pub const kIOMapReadOnly: c_uint = 0x00001000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapstatic?language=objc)
pub const kIOMapStatic: c_uint = 0x01000000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapreference?language=objc)
pub const kIOMapReference: c_uint = 0x02000000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapunique?language=objc)
pub const kIOMapUnique: c_uint = 0x04000000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapprefault?language=objc)
pub const kIOMapPrefault: c_uint = 0x10000000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapoverwrite?language=objc)
pub const kIOMapOverwrite: c_uint = 0x20000000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapguardedmask?language=objc)
pub const kIOMapGuardedMask: c_uint = 0xC0000000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapguardedsmall?language=objc)
pub const kIOMapGuardedSmall: c_uint = 0x40000000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomapguardedlarge?language=objc)
pub const kIOMapGuardedLarge: c_uint = 0x80000000;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iooptionbits?language=objc)
pub type IOOptionBits = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iofixed?language=objc)
pub type IOFixed = i32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ioversion?language=objc)
pub type IOVersion = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ioitemcount?language=objc)
pub type IOItemCount = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iocachemode?language=objc)
pub type IOCacheMode = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iobytecount32?language=objc)
pub type IOByteCount32 = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iobytecount64?language=objc)
pub type IOByteCount64 = u64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iophysicaladdress32?language=objc)
pub type IOPhysicalAddress32 = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iophysicaladdress64?language=objc)
pub type IOPhysicalAddress64 = u64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iophysicallength32?language=objc)
pub type IOPhysicalLength32 = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iophysicallength64?language=objc)
pub type IOPhysicalLength64 = u64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iovirtualaddress?language=objc)
#[cfg(feature = "libc")]
pub type IOVirtualAddress = libc::mach_vm_address_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iobytecount?language=objc)
pub type IOByteCount = IOByteCount64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iologicaladdress?language=objc)
#[cfg(feature = "libc")]
pub type IOLogicalAddress = IOVirtualAddress;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iophysicaladdress?language=objc)
pub type IOPhysicalAddress = IOPhysicalAddress64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iophysicallength?language=objc)
pub type IOPhysicalLength = IOPhysicalLength64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iophysicalrange?language=objc)
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IOPhysicalRange {
    pub address: IOPhysicalAddress,
    pub length: IOByteCount,
}

#[cfg(feature = "objc2")]
unsafe impl Encode for IOPhysicalRange {
    const ENCODING: Encoding = Encoding::Struct(
        "?",
        &[<IOPhysicalAddress>::ENCODING, <IOByteCount>::ENCODING],
    );
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IOPhysicalRange {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iovirtualrange?language=objc)
#[cfg(feature = "libc")]
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IOVirtualRange {
    pub address: IOVirtualAddress,
    pub length: IOByteCount,
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl Encode for IOVirtualRange {
    const ENCODING: Encoding = Encoding::Struct(
        "?",
        &[<IOVirtualAddress>::ENCODING, <IOByteCount>::ENCODING],
    );
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl RefEncode for IOVirtualRange {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ioaddressrange?language=objc)
#[cfg(feature = "libc")]
pub type IOAddressRange = IOVirtualRange;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ionamedvalue?language=objc)
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IONamedValue {
    pub value: c_int,
    pub name: *const c_char,
}

#[cfg(feature = "objc2")]
unsafe impl Encode for IONamedValue {
    const ENCODING: Encoding =
        Encoding::Struct("?", &[<c_int>::ENCODING, <*const c_char>::ENCODING]);
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IONamedValue {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ioalignment?language=objc)
pub type IOAlignment = c_uint;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/io_object_t?language=objc)
#[cfg(feature = "libc")]
pub type io_object_t = libc::mach_port_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/io_connect_t?language=objc)
#[cfg(feature = "libc")]
pub type io_connect_t = io_object_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/io_enumerator_t?language=objc)
#[cfg(feature = "libc")]
pub type io_enumerator_t = io_object_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/io_ident_t?language=objc)
#[cfg(feature = "libc")]
pub type io_ident_t = io_object_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/io_iterator_t?language=objc)
#[cfg(feature = "libc")]
pub type io_iterator_t = io_object_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/io_registry_entry_t?language=objc)
#[cfg(feature = "libc")]
pub type io_registry_entry_t = io_object_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/io_service_t?language=objc)
#[cfg(feature = "libc")]
pub type io_service_t = io_object_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/uext_object_t?language=objc)
#[cfg(feature = "libc")]
pub type uext_object_t = io_object_t;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/knanosecondscale?language=objc)
pub const kNanosecondScale: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kmicrosecondscale?language=objc)
pub const kMicrosecondScale: c_uint = 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kmillisecondscale?language=objc)
pub const kMillisecondScale: c_uint = 1000 * 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ksecondscale?language=objc)
pub const kSecondScale: c_uint = 1000 * 1000 * 1000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ktickscale?language=objc)
pub const kTickScale: c_uint = kSecondScale / 100;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioconnectmethodvaroutputsize?language=objc)
pub const kIOConnectMethodVarOutputSize: c_int = -3;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iodevicenumber?language=objc)
pub type IODeviceNumber = c_uint;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaxbusstall40usec?language=objc)
pub const kIOMaxBusStall40usec: c_uint = 40000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaxbusstall30usec?language=objc)
pub const kIOMaxBusStall30usec: c_uint = 30000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaxbusstall25usec?language=objc)
pub const kIOMaxBusStall25usec: c_uint = 25000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaxbusstall20usec?language=objc)
pub const kIOMaxBusStall20usec: c_uint = 20000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaxbusstall10usec?language=objc)
pub const kIOMaxBusStall10usec: c_uint = 10000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaxbusstall5usec?language=objc)
pub const kIOMaxBusStall5usec: c_uint = 5000;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomaxbusstallnone?language=objc)
pub const kIOMaxBusStallNone: c_uint = 0;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatesleepdescriptionhibernatestateinactive?language=objc)
pub const kIOSystemStateSleepDescriptionHibernateStateInactive: c_uint = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatesleepdescriptionhibernatestatehibernating?language=objc)
pub const kIOSystemStateSleepDescriptionHibernateStateHibernating: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiosystemstatesleepdescriptionhibernatestatewakingfromhibernate?language=objc)
pub const kIOSystemStateSleepDescriptionHibernateStateWakingFromHibernate: c_uint = 2;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kfirstiokitnotificationtype?language=objc)
pub const kFirstIOKitNotificationType: c_uint = 100;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioservicepublishnotificationtype?language=objc)
pub const kIOServicePublishNotificationType: c_uint = 100;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioservicematchednotificationtype?language=objc)
pub const kIOServiceMatchedNotificationType: c_uint = 101;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserviceterminatednotificationtype?language=objc)
pub const kIOServiceTerminatedNotificationType: c_uint = 102;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioasynccompletionnotificationtype?language=objc)
pub const kIOAsyncCompletionNotificationType: c_uint = 150;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioservicemessagenotificationtype?language=objc)
pub const kIOServiceMessageNotificationType: c_uint = 160;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/klastiokitnotificationtype?language=objc)
pub const kLastIOKitNotificationType: c_uint = 199;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiokitnoticationtypemask?language=objc)
pub const kIOKitNoticationTypeMask: c_uint = 0x00000FFF;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiokitnoticationtypesizeadjshift?language=objc)
pub const kIOKitNoticationTypeSizeAdjShift: c_uint = 30;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiokitnoticationmsgsizemask?language=objc)
pub const kIOKitNoticationMsgSizeMask: c_uint = 3;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kosnotificationmessageid?language=objc)
pub const kOSNotificationMessageID: c_uint = 53;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kosasynccompletemessageid?language=objc)
pub const kOSAsyncCompleteMessageID: c_uint = 57;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kmaxasyncargs?language=objc)
pub const kMaxAsyncArgs: c_uint = 16;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioasyncreservedindex?language=objc)
pub const kIOAsyncReservedIndex: c_uint = 0;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioasyncreservedcount?language=objc)
pub const kIOAsyncReservedCount: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioasynccalloutfuncindex?language=objc)
pub const kIOAsyncCalloutFuncIndex: c_uint = kIOAsyncReservedCount;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioasynccalloutrefconindex?language=objc)
pub const kIOAsyncCalloutRefconIndex: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioasynccalloutcount?language=objc)
pub const kIOAsyncCalloutCount: c_uint = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchingcalloutfuncindex?language=objc)
pub const kIOMatchingCalloutFuncIndex: c_uint = kIOAsyncReservedCount;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchingcalloutrefconindex?language=objc)
pub const kIOMatchingCalloutRefconIndex: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomatchingcalloutcount?language=objc)
pub const kIOMatchingCalloutCount: c_uint = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiointerestcalloutfuncindex?language=objc)
pub const kIOInterestCalloutFuncIndex: c_uint = kIOAsyncReservedCount;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiointerestcalloutrefconindex?language=objc)
pub const kIOInterestCalloutRefconIndex: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiointerestcalloutserviceindex?language=objc)
pub const kIOInterestCalloutServiceIndex: c_uint = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiointerestcalloutcount?language=objc)
pub const kIOInterestCalloutCount: c_uint = 4;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kosasyncrefcount?language=objc)
pub const kOSAsyncRefCount: c_uint = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kosasyncrefsize?language=objc)
pub const kOSAsyncRefSize: c_uint = 32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ioserviceinterestcontent?language=objc)
#[cfg(feature = "libc")]
#[repr(C, packed(4))]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IOServiceInterestContent {
    pub messageType: libc::natural_t,
    pub messageArgument: [*mut c_void; 1],
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl Encode for IOServiceInterestContent {
    const ENCODING: Encoding = Encoding::Struct(
        "IOServiceInterestContent",
        &[<libc::natural_t>::ENCODING, <[*mut c_void; 1]>::ENCODING],
    );
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl RefEncode for IOServiceInterestContent {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ioasynccompletioncontent?language=objc)
#[repr(C, packed(4))]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IOAsyncCompletionContent {
    pub result: IOReturn,
    pub args: *mut *mut c_void,
}

#[cfg(feature = "objc2")]
unsafe impl Encode for IOAsyncCompletionContent {
    const ENCODING: Encoding = Encoding::Struct(
        "IOAsyncCompletionContent",
        &[<IOReturn>::ENCODING, <*mut *mut c_void>::ENCODING],
    );
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IOAsyncCompletionContent {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/ionotificationport?language=objc)
#[repr(C)]
#[derive(Debug)]
pub struct IONotificationPort {
    inner: [u8; 0],
    _p: UnsafeCell<PhantomData<(*const UnsafeCell<()>, PhantomPinned)>>,
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IONotificationPort {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Encoding::Struct("IONotificationPort", &[]));
}

/// IOKitLib
/// IOKitLib implements non-kernel task access to common IOKit object types - IORegistryEntry, IOService, IOIterator etc. These functions are generic - families may provide API that is more specific.
/// <br>
/// IOKitLib represents IOKit objects outside the kernel with the types io_object_t, io_registry_entry_t, io_service_t,
/// &
/// io_connect_t. Function names usually begin with the type of object they are compatible with - eg. IOObjectRelease can be used with any io_object_t. Inside the kernel, the c++ class hierarchy allows the subclasses of each object type to receive the same requests from user level clients, for example in the kernel, IOService is a subclass of IORegistryEntry, which means any of the IORegistryEntryXXX functions in IOKitLib may be used with io_service_t's as well as io_registry_t's. There are functions available to introspect the class of the kernel object which any io_object_t et al. represents.
/// IOKit objects returned by all functions should be released with IOObjectRelease.
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/ionotificationportref?language=objc)
pub type IONotificationPortRef = *mut IONotificationPort;

/// Callback function to be notified of IOService publication.
///
/// Parameter `refcon`: The refcon passed when the notification was installed.
///
/// Parameter `iterator`: The notification iterator which now has new objects.
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/ioservicematchingcallback?language=objc)
#[cfg(feature = "libc")]
pub type IOServiceMatchingCallback =
    Option<unsafe extern "C-unwind" fn(*mut c_void, io_iterator_t)>;

/// Callback function to be notified of changes in state of an IOService.
///
/// Parameter `refcon`: The refcon passed when the notification was installed.
///
/// Parameter `service`: The IOService whose state has changed.
///
/// Parameter `messageType`: A messageType enum, defined by IOKit/IOMessage.h or by the IOService's family.
///
/// Parameter `messageArgument`: An argument for the message, dependent on the messageType.  If the message data is larger than sizeof(void*), then messageArgument contains a pointer to the message data; otherwise, messageArgument contains the message data.
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/ioserviceinterestcallback?language=objc)
#[cfg(feature = "libc")]
pub type IOServiceInterestCallback =
    Option<unsafe extern "C-unwind" fn(*mut c_void, io_service_t, u32, *mut c_void)>;

extern "C" {
    /// The default mach port used to initiate communication with IOKit.
    ///
    /// When specifying a main port to IOKit functions, the NULL argument indicates "use the default". This is a synonym for NULL, if you'd rather use a named constant.
    ///
    /// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomainportdefault?language=objc)
    #[cfg(feature = "libc")]
    pub static kIOMainPortDefault: libc::mach_port_t;
}

extern "C-unwind" {
    /// Returns the mach port used to initiate communication with IOKit.
    ///
    /// Functions that don't specify an existing object require the IOKit main port to be passed. This function obtains that port.
    ///
    /// Parameter `bootstrapPort`: Pass MACH_PORT_NULL for the default.
    ///
    /// Parameter `mainPort`: The main port is returned.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `main_port` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOMainPort(
        bootstrap_port: libc::mach_port_t,
        main_port: *mut libc::mach_port_t,
    ) -> libc::kern_return_t;
}

extern "C" {
    /// Deprecated name for kIOMainPortDefault.
    ///
    /// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/kiomasterportdefault?language=objc)
    #[cfg(feature = "libc")]
    #[deprecated]
    pub static kIOMasterPortDefault: libc::mach_port_t;
}

extern "C-unwind" {
    /// Deprecated name for IOMainPort().
    ///
    /// # Safety
    ///
    /// `main_port` must be a valid pointer.
    #[cfg(feature = "libc")]
    #[deprecated]
    pub fn IOMasterPort(
        bootstrap_port: libc::mach_port_t,
        main_port: *mut libc::mach_port_t,
    ) -> libc::kern_return_t;
}

impl IONotificationPort {
    /// Creates and returns a notification object for receiving IOKit notifications of new devices or state changes.
    ///
    /// Creates the notification object to receive notifications from IOKit of new device arrivals or state changes. The notification object can be supply a CFRunLoopSource, or mach_port_t to be used to listen for events.
    ///
    /// Parameter `mainPort`: The main port obtained from IOMainPort(). Pass kIOMainPortDefault to look up the default main port.
    ///
    /// Returns: A reference to the notification object.
    #[doc(alias = "IONotificationPortCreate")]
    #[cfg(feature = "libc")]
    #[inline]
    pub fn create(main_port: libc::mach_port_t) -> IONotificationPortRef {
        extern "C-unwind" {
            fn IONotificationPortCreate(main_port: libc::mach_port_t) -> IONotificationPortRef;
        }
        unsafe { IONotificationPortCreate(main_port) }
    }

    /// Destroys a notification object created with IONotificationPortCreate.
    /// Also destroys any mach_port's or CFRunLoopSources obtained from
    /// <code>
    ///
    /// ```text
    ///  IONotificationPortGetRunLoopSource
    /// ```
    ///
    /// </code>
    /// or
    /// <code>
    ///
    /// ```text
    ///  IONotificationPortGetMachPort
    /// ```
    ///
    /// </code>
    ///
    /// Parameter `notify`: A reference to the notification object.
    ///
    /// # Safety
    ///
    /// `notify` must be a valid pointer.
    #[doc(alias = "IONotificationPortDestroy")]
    #[inline]
    pub unsafe fn destroy(notify: IONotificationPortRef) {
        extern "C-unwind" {
            fn IONotificationPortDestroy(notify: IONotificationPortRef);
        }
        unsafe { IONotificationPortDestroy(notify) }
    }

    /// Returns a CFRunLoopSource to be used to listen for notifications.
    ///
    /// A notification object may deliver notifications to a CFRunLoop
    /// by adding the run loop source returned by this function to the run loop.
    ///
    /// The caller should not release this CFRunLoopSource. Just call
    /// <code>
    ///
    /// ```text
    ///  IONotificationPortDestroy
    /// ```
    ///
    /// </code>
    /// to dispose of the
    /// IONotificationPortRef and the CFRunLoopSource when done.
    ///
    /// Parameter `notify`: The notification object.
    ///
    /// Returns: A CFRunLoopSourceRef for the notification object.
    ///
    /// # Safety
    ///
    /// `notify` must be a valid pointer.
    #[doc(alias = "IONotificationPortGetRunLoopSource")]
    #[inline]
    pub unsafe fn run_loop_source(
        notify: IONotificationPortRef,
    ) -> Option<CFRetained<CFRunLoopSource>> {
        extern "C-unwind" {
            fn IONotificationPortGetRunLoopSource(
                notify: IONotificationPortRef,
            ) -> Option<NonNull<CFRunLoopSource>>;
        }
        let ret = unsafe { IONotificationPortGetRunLoopSource(notify) };
        ret.map(|ret| unsafe { CFRetained::retain(ret) })
    }

    /// Returns a mach_port to be used to listen for notifications.
    ///
    /// A notification object may deliver notifications to a mach messaging client
    /// if they listen for messages on the port obtained from this function.
    /// Callbacks associated with the notifications may be delivered by calling
    /// IODispatchCalloutFromMessage with messages received.
    ///
    /// The caller should not release this mach_port_t. Just call
    /// <code>
    ///
    /// ```text
    ///  IONotificationPortDestroy
    /// ```
    ///
    /// </code>
    /// to dispose of the
    /// mach_port_t and IONotificationPortRef when done.
    ///
    /// Parameter `notify`: The notification object.
    ///
    /// Returns: A mach_port for the notification object.
    ///
    /// # Safety
    ///
    /// `notify` must be a valid pointer.
    #[doc(alias = "IONotificationPortGetMachPort")]
    #[cfg(feature = "libc")]
    #[inline]
    pub unsafe fn mach_port(notify: IONotificationPortRef) -> libc::mach_port_t {
        extern "C-unwind" {
            fn IONotificationPortGetMachPort(notify: IONotificationPortRef) -> libc::mach_port_t;
        }
        unsafe { IONotificationPortGetMachPort(notify) }
    }

    /// Configure a notification port to be an importance receiver.
    ///
    /// Sets the MACH_PORT_IMPORTANCE_RECEIVER attribute on the underlying mach port.
    /// Importance-donating messages sent to a notification port with this
    /// attribute enabled will boost the importance of the receiving process for the
    /// duration of the notification handler.
    ///
    /// Parameter `notify`: The notification object.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `notify` must be a valid pointer.
    #[doc(alias = "IONotificationPortSetImportanceReceiver")]
    #[cfg(feature = "libc")]
    #[inline]
    pub unsafe fn set_importance_receiver(notify: IONotificationPortRef) -> libc::kern_return_t {
        extern "C-unwind" {
            fn IONotificationPortSetImportanceReceiver(
                notify: IONotificationPortRef,
            ) -> libc::kern_return_t;
        }
        unsafe { IONotificationPortSetImportanceReceiver(notify) }
    }

    /// Sets a dispatch queue to be used to listen for notifications.
    ///
    /// A notification object may deliver notifications to a dispatch client.
    ///
    /// Parameter `notify`: The notification object.
    ///
    /// Parameter `queue`: A dispatch queue.
    ///
    /// # Safety
    ///
    /// - `notify` must be a valid pointer.
    /// - `queue` possibly has additional threading requirements.
    /// - `queue` might not allow `None`.
    #[doc(alias = "IONotificationPortSetDispatchQueue")]
    #[cfg(feature = "dispatch2")]
    #[inline]
    pub unsafe fn set_dispatch_queue(notify: IONotificationPortRef, queue: Option<&DispatchQueue>) {
        extern "C-unwind" {
            fn IONotificationPortSetDispatchQueue(
                notify: IONotificationPortRef,
                queue: Option<&DispatchQueue>,
            );
        }
        unsafe { IONotificationPortSetDispatchQueue(notify, queue) }
    }
}

extern "C-unwind" {
    /// Creates and returns a mach port suitable for receiving IOKit messages of the specified type.
    ///
    /// In the future IOKit may use specialized messages and ports
    /// instead of the standard ports created by mach_port_allocate(). Use this
    /// function instead of mach_port_allocate() to ensure compatibility with future
    /// revisions of IOKit.
    ///
    /// Parameter `msgType`: Type of message to be sent to this port
    /// (kOSNotificationMessageID or kOSAsyncCompleteMessageID)
    ///
    /// Parameter `recvPort`: The created port is returned.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `recv_port` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOCreateReceivePort(
        msg_type: u32,
        recv_port: *mut libc::mach_port_t,
    ) -> libc::kern_return_t;
}

/// Releases an object handle previously returned by IOKitLib.
///
/// All objects returned by IOKitLib should be released with this function when access to them is no longer needed. Using the object after it has been released may or may not return an error, depending on how many references the task has to the same object in the kernel.
///
/// Parameter `object`: The IOKit object to release.
///
/// Returns: A kern_return_t error code.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOObjectRelease(object: io_object_t) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOObjectRelease(object: io_object_t) -> libc::kern_return_t;
    }
    unsafe { IOObjectRelease(object) }
}

/// Retains an object handle previously returned by IOKitLib.
///
/// Gives the caller an additional reference to an existing object handle previously returned by IOKitLib.
///
/// Parameter `object`: The IOKit object to retain.
///
/// Returns: A kern_return_t error code.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOObjectRetain(object: io_object_t) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOObjectRetain(object: io_object_t) -> libc::kern_return_t;
    }
    unsafe { IOObjectRetain(object) }
}

extern "C-unwind" {
    /// Return the class name of an IOKit object.
    ///
    /// This function uses the OSMetaClass system in the kernel to derive the name of the class the object is an instance of.
    ///
    /// Parameter `object`: The IOKit object.
    ///
    /// Parameter `className`: Caller allocated buffer to receive the name string.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `class_name` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IOObjectGetClass(object: io_object_t, class_name: io_name_t) -> libc::kern_return_t;
}

/// Return the class name of an IOKit object.
///
/// This function does the same thing as IOObjectGetClass, but returns the result as a CFStringRef.
///
/// Parameter `object`: The IOKit object.
///
/// Returns: The resulting CFStringRef. This should be released by the caller. If a valid object is not passed in, then NULL is returned.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOObjectCopyClass(object: io_object_t) -> Option<CFRetained<CFString>> {
    extern "C-unwind" {
        fn IOObjectCopyClass(object: io_object_t) -> Option<NonNull<CFString>>;
    }
    let ret = unsafe { IOObjectCopyClass(object) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// Return the superclass name of the given class.
///
/// This function uses the OSMetaClass system in the kernel to derive the name of the superclass of the class.
///
/// Parameter `classname`: The name of the class as a CFString.
///
/// Returns: The resulting CFStringRef. This should be released by the caller. If there is no superclass, or a valid class name is not passed in, then NULL is returned.
///
/// # Safety
///
/// `classname` might not allow `None`.
#[inline]
pub unsafe extern "C-unwind" fn IOObjectCopySuperclassForClass(
    classname: Option<&CFString>,
) -> Option<CFRetained<CFString>> {
    extern "C-unwind" {
        fn IOObjectCopySuperclassForClass(
            classname: Option<&CFString>,
        ) -> Option<NonNull<CFString>>;
    }
    let ret = unsafe { IOObjectCopySuperclassForClass(classname) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// Return the bundle identifier of the given class.
///
/// This function uses the OSMetaClass system in the kernel to derive the name of the kmod, which is the same as the bundle identifier.
///
/// Parameter `classname`: The name of the class as a CFString.
///
/// Returns: The resulting CFStringRef. This should be released by the caller. If a valid class name is not passed in, then NULL is returned.
///
/// # Safety
///
/// `classname` might not allow `None`.
#[inline]
pub unsafe extern "C-unwind" fn IOObjectCopyBundleIdentifierForClass(
    classname: Option<&CFString>,
) -> Option<CFRetained<CFString>> {
    extern "C-unwind" {
        fn IOObjectCopyBundleIdentifierForClass(
            classname: Option<&CFString>,
        ) -> Option<NonNull<CFString>>;
    }
    let ret = unsafe { IOObjectCopyBundleIdentifierForClass(classname) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// Performs an OSDynamicCast operation on an IOKit object.
///
/// This function uses the OSMetaClass system in the kernel to determine if the object will dynamic cast to a class, specified as a C-string. In other words, if the object is of that class or a subclass.
///
/// Parameter `object`: An IOKit object.
///
/// Parameter `className`: The name of the class, as a C-string.
///
/// Returns: If the object handle is valid, and represents an object in the kernel that dynamic casts to the class true is returned, otherwise false.
///
/// # Safety
///
/// `class_name` Array TODO.
#[cfg(feature = "libc")]
#[inline]
pub unsafe extern "C-unwind" fn IOObjectConformsTo(
    object: io_object_t,
    class_name: io_name_t,
) -> bool {
    extern "C-unwind" {
        fn IOObjectConformsTo(object: io_object_t, class_name: io_name_t) -> libc::boolean_t;
    }
    let ret = unsafe { IOObjectConformsTo(object, class_name) };
    ret != 0
}

/// Checks two object handles to see if they represent the same kernel object.
///
/// If two object handles are returned by IOKitLib functions, this function will compare them to see if they represent the same kernel object.
///
/// Parameter `object`: An IOKit object.
///
/// Parameter `anObject`: Another IOKit object.
///
/// Returns: If both object handles are valid, and represent the same object in the kernel true is returned, otherwise false.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOObjectIsEqualTo(object: io_object_t, an_object: io_object_t) -> bool {
    extern "C-unwind" {
        fn IOObjectIsEqualTo(object: io_object_t, an_object: io_object_t) -> libc::boolean_t;
    }
    let ret = unsafe { IOObjectIsEqualTo(object, an_object) };
    ret != 0
}

/// Returns kernel retain count of an IOKit object.
///
/// This function may be used in diagnostics to determine the current retain count of the kernel object at the kernel level.
///
/// Parameter `object`: An IOKit object.
///
/// Returns: If the object handle is valid, the kernel objects retain count is returned, otherwise zero is returned.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOObjectGetKernelRetainCount(object: io_object_t) -> u32 {
    extern "C-unwind" {
        fn IOObjectGetKernelRetainCount(object: io_object_t) -> u32;
    }
    unsafe { IOObjectGetKernelRetainCount(object) }
}

/// Returns the retain count for the current process of an IOKit object.
///
/// This function may be used in diagnostics to determine the current retain count for the calling process of the kernel object.
///
/// Parameter `object`: An IOKit object.
///
/// Returns: If the object handle is valid, the objects user retain count is returned, otherwise zero is returned.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOObjectGetUserRetainCount(object: io_object_t) -> u32 {
    extern "C-unwind" {
        fn IOObjectGetUserRetainCount(object: io_object_t) -> u32;
    }
    unsafe { IOObjectGetUserRetainCount(object) }
}

/// Returns kernel retain count of an IOKit object. Identical to IOObjectGetKernelRetainCount() but available prior to Mac OS 10.6.
///
/// This function may be used in diagnostics to determine the current retain count of the kernel object at the kernel level.
///
/// Parameter `object`: An IOKit object.
///
/// Returns: If the object handle is valid, the kernel objects retain count is returned, otherwise zero is returned.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOObjectGetRetainCount(object: io_object_t) -> u32 {
    extern "C-unwind" {
        fn IOObjectGetRetainCount(object: io_object_t) -> u32;
    }
    unsafe { IOObjectGetRetainCount(object) }
}

/// Returns the next object in an iteration.
///
/// This function returns the next object in an iteration, or zero if no more remain or the iterator is invalid.
///
/// Parameter `iterator`: An IOKit iterator handle.
///
/// Returns: If the iterator handle is valid, the next element in the iteration is returned, otherwise zero is returned. The element should be released by the caller when it is finished.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOIteratorNext(iterator: io_iterator_t) -> io_object_t {
    extern "C-unwind" {
        fn IOIteratorNext(iterator: io_iterator_t) -> io_object_t;
    }
    unsafe { IOIteratorNext(iterator) }
}

/// Resets an iteration back to the beginning.
///
/// If an iterator is invalid, or if the caller wants to start over, IOIteratorReset will set the iteration back to the beginning.
///
/// Parameter `iterator`: An IOKit iterator handle.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOIteratorReset(iterator: io_iterator_t) {
    extern "C-unwind" {
        fn IOIteratorReset(iterator: io_iterator_t);
    }
    unsafe { IOIteratorReset(iterator) }
}

/// Checks an iterator is still valid.
///
/// Some iterators will be made invalid if changes are made to the structure they are iterating over. This function checks the iterator is still valid and should be called when IOIteratorNext returns zero. An invalid iterator can be reset and the iteration restarted.
///
/// Parameter `iterator`: An IOKit iterator handle.
///
/// Returns: True if the iterator handle is valid, otherwise false is returned.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOIteratorIsValid(iterator: io_iterator_t) -> bool {
    extern "C-unwind" {
        fn IOIteratorIsValid(iterator: io_iterator_t) -> libc::boolean_t;
    }
    let ret = unsafe { IOIteratorIsValid(iterator) };
    ret != 0
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `notification_type` Array TODO.
    /// - `matching` generics must be of the correct type.
    /// - `matching` might not allow `None`.
    /// - `notification` must be a valid pointer.
    #[cfg(feature = "libc")]
    #[deprecated]
    pub fn IOServiceAddNotification(
        main_port: libc::mach_port_t,
        notification_type: io_name_t,
        matching: Option<&CFDictionary>,
        wake_port: libc::mach_port_t,
        reference: usize,
        notification: *mut io_iterator_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Register for notification of state changes in an IOService.
    ///
    /// IOService objects deliver notifications of their state changes to their clients via the IOService::messageClients API, and to other interested parties including callers of this function. Message types are defined IOKit/IOMessage.h.
    ///
    /// Parameter `notifyPort`: A IONotificationPortRef object that controls how messages will be sent when the notification is fired. See IONotificationPortCreate.
    ///
    /// Parameter `interestType`: A notification type from IOKitKeys.h
    /// <br>
    /// kIOGeneralInterest General state changes delivered via the IOService::messageClients API.
    /// <br>
    /// kIOBusyInterest Delivered when the IOService changes its busy state to or from zero. The message argument contains the new busy state causing the notification.
    ///
    /// Parameter `callback`: A callback function called when the notification fires, with messageType and messageArgument for the state change.
    ///
    /// Parameter `refCon`: A reference constant for the callbacks use.
    ///
    /// Parameter `notification`: An object handle is returned on success, and should be released by the caller when the notification is to be destroyed.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `notify_port` must be a valid pointer.
    /// - `interest_type` Array TODO.
    /// - `callback` must be implemented correctly.
    /// - `ref_con` must be a valid pointer.
    /// - `notification` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOServiceAddInterestNotification(
        notify_port: IONotificationPortRef,
        service: io_service_t,
        interest_type: io_name_t,
        callback: IOServiceInterestCallback,
        ref_con: *mut c_void,
        notification: *mut io_object_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Match an IOService objects with matching dictionary.
    ///
    /// This function calls the matching method of an IOService object and returns the boolean result.
    ///
    /// Parameter `service`: The IOService object to match.
    ///
    /// Parameter `matching`: A CF dictionary containing matching information. IOKitLib can construct matching dictionaries for common criteria with helper functions such as IOServiceMatching, IOServiceNameMatching, IOBSDNameMatching.
    ///
    /// Parameter `matches`: The boolean result is returned.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `matching` generics must be of the correct type.
    /// - `matching` might not allow `None`.
    /// - `matches` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOServiceMatchPropertyTable(
        service: io_service_t,
        matching: Option<&CFDictionary>,
        matches: *mut libc::boolean_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns the busyState of an IOService.
    ///
    /// Many activities in IOService are asynchronous. When registration, matching, or termination is in progress on an IOService, its busyState is increased by one. Change in busyState to or from zero also changes the IOService's provider's busyState by one, which means that an IOService is marked busy when any of the above activities is occurring on it or any of its clients.
    ///
    /// Parameter `service`: The IOService whose busyState to return.
    ///
    /// Parameter `busyState`: The busyState count is returned.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `busy_state` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOServiceGetBusyState(
        service: io_service_t,
        busy_state: *mut u32,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns the busyState of all IOServices.
    ///
    /// Many activities in IOService are asynchronous. When registration, matching, or termination is in progress on an IOService, its busyState is increased by one. Change in busyState to or from zero also changes the IOService's provider's busyState by one, which means that an IOService is marked busy when any of the above activities is occurring on it or any of its clients. IOKitGetBusyState returns the busy state of the root of the service plane which reflects the busy state of all IOServices.
    ///
    /// Parameter `mainPort`: The main port obtained from IOMainPort(). Pass kIOMainPortDefault to look up the default main port.
    ///
    /// Parameter `busyState`: The busyState count is returned.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `busy_state` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOKitGetBusyState(
        main_port: libc::mach_port_t,
        busy_state: *mut u32,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// A request to create a connection to an IOService.
    ///
    /// A non kernel client may request a connection be opened via the IOServiceOpen() library function, which will call IOService::newUserClient in the kernel. The rules
    /// &
    /// capabilities of user level clients are family dependent, the default IOService implementation returns kIOReturnUnsupported.
    ///
    /// Parameter `service`: The IOService object to open a connection to, usually obtained via the IOServiceGetMatchingServices or IOServiceAddNotification APIs.
    ///
    /// Parameter `owningTask`: The mach task requesting the connection.
    ///
    /// Parameter `type`: A constant specifying the type of connection to be created,  interpreted only by the IOService's family.
    ///
    /// Parameter `connect`: An io_connect_t handle is returned on success, to be used with the IOConnectXXX APIs. It should be destroyed with IOServiceClose().
    ///
    /// Returns: A return code generated by IOService::newUserClient.
    ///
    /// # Safety
    ///
    /// `connect` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOServiceOpen(
        service: io_service_t,
        owning_task: task_port_t,
        r#type: u32,
        connect: *mut io_connect_t,
    ) -> libc::kern_return_t;
}

/// A request to rescan a bus for device changes.
///
/// A non kernel client may request a bus or controller rescan for added or removed devices, if the bus family does automatically notice such changes. For example, SCSI bus controllers do not notice device changes. The implementation of this routine is family dependent, and the default IOService implementation returns kIOReturnUnsupported.
///
/// Parameter `service`: The IOService object to request a rescan, usually obtained via the IOServiceGetMatchingServices or IOServiceAddNotification APIs.
///
/// Parameter `options`: An options mask, interpreted only by the IOService's family.
///
/// Returns: A return code generated by IOService::requestProbe.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOServiceRequestProbe(
    service: io_service_t,
    options: u32,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOServiceRequestProbe(service: io_service_t, options: u32) -> libc::kern_return_t;
    }
    unsafe { IOServiceRequestProbe(service, options) }
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioserviceinteractionallowed?language=objc)
pub const kIOServiceInteractionAllowed: c_uint = 0x00000001;

/// Authorize access to an IOService.
///
/// Determine whether this application is authorized to invoke IOServiceOpen() for a given IOService, either by confirming that it has been previously authorized by the user, or by soliciting the console user.
///
/// Parameter `service`: The IOService object to be authorized, usually obtained via the IOServiceGetMatchingServices or IOServiceAddNotification APIs.
///
/// Parameter `options`: kIOServiceInteractionAllowed may be set to permit user interaction, if required.
///
/// Returns: kIOReturnSuccess if the IOService is authorized, kIOReturnNotPermitted if the IOService is not authorized.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOServiceAuthorize(
    service: io_service_t,
    options: u32,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOServiceAuthorize(service: io_service_t, options: u32) -> libc::kern_return_t;
    }
    unsafe { IOServiceAuthorize(service, options) }
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOServiceOpenAsFileDescriptor(
    service: io_service_t,
    oflag: c_int,
) -> c_int {
    extern "C-unwind" {
        fn IOServiceOpenAsFileDescriptor(service: io_service_t, oflag: c_int) -> c_int;
    }
    unsafe { IOServiceOpenAsFileDescriptor(service, oflag) }
}

/// Close a connection to an IOService and destroy the connect handle.
///
/// A connection created with the IOServiceOpen should be closed when the connection is no longer to be used with IOServiceClose.
///
/// Parameter `connect`: The connect handle created by IOServiceOpen. It will be destroyed by this function, and should not be released with IOObjectRelease.
///
/// Returns: A kern_return_t error code.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOServiceClose(connect: io_connect_t) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOServiceClose(connect: io_connect_t) -> libc::kern_return_t;
    }
    unsafe { IOServiceClose(connect) }
}

/// Adds a reference to the connect handle.
///
/// Adds a reference to the connect handle.
///
/// Parameter `connect`: The connect handle created by IOServiceOpen.
///
/// Returns: A kern_return_t error code.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectAddRef(connect: io_connect_t) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectAddRef(connect: io_connect_t) -> libc::kern_return_t;
    }
    unsafe { IOConnectAddRef(connect) }
}

/// Remove a reference to the connect handle.
///
/// Removes a reference to the connect handle.  If the last reference is removed an implicit IOServiceClose is performed.
///
/// Parameter `connect`: The connect handle created by IOServiceOpen.
///
/// Returns: A kern_return_t error code.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectRelease(connect: io_connect_t) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectRelease(connect: io_connect_t) -> libc::kern_return_t;
    }
    unsafe { IOConnectRelease(connect) }
}

extern "C-unwind" {
    /// Returns the IOService a connect handle was opened on.
    ///
    /// Finds the service object a connection was opened on.
    ///
    /// Parameter `connect`: The connect handle created by IOServiceOpen.
    ///
    /// Parameter `service`: On success, the service handle the connection was opened on, which should be released with IOObjectRelease.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `service` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectGetService(
        connect: io_connect_t,
        service: *mut io_service_t,
    ) -> libc::kern_return_t;
}

/// Set a port to receive family specific notifications.
///
/// This is a generic method to pass a mach port send right to be be used by family specific notifications.
///
/// Parameter `connect`: The connect handle created by IOServiceOpen.
///
/// Parameter `type`: The type of notification requested, not interpreted by IOKit and family defined.
///
/// Parameter `port`: The port to which to send notifications.
///
/// Parameter `reference`: Some families may support passing a reference parameter for the callers use with the notification.
///
/// Returns: A kern_return_t error code.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectSetNotificationPort(
    connect: io_connect_t,
    r#type: u32,
    port: libc::mach_port_t,
    reference: usize,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectSetNotificationPort(
            connect: io_connect_t,
            r#type: u32,
            port: libc::mach_port_t,
            reference: usize,
        ) -> libc::kern_return_t;
    }
    unsafe { IOConnectSetNotificationPort(connect, r#type, port, reference) }
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `at_address` must be a valid pointer.
    /// - `of_size` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectMapMemory(
        connect: io_connect_t,
        memory_type: u32,
        into_task: task_port_t,
        at_address: *mut libc::mach_vm_address_t,
        of_size: *mut libc::mach_vm_size_t,
        options: IOOptionBits,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Map hardware or shared memory into the caller's task.
    ///
    /// This is a generic method to create a mapping in the callers task. The family will interpret the type parameter to determine what sort of mapping is being requested. Cache modes and placed mappings may be requested by the caller.
    ///
    /// Parameter `connect`: The connect handle created by IOServiceOpen.
    ///
    /// Parameter `memoryType`: What is being requested to be mapped, not interpreted by IOKit and family defined. The family may support physical hardware or shared memory mappings.
    ///
    /// Parameter `intoTask`: The task port for the task in which to create the mapping. This may be different to the task which the opened the connection.
    ///
    /// Parameter `atAddress`: An in/out parameter - if the kIOMapAnywhere option is not set, the caller should pass the address where it requests the mapping be created, otherwise nothing need to set on input. The address of the mapping created is passed back on success.
    ///
    /// Parameter `ofSize`: The size of the mapping created is passed back on success.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `at_address` must be a valid pointer.
    /// - `of_size` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectMapMemory64(
        connect: io_connect_t,
        memory_type: u32,
        into_task: task_port_t,
        at_address: *mut libc::mach_vm_address_t,
        of_size: *mut libc::mach_vm_size_t,
        options: IOOptionBits,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    #[cfg(feature = "libc")]
    pub fn IOConnectUnmapMemory(
        connect: io_connect_t,
        memory_type: u32,
        from_task: task_port_t,
        at_address: libc::mach_vm_address_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Remove a mapping made with IOConnectMapMemory64.
    ///
    /// This is a generic method to remove a mapping in the callers task.
    ///
    /// Parameter `connect`: The connect handle created by IOServiceOpen.
    ///
    /// Parameter `memoryType`: The memory type originally requested in IOConnectMapMemory.
    ///
    /// Parameter `fromTask`: The task port for the task in which to remove the mapping. This may be different to the task which the opened the connection.
    ///
    /// Parameter `atAddress`: The address of the mapping to be removed.
    ///
    /// Returns: A kern_return_t error code.
    #[cfg(feature = "libc")]
    pub fn IOConnectUnmapMemory64(
        connect: io_connect_t,
        memory_type: u32,
        from_task: task_port_t,
        at_address: libc::mach_vm_address_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Set CF container based properties on a connection.
    ///
    /// This is a generic method to pass a CF container of properties to the connection. The properties are interpreted by the family and commonly represent configuration settings, but may be interpreted as anything.
    ///
    /// Parameter `connect`: The connect handle created by IOServiceOpen.
    ///
    /// Parameter `properties`: A CF container - commonly a CFDictionary but this is not enforced. The container should consist of objects which are understood by IOKit - these are currently : CFDictionary, CFArray, CFSet, CFString, CFData, CFNumber, CFBoolean, and are passed in the kernel as the corresponding OSDictionary etc. objects.
    ///
    /// Returns: A kern_return_t error code returned by the family.
    ///
    /// # Safety
    ///
    /// - `properties` should be of the correct type.
    /// - `properties` might not allow `None`.
    #[cfg(feature = "libc")]
    pub fn IOConnectSetCFProperties(
        connect: io_connect_t,
        properties: Option<&CFType>,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Set a CF container based property on a connection.
    ///
    /// This is a generic method to pass a CF property to the connection. The property is interpreted by the family and commonly represent configuration settings, but may be interpreted as anything.
    ///
    /// Parameter `connect`: The connect handle created by IOServiceOpen.
    ///
    /// Parameter `propertyName`: The name of the property as a CFString.
    ///
    /// Parameter `property`: A CF container - should consist of objects which are understood by IOKit - these are currently : CFDictionary, CFArray, CFSet, CFString, CFData, CFNumber, CFBoolean, and are passed in the kernel as the corresponding OSDictionary etc. objects.
    ///
    /// Returns: A kern_return_t error code returned by the object.
    ///
    /// # Safety
    ///
    /// - `property_name` might not allow `None`.
    /// - `property` should be of the correct type.
    /// - `property` might not allow `None`.
    #[cfg(feature = "libc")]
    pub fn IOConnectSetCFProperty(
        connect: io_connect_t,
        property_name: Option<&CFString>,
        property: Option<&CFType>,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `input` must be a valid pointer.
    /// - `input_struct` must be a valid pointer.
    /// - `output` must be a valid pointer.
    /// - `output_cnt` must be a valid pointer.
    /// - `output_struct` must be a valid pointer.
    /// - `output_struct_cnt` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectCallMethod(
        connection: libc::mach_port_t,
        selector: u32,
        input: *const u64,
        input_cnt: u32,
        input_struct: *const c_void,
        input_struct_cnt: usize,
        output: *mut u64,
        output_cnt: *mut u32,
        output_struct: *mut c_void,
        output_struct_cnt: *mut usize,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `reference` must be a valid pointer.
    /// - `input` must be a valid pointer.
    /// - `input_struct` must be a valid pointer.
    /// - `output` must be a valid pointer.
    /// - `output_cnt` must be a valid pointer.
    /// - `output_struct` must be a valid pointer.
    /// - `output_struct_cnt` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectCallAsyncMethod(
        connection: libc::mach_port_t,
        selector: u32,
        wake_port: libc::mach_port_t,
        reference: *mut u64,
        reference_cnt: u32,
        input: *const u64,
        input_cnt: u32,
        input_struct: *const c_void,
        input_struct_cnt: usize,
        output: *mut u64,
        output_cnt: *mut u32,
        output_struct: *mut c_void,
        output_struct_cnt: *mut usize,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `input_struct` must be a valid pointer.
    /// - `output_struct` must be a valid pointer.
    /// - `output_struct_cnt` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectCallStructMethod(
        connection: libc::mach_port_t,
        selector: u32,
        input_struct: *const c_void,
        input_struct_cnt: usize,
        output_struct: *mut c_void,
        output_struct_cnt: *mut usize,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `reference` must be a valid pointer.
    /// - `input_struct` must be a valid pointer.
    /// - `output_struct` must be a valid pointer.
    /// - `output_struct_cnt` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectCallAsyncStructMethod(
        connection: libc::mach_port_t,
        selector: u32,
        wake_port: libc::mach_port_t,
        reference: *mut u64,
        reference_cnt: u32,
        input_struct: *const c_void,
        input_struct_cnt: usize,
        output_struct: *mut c_void,
        output_struct_cnt: *mut usize,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `input` must be a valid pointer.
    /// - `output` must be a valid pointer.
    /// - `output_cnt` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectCallScalarMethod(
        connection: libc::mach_port_t,
        selector: u32,
        input: *const u64,
        input_cnt: u32,
        output: *mut u64,
        output_cnt: *mut u32,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `reference` must be a valid pointer.
    /// - `input` must be a valid pointer.
    /// - `output` must be a valid pointer.
    /// - `output_cnt` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOConnectCallAsyncScalarMethod(
        connection: libc::mach_port_t,
        selector: u32,
        wake_port: libc::mach_port_t,
        reference: *mut u64,
        reference_cnt: u32,
        input: *const u64,
        input_cnt: u32,
        output: *mut u64,
        output_cnt: *mut u32,
    ) -> libc::kern_return_t;
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectTrap0(connect: io_connect_t, index: u32) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectTrap0(connect: io_connect_t, index: u32) -> libc::kern_return_t;
    }
    unsafe { IOConnectTrap0(connect, index) }
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectTrap1(
    connect: io_connect_t,
    index: u32,
    p1: usize,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectTrap1(connect: io_connect_t, index: u32, p1: usize) -> libc::kern_return_t;
    }
    unsafe { IOConnectTrap1(connect, index, p1) }
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectTrap2(
    connect: io_connect_t,
    index: u32,
    p1: usize,
    p2: usize,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectTrap2(
            connect: io_connect_t,
            index: u32,
            p1: usize,
            p2: usize,
        ) -> libc::kern_return_t;
    }
    unsafe { IOConnectTrap2(connect, index, p1, p2) }
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectTrap3(
    connect: io_connect_t,
    index: u32,
    p1: usize,
    p2: usize,
    p3: usize,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectTrap3(
            connect: io_connect_t,
            index: u32,
            p1: usize,
            p2: usize,
            p3: usize,
        ) -> libc::kern_return_t;
    }
    unsafe { IOConnectTrap3(connect, index, p1, p2, p3) }
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectTrap4(
    connect: io_connect_t,
    index: u32,
    p1: usize,
    p2: usize,
    p3: usize,
    p4: usize,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectTrap4(
            connect: io_connect_t,
            index: u32,
            p1: usize,
            p2: usize,
            p3: usize,
            p4: usize,
        ) -> libc::kern_return_t;
    }
    unsafe { IOConnectTrap4(connect, index, p1, p2, p3, p4) }
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectTrap5(
    connect: io_connect_t,
    index: u32,
    p1: usize,
    p2: usize,
    p3: usize,
    p4: usize,
    p5: usize,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectTrap5(
            connect: io_connect_t,
            index: u32,
            p1: usize,
            p2: usize,
            p3: usize,
            p4: usize,
            p5: usize,
        ) -> libc::kern_return_t;
    }
    unsafe { IOConnectTrap5(connect, index, p1, p2, p3, p4, p5) }
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectTrap6(
    connect: io_connect_t,
    index: u32,
    p1: usize,
    p2: usize,
    p3: usize,
    p4: usize,
    p5: usize,
    p6: usize,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectTrap6(
            connect: io_connect_t,
            index: u32,
            p1: usize,
            p2: usize,
            p3: usize,
            p4: usize,
            p5: usize,
            p6: usize,
        ) -> libc::kern_return_t;
    }
    unsafe { IOConnectTrap6(connect, index, p1, p2, p3, p4, p5, p6) }
}

/// Inform a connection of a second connection.
///
/// This is a generic method to inform a family connection of a second connection, and is rarely used.
///
/// Parameter `connect`: The connect handle created by IOServiceOpen.
///
/// Parameter `client`: Another connect handle created by IOServiceOpen.
///
/// Returns: A kern_return_t error code returned by the family.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOConnectAddClient(
    connect: io_connect_t,
    client: io_connect_t,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOConnectAddClient(connect: io_connect_t, client: io_connect_t) -> libc::kern_return_t;
    }
    unsafe { IOConnectAddClient(connect, client) }
}

/// Return a handle to the registry root.
///
/// This method provides an accessor to the root of the registry for the machine. The root may be passed to a registry iterator when iterating a plane, and contains properties that describe the available planes, and diagnostic information for IOKit.
///
/// Parameter `mainPort`: The main port obtained from IOMainPort(). Pass kIOMainPortDefault to look up the default main port.
///
/// Returns: A handle to the IORegistryEntry root instance, to be released with IOObjectRelease by the caller, or MACH_PORT_NULL on failure.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IORegistryGetRootEntry(
    main_port: libc::mach_port_t,
) -> io_registry_entry_t {
    extern "C-unwind" {
        fn IORegistryGetRootEntry(main_port: libc::mach_port_t) -> io_registry_entry_t;
    }
    unsafe { IORegistryGetRootEntry(main_port) }
}

extern "C-unwind" {
    /// Looks up a registry entry by path.
    ///
    /// This function parses paths to lookup registry entries. The path should begin with '
    /// <plane
    /// name>:' If there are characters remaining unparsed after an entry has been looked up, this is considered an invalid lookup. Paths are further documented in IORegistryEntry.h
    ///
    /// Parameter `mainPort`: The main port obtained from IOMainPort(). Pass kIOMainPortDefault to look up the default main port.
    ///
    /// Parameter `path`: A C-string path.
    ///
    /// Returns: A handle to the IORegistryEntry which was found with the path, to be released with IOObjectRelease by the caller, or MACH_PORT_NULL on failure.
    ///
    /// # Safety
    ///
    /// `path` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryFromPath(
        main_port: libc::mach_port_t,
        path: io_string_t,
    ) -> io_registry_entry_t;
}

extern "C-unwind" {
    /// Looks up a registry entry by path.
    ///
    /// This function parses paths to lookup registry entries. The path should begin with '
    /// <plane
    /// name>:' If there are characters remaining unparsed after an entry has been looked up, this is considered an invalid lookup. Paths are further documented in IORegistryEntry.h
    ///
    /// Parameter `mainPort`: The main port obtained from IOMainPort(). Pass kIOMainPortDefault to look up the default main port.
    ///
    /// Parameter `path`: A CFString path.
    ///
    /// Returns: A handle to the IORegistryEntry which was found with the path, to be released with IOObjectRelease by the caller, or MACH_PORT_NULL on failure.
    ///
    /// # Safety
    ///
    /// `path` might not allow `None`.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryCopyFromPath(
        main_port: libc::mach_port_t,
        path: Option<&CFString>,
    ) -> io_registry_entry_t;
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioregistryiteraterecursively?language=objc)
pub const kIORegistryIterateRecursively: c_uint = 0x00000001;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kioregistryiterateparents?language=objc)
pub const kIORegistryIterateParents: c_uint = 0x00000002;

extern "C-unwind" {
    /// Create an iterator rooted at the registry root.
    ///
    /// This method creates an IORegistryIterator in the kernel that is set up with options to iterate children of the registry root entry, and to recurse automatically into entries as they are returned, or only when instructed with calls to IORegistryIteratorEnterEntry. The iterator object keeps track of entries that have been recursed into previously to avoid loops.
    ///
    /// Parameter `mainPort`: The main port obtained from IOMainPort(). Pass kIOMainPortDefault to look up the default main port.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `options`: kIORegistryIterateRecursively may be set to recurse automatically into each entry as it is returned from IOIteratorNext calls on the registry iterator.
    ///
    /// Parameter `iterator`: A created iterator handle, to be released by the caller when it has finished with it.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `iterator` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryCreateIterator(
        main_port: libc::mach_port_t,
        plane: io_name_t,
        options: IOOptionBits,
        iterator: *mut io_iterator_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Create an iterator rooted at a given registry entry.
    ///
    /// This method creates an IORegistryIterator in the kernel that is set up with options to iterate children or parents of a root entry, and to recurse automatically into entries as they are returned, or only when instructed with calls to IORegistryIteratorEnterEntry. The iterator object keeps track of entries that have been recursed into previously to avoid loops.
    ///
    /// Parameter `entry`: The root entry to begin the iteration at.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `options`: kIORegistryIterateRecursively may be set to recurse automatically into each entry as it is returned from IOIteratorNext calls on the registry iterator. kIORegistryIterateParents may be set to iterate the parents of each entry, by default the children are iterated.
    ///
    /// Parameter `iterator`: A created iterator handle, to be released by the caller when it has finished with it.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `iterator` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryCreateIterator(
        entry: io_registry_entry_t,
        plane: io_name_t,
        options: IOOptionBits,
        iterator: *mut io_iterator_t,
    ) -> libc::kern_return_t;
}

/// Recurse into the current entry in the registry iteration.
///
/// This method makes the current entry, ie. the last entry returned by IOIteratorNext, the root in a new level of recursion.
///
/// Returns: A kern_return_t error code.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IORegistryIteratorEnterEntry(
    iterator: io_iterator_t,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IORegistryIteratorEnterEntry(iterator: io_iterator_t) -> libc::kern_return_t;
    }
    unsafe { IORegistryIteratorEnterEntry(iterator) }
}

/// Exits a level of recursion, restoring the current entry.
///
/// This method undoes an IORegistryIteratorEnterEntry, restoring the current entry. If there are no more levels of recursion to exit false is returned, otherwise true is returned.
///
/// Returns: kIOReturnSuccess if a level of recursion was undone, kIOReturnNoDevice if no recursive levels are left in the iteration.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IORegistryIteratorExitEntry(
    iterator: io_iterator_t,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IORegistryIteratorExitEntry(iterator: io_iterator_t) -> libc::kern_return_t;
    }
    unsafe { IORegistryIteratorExitEntry(iterator) }
}

extern "C-unwind" {
    /// Returns a C-string name assigned to a registry entry.
    ///
    /// Registry entries can be named in a particular plane, or globally. This function returns the entry's global name. The global name defaults to the entry's meta class name if it has not been named.
    ///
    /// Parameter `entry`: The registry entry handle whose name to look up.
    ///
    /// Parameter `name`: The caller's buffer to receive the name.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `name` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetName(
        entry: io_registry_entry_t,
        name: io_name_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns a C-string name assigned to a registry entry, in a specified plane.
    ///
    /// Registry entries can be named in a particular plane, or globally. This function returns the entry's name in the specified plane or global name if it has not been named in that plane. The global name defaults to the entry's meta class name if it has not been named.
    ///
    /// Parameter `entry`: The registry entry handle whose name to look up.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `name`: The caller's buffer to receive the name.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `name` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetNameInPlane(
        entry: io_registry_entry_t,
        plane: io_name_t,
        name: io_name_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns a C-string location assigned to a registry entry, in a specified plane.
    ///
    /// Registry entries can given a location string in a particular plane, or globally. If the entry has had a location set in the specified plane that location string will be returned, otherwise the global location string is returned. If no global location string has been set, an error is returned.
    ///
    /// Parameter `entry`: The registry entry handle whose name to look up.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `location`: The caller's buffer to receive the location string.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `location` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetLocationInPlane(
        entry: io_registry_entry_t,
        plane: io_name_t,
        location: io_name_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Create a path for a registry entry.
    ///
    /// The path for a registry entry is copied to the caller's buffer. The path describes the entry's attachment in a particular plane, which must be specified. The path begins with the plane name followed by a colon, and then followed by '/' separated path components for each of the entries between the root and the registry entry. An alias may also exist for the entry, and will be returned if available.
    ///
    /// Parameter `entry`: The registry entry handle whose path to look up.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `path`: A char buffer allocated by the caller.
    ///
    /// Returns: IORegistryEntryGetPath will fail if the entry is not attached in the plane, or if the buffer is not large enough to contain the path.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `path` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetPath(
        entry: io_registry_entry_t,
        plane: io_name_t,
        path: io_string_t,
    ) -> libc::kern_return_t;
}

/// Create a path for a registry entry.
///
/// The path for a registry entry is returned as a CFString The path describes the entry's attachment in a particular plane, which must be specified. The path begins with the plane name followed by a colon, and then followed by '/' separated path components for each of the entries between the root and the registry entry. An alias may also exist for the entry, and will be returned if available.
///
/// Parameter `entry`: The registry entry handle whose path to look up.
///
/// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
///
/// Returns: An instance of CFString on success, to be released by the caller. IORegistryEntryCopyPath will fail if the entry is not attached in the plane.
///
/// # Safety
///
/// `plane` Array TODO.
#[cfg(feature = "libc")]
#[inline]
pub unsafe extern "C-unwind" fn IORegistryEntryCopyPath(
    entry: io_registry_entry_t,
    plane: io_name_t,
) -> Option<CFRetained<CFString>> {
    extern "C-unwind" {
        fn IORegistryEntryCopyPath(
            entry: io_registry_entry_t,
            plane: io_name_t,
        ) -> Option<NonNull<CFString>>;
    }
    let ret = unsafe { IORegistryEntryCopyPath(entry, plane) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

extern "C-unwind" {
    /// Returns an ID for the registry entry that is global to all tasks.
    ///
    /// The entry ID returned by IORegistryEntryGetRegistryEntryID can be used to identify a registry entry across all tasks. A registry entry may be looked up by its entryID by creating a matching dictionary with IORegistryEntryIDMatching() to be used with the IOKit matching functions. The ID is valid only until the machine reboots.
    ///
    /// Parameter `entry`: The registry entry handle whose ID to look up.
    ///
    /// Parameter `entryID`: The resulting ID.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// `entry_id` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetRegistryEntryID(
        entry: io_registry_entry_t,
        entry_id: *mut u64,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Create a CF dictionary representation of a registry entry's property table.
    ///
    /// This function creates an instantaneous snapshot of a registry entry's property table, creating a CFDictionary analogue in the caller's task. Not every object available in the kernel is represented as a CF container; currently OSDictionary, OSArray, OSSet, OSSymbol, OSString, OSData, OSNumber, OSBoolean are created as their CF counterparts.
    ///
    /// Parameter `entry`: The registry entry handle whose property table to copy.
    ///
    /// Parameter `properties`: A CFDictionary is created and returned the caller on success. The caller should release with CFRelease.
    ///
    /// Parameter `allocator`: The CF allocator to use when creating the CF containers.
    ///
    /// Parameter `options`: No options are currently defined.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `properties` must be a valid pointer.
    /// - `allocator` might not allow `None`.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryCreateCFProperties(
        entry: io_registry_entry_t,
        properties: *mut *mut CFMutableDictionary,
        allocator: Option<&CFAllocator>,
        options: IOOptionBits,
    ) -> libc::kern_return_t;
}

/// Create a CF representation of a registry entry's property.
///
/// This function creates an instantaneous snapshot of a registry entry property, creating a CF container analogue in the caller's task. Not every object available in the kernel is represented as a CF container; currently OSDictionary, OSArray, OSSet, OSSymbol, OSString, OSData, OSNumber, OSBoolean are created as their CF counterparts.
///
/// Parameter `entry`: The registry entry handle whose property to copy.
///
/// Parameter `key`: A CFString specifying the property name.
///
/// Parameter `allocator`: The CF allocator to use when creating the CF container.
///
/// Parameter `options`: No options are currently defined.
///
/// Returns: A CF container is created and returned the caller on success. The caller should release with CFRelease.
///
/// # Safety
///
/// - `key` might not allow `None`.
/// - `allocator` might not allow `None`.
#[cfg(feature = "libc")]
#[inline]
pub unsafe extern "C-unwind" fn IORegistryEntryCreateCFProperty(
    entry: io_registry_entry_t,
    key: Option<&CFString>,
    allocator: Option<&CFAllocator>,
    options: IOOptionBits,
) -> Option<CFRetained<CFType>> {
    extern "C-unwind" {
        fn IORegistryEntryCreateCFProperty(
            entry: io_registry_entry_t,
            key: Option<&CFString>,
            allocator: Option<&CFAllocator>,
            options: IOOptionBits,
        ) -> Option<NonNull<CFType>>;
    }
    let ret = unsafe { IORegistryEntryCreateCFProperty(entry, key, allocator, options) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// Create a CF representation of a registry entry's property.
///
/// This function creates an instantaneous snapshot of a registry entry property, creating a CF container analogue in the caller's task. Not every object available in the kernel is represented as a CF container; currently OSDictionary, OSArray, OSSet, OSSymbol, OSString, OSData, OSNumber, OSBoolean are created as their CF counterparts.
/// This function will search for a property, starting first with specified registry entry's property table, then iterating recusively through either the parent registry entries or the child registry entries of this entry. Once the first occurrence is found, it will lookup and return the value of the property, using the same semantics as IORegistryEntryCreateCFProperty. The iteration keeps track of entries that have been recursed into previously to avoid loops.
///
/// Parameter `entry`: The registry entry at which to start the search.
///
/// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
///
/// Parameter `key`: A CFString specifying the property name.
///
/// Parameter `allocator`: The CF allocator to use when creating the CF container.
///
/// Parameter `options`: kIORegistryIterateRecursively may be set to recurse automatically into the registry hierarchy. Without this option, this method degenerates into the standard IORegistryEntryCreateCFProperty() call. kIORegistryIterateParents may be set to iterate the parents of the entry, in place of the children.
///
/// Returns: A CF container is created and returned the caller on success. The caller should release with CFRelease.
///
/// # Safety
///
/// - `plane` Array TODO.
/// - `key` might not allow `None`.
/// - `allocator` might not allow `None`.
#[cfg(feature = "libc")]
#[inline]
pub unsafe extern "C-unwind" fn IORegistryEntrySearchCFProperty(
    entry: io_registry_entry_t,
    plane: io_name_t,
    key: Option<&CFString>,
    allocator: Option<&CFAllocator>,
    options: IOOptionBits,
) -> Option<CFRetained<CFType>> {
    extern "C-unwind" {
        fn IORegistryEntrySearchCFProperty(
            entry: io_registry_entry_t,
            plane: io_name_t,
            key: Option<&CFString>,
            allocator: Option<&CFAllocator>,
            options: IOOptionBits,
        ) -> Option<NonNull<CFType>>;
    }
    let ret = unsafe { IORegistryEntrySearchCFProperty(entry, plane, key, allocator, options) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `property_name` Array TODO.
    /// - `buffer` Array TODO.
    /// - `size` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetProperty(
        entry: io_registry_entry_t,
        property_name: io_name_t,
        buffer: io_struct_inband_t,
        size: *mut u32,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Set CF container based properties in a registry entry.
    ///
    /// This is a generic method to pass a CF container of properties to an object in the registry. Setting properties in a registry entry is not generally supported, it is more common to support IOConnectSetCFProperties for connection based property setting. The properties are interpreted by the object.
    ///
    /// Parameter `entry`: The registry entry whose properties to set.
    ///
    /// Parameter `properties`: A CF container - commonly a CFDictionary but this is not enforced. The container should consist of objects which are understood by IOKit - these are currently : CFDictionary, CFArray, CFSet, CFString, CFData, CFNumber, CFBoolean, and are passed in the kernel as the corresponding OSDictionary etc. objects.
    ///
    /// Returns: A kern_return_t error code returned by the object.
    ///
    /// # Safety
    ///
    /// - `properties` should be of the correct type.
    /// - `properties` might not allow `None`.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntrySetCFProperties(
        entry: io_registry_entry_t,
        properties: Option<&CFType>,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Set a CF container based property in a registry entry.
    ///
    /// This is a generic method to pass a CF container as a property to an object in the registry. Setting properties in a registry entry is not generally supported, it is more common to support IOConnectSetCFProperty for connection based property setting. The property is interpreted by the object.
    ///
    /// Parameter `entry`: The registry entry whose property to set.
    ///
    /// Parameter `propertyName`: The name of the property as a CFString.
    ///
    /// Parameter `property`: A CF container - should consist of objects which are understood by IOKit - these are currently : CFDictionary, CFArray, CFSet, CFString, CFData, CFNumber, CFBoolean, and are passed in the kernel as the corresponding OSDictionary etc. objects.
    ///
    /// Returns: A kern_return_t error code returned by the object.
    ///
    /// # Safety
    ///
    /// - `property_name` might not allow `None`.
    /// - `property` should be of the correct type.
    /// - `property` might not allow `None`.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntrySetCFProperty(
        entry: io_registry_entry_t,
        property_name: Option<&CFString>,
        property: Option<&CFType>,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns an iterator over an registry entry's child entries in a plane.
    ///
    /// This method creates an iterator which will return each of a registry entry's child entries in a specified plane.
    ///
    /// Parameter `entry`: The registry entry whose children to iterate over.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `iterator`: The created iterator over the children of the entry, on success. The iterator must be released when the iteration is finished.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `iterator` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetChildIterator(
        entry: io_registry_entry_t,
        plane: io_name_t,
        iterator: *mut io_iterator_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns the first child of a registry entry in a plane.
    ///
    /// This function will return the child which first attached to a registry entry in a plane.
    ///
    /// Parameter `entry`: The registry entry whose child to look up.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `child`: The first child of the registry entry, on success. The child must be released by the caller.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `child` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetChildEntry(
        entry: io_registry_entry_t,
        plane: io_name_t,
        child: *mut io_registry_entry_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns an iterator over an registry entry's parent entries in a plane.
    ///
    /// This method creates an iterator which will return each of a registry entry's parent entries in a specified plane.
    ///
    /// Parameter `entry`: The registry entry whose parents to iterate over.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `iterator`: The created iterator over the parents of the entry, on success. The iterator must be released when the iteration is finished.
    ///
    /// Returns: A kern_return_t error.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `iterator` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetParentIterator(
        entry: io_registry_entry_t,
        plane: io_name_t,
        iterator: *mut io_iterator_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// Returns the first parent of a registry entry in a plane.
    ///
    /// This function will return the parent to which the registry entry was first attached in a plane.
    ///
    /// Parameter `entry`: The registry entry whose parent to look up.
    ///
    /// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
    ///
    /// Parameter `parent`: The first parent of the registry entry, on success. The parent must be released by the caller.
    ///
    /// Returns: A kern_return_t error code.
    ///
    /// # Safety
    ///
    /// - `plane` Array TODO.
    /// - `parent` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IORegistryEntryGetParentEntry(
        entry: io_registry_entry_t,
        plane: io_name_t,
        parent: *mut io_registry_entry_t,
    ) -> libc::kern_return_t;
}

/// Determines if the registry entry is attached in a plane.
///
/// This method determines if the entry is attached in a plane to any other entry.
///
/// Parameter `entry`: The registry entry.
///
/// Parameter `plane`: The name of an existing registry plane. Plane names are defined in IOKitKeys.h, eg. kIOServicePlane.
///
/// Returns: If the entry has a parent in the plane, true is returned, otherwise false is returned.
///
/// # Safety
///
/// `plane` Array TODO.
#[cfg(feature = "libc")]
#[inline]
pub unsafe extern "C-unwind" fn IORegistryEntryInPlane(
    entry: io_registry_entry_t,
    plane: io_name_t,
) -> bool {
    extern "C-unwind" {
        fn IORegistryEntryInPlane(entry: io_registry_entry_t, plane: io_name_t) -> libc::boolean_t;
    }
    let ret = unsafe { IORegistryEntryInPlane(entry, plane) };
    ret != 0
}

/// Create a matching dictionary that specifies an IOService class match.
///
/// A very common matching criteria for IOService is based on its class. IOServiceMatching will create a matching dictionary that specifies any IOService of a class, or its subclasses. The class is specified by C-string name.
///
/// Parameter `name`: The class name, as a const C-string. Class matching is successful on IOService's of this class or any subclass.
///
/// Returns: The matching dictionary created, is returned on success, or zero on failure. The dictionary is commonly passed to IOServiceGetMatchingServices or IOServiceAddNotification which will consume a reference, otherwise it should be released with CFRelease by the caller.
///
/// # Safety
///
/// - `name` must be a valid pointer.
/// - The returned generics must be of the correct type.
#[inline]
pub unsafe extern "C-unwind" fn IOServiceMatching(
    name: *const c_char,
) -> Option<CFRetained<CFMutableDictionary>> {
    extern "C-unwind" {
        fn IOServiceMatching(name: *const c_char) -> Option<NonNull<CFMutableDictionary>>;
    }
    let ret = unsafe { IOServiceMatching(name) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// Create a matching dictionary that specifies an IOService name match.
///
/// A common matching criteria for IOService is based on its name. IOServiceNameMatching will create a matching dictionary that specifies an IOService with a given name. Some IOServices created from the device tree will perform name matching on the standard compatible, name, model properties.
///
/// Parameter `name`: The IOService name, as a const C-string.
///
/// Returns: The matching dictionary created, is returned on success, or zero on failure. The dictionary is commonly passed to IOServiceGetMatchingServices or IOServiceAddNotification which will consume a reference, otherwise it should be released with CFRelease by the caller.
///
/// # Safety
///
/// - `name` must be a valid pointer.
/// - The returned generics must be of the correct type.
#[inline]
pub unsafe extern "C-unwind" fn IOServiceNameMatching(
    name: *const c_char,
) -> Option<CFRetained<CFMutableDictionary>> {
    extern "C-unwind" {
        fn IOServiceNameMatching(name: *const c_char) -> Option<NonNull<CFMutableDictionary>>;
    }
    let ret = unsafe { IOServiceNameMatching(name) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// Create a matching dictionary that specifies an IOService match based on BSD device name.
///
/// IOServices that represent BSD devices have an associated BSD name. This function creates a matching dictionary that will match IOService's with a given BSD name.
///
/// Parameter `mainPort`: The main port obtained from IOMainPort(). Pass kIOMainPortDefault to look up the default main port.
///
/// Parameter `options`: No options are currently defined.
///
/// Parameter `bsdName`: The BSD name, as a const char *.
///
/// Returns: The matching dictionary created, is returned on success, or zero on failure. The dictionary is commonly passed to IOServiceGetMatchingServices or IOServiceAddNotification which will consume a reference, otherwise it should be released with CFRelease by the caller.
///
/// # Safety
///
/// - `bsd_name` must be a valid pointer.
/// - The returned generics must be of the correct type.
#[cfg(feature = "libc")]
#[inline]
pub unsafe extern "C-unwind" fn IOBSDNameMatching(
    main_port: libc::mach_port_t,
    options: u32,
    bsd_name: *const c_char,
) -> Option<CFRetained<CFMutableDictionary>> {
    extern "C-unwind" {
        fn IOBSDNameMatching(
            main_port: libc::mach_port_t,
            options: u32,
            bsd_name: *const c_char,
        ) -> Option<NonNull<CFMutableDictionary>>;
    }
    let ret = unsafe { IOBSDNameMatching(main_port, options, bsd_name) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// # Safety
///
/// - `path` must be a valid pointer.
/// - The returned generics must be of the correct type.
#[cfg(feature = "libc")]
#[deprecated]
#[inline]
pub unsafe extern "C-unwind" fn IOOpenFirmwarePathMatching(
    main_port: libc::mach_port_t,
    options: u32,
    path: *const c_char,
) -> Option<CFRetained<CFMutableDictionary>> {
    extern "C-unwind" {
        fn IOOpenFirmwarePathMatching(
            main_port: libc::mach_port_t,
            options: u32,
            path: *const c_char,
        ) -> Option<NonNull<CFMutableDictionary>>;
    }
    let ret = unsafe { IOOpenFirmwarePathMatching(main_port, options, path) };
    ret.map(|ret| unsafe { CFRetained::retain(ret) })
}

/// Create a matching dictionary that specifies an IOService match based on a registry entry ID.
///
/// This function creates a matching dictionary that will match a registered, active IOService found with the given registry entry ID. The entry ID for a registry entry is returned by IORegistryEntryGetRegistryEntryID().
///
/// Parameter `entryID`: The registry entry ID to be found.
///
/// Returns: The matching dictionary created, is returned on success, or zero on failure. The dictionary is commonly passed to IOServiceGetMatchingServices or IOServiceAddNotification which will consume a reference, otherwise it should be released with CFRelease by the caller.
///
/// # Safety
///
/// The returned generics must be of the correct type.
#[inline]
pub unsafe extern "C-unwind" fn IORegistryEntryIDMatching(
    entry_id: u64,
) -> Option<CFRetained<CFMutableDictionary>> {
    extern "C-unwind" {
        fn IORegistryEntryIDMatching(entry_id: u64) -> Option<NonNull<CFMutableDictionary>>;
    }
    let ret = unsafe { IORegistryEntryIDMatching(entry_id) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `open_firmware_path` Array TODO.
    /// - `bsd_name` Array TODO.
    #[cfg(feature = "libc")]
    #[deprecated]
    pub fn IOServiceOFPathToBSDName(
        main_port: libc::mach_port_t,
        open_firmware_path: io_name_t,
        bsd_name: io_name_t,
    ) -> libc::kern_return_t;
}

/// standard callback function for asynchronous I/O requests with
/// no extra arguments beyond a refcon and result code.
///
/// Parameter `refcon`: The refcon passed into the original I/O request
///
/// Parameter `result`: The result of the I/O operation
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/ioasynccallback0?language=objc)
pub type IOAsyncCallback0 = Option<unsafe extern "C-unwind" fn(*mut c_void, IOReturn)>;

/// standard callback function for asynchronous I/O requests with
/// one extra argument beyond a refcon and result code.
/// This is often a count of the number of bytes transferred
///
/// Parameter `refcon`: The refcon passed into the original I/O request
///
/// Parameter `result`: The result of the I/O operation
///
/// Parameter `arg0`: Extra argument
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/ioasynccallback1?language=objc)
pub type IOAsyncCallback1 = Option<unsafe extern "C-unwind" fn(*mut c_void, IOReturn, *mut c_void)>;

/// standard callback function for asynchronous I/O requests with
/// two extra arguments beyond a refcon and result code.
///
/// Parameter `refcon`: The refcon passed into the original I/O request
///
/// Parameter `result`: The result of the I/O operation
///
/// Parameter `arg0`: Extra argument
///
/// Parameter `arg1`: Extra argument
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/ioasynccallback2?language=objc)
pub type IOAsyncCallback2 =
    Option<unsafe extern "C-unwind" fn(*mut c_void, IOReturn, *mut c_void, *mut c_void)>;

/// standard callback function for asynchronous I/O requests with
/// lots of extra arguments beyond a refcon and result code.
///
/// Parameter `refcon`: The refcon passed into the original I/O request
///
/// Parameter `result`: The result of the I/O operation
///
/// Parameter `args`: Array of extra arguments
///
/// Parameter `numArgs`: Number of extra arguments
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/ioasynccallback?language=objc)
pub type IOAsyncCallback =
    Option<unsafe extern "C-unwind" fn(*mut c_void, IOReturn, *mut *mut c_void, u32)>;

extern "C-unwind" {
    /// # Safety
    ///
    /// `buffer` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOCatalogueSendData(
        main_port: libc::mach_port_t,
        flag: u32,
        buffer: *const c_char,
        size: u32,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// `description` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IOCatalogueTerminate(
        main_port: libc::mach_port_t,
        flag: u32,
        description: io_name_t,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// - `buffer` must be a valid pointer.
    /// - `size` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOCatalogueGetData(
        main_port: libc::mach_port_t,
        flag: u32,
        buffer: *mut *mut c_char,
        size: *mut u32,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// `name` Array TODO.
    #[cfg(feature = "libc")]
    pub fn IOCatalogueModuleLoaded(
        main_port: libc::mach_port_t,
        name: io_name_t,
    ) -> libc::kern_return_t;
}

#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IOCatalogueReset(
    main_port: libc::mach_port_t,
    flag: u32,
) -> libc::kern_return_t {
    extern "C-unwind" {
        fn IOCatalogueReset(main_port: libc::mach_port_t, flag: u32) -> libc::kern_return_t;
    }
    unsafe { IOCatalogueReset(main_port, flag) }
}

/// Represents an entry within the data queue
///
/// This is a variable sized struct.  The data field simply represents the start of the data region.  The size of the data region is stored in the size field.  The whole size of the specific entry is the size of a UInt32 plus the size of the data region.
/// Field: size The size of the following data region.
/// Field: data Represents the beginning of the data region.  The address of the data field is a pointer to the start of the data region.
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/iodataqueueentry?language=objc)
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IODataQueueEntry {
    pub size: u32,
    pub data: [u8; 4],
}

#[cfg(feature = "objc2")]
unsafe impl Encode for IODataQueueEntry {
    const ENCODING: Encoding =
        Encoding::Struct("_IODataQueueEntry", &[<u32>::ENCODING, <[u8; 4]>::ENCODING]);
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IODataQueueEntry {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// A struct mapping to the header region of a data queue.
///
/// This struct is variable sized.  The struct represents the data queue header information plus a pointer to the actual data queue itself.  The size of the struct is the combined size of the header fields (3 * sizeof(UInt32)) plus the actual size of the queue region.  This size is stored in the queueSize field.
/// Field: queueSize The size of the queue region pointed to by the queue field.
/// Field: head The location of the queue head.  This field is represented as a byte offset from the beginning of the queue memory region.
/// Field: tail The location of the queue tail.  This field is represented as a byte offset from the beginning of the queue memory region.
/// Field: queue Represents the beginning of the queue memory region.  The size of the region pointed to by queue is stored in the queueSize field.
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/iodataqueuememory?language=objc)
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IODataQueueMemory {
    pub queueSize: u32,
    pub head: u32,
    pub tail: u32,
    pub queue: [IODataQueueEntry; 1],
}

#[cfg(feature = "objc2")]
unsafe impl Encode for IODataQueueMemory {
    const ENCODING: Encoding = Encoding::Struct(
        "_IODataQueueMemory",
        &[
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <[IODataQueueEntry; 1]>::ENCODING,
        ],
    );
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IODataQueueMemory {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// Used to determine if more data is available on the queue.
///
/// Parameter `dataQueue`: The IODataQueueMemory region mapped from the kernel.
///
/// Returns: Returns true if data is available and false if not.
///
/// # Safety
///
/// `data_queue` must be a valid pointer.
#[inline]
pub unsafe extern "C-unwind" fn IODataQueueDataAvailable(
    data_queue: *mut IODataQueueMemory,
) -> bool {
    extern "C-unwind" {
        fn IODataQueueDataAvailable(data_queue: *mut IODataQueueMemory) -> Boolean;
    }
    let ret = unsafe { IODataQueueDataAvailable(data_queue) };
    ret != 0
}

extern "C-unwind" {
    /// Used to peek at the next entry on the queue.
    ///
    /// This function can be used to look at the next entry which allows the entry to be received without having to copy it with IODataQueueDequeue.  In order to do this, call IODataQueuePeek to get the entry.  Then call IODataQueueDequeue with a NULL data pointer.  That will cause the head to be moved to the next entry, but no memory to be copied.
    ///
    /// Parameter `dataQueue`: The IODataQueueMemory region mapped from the kernel.
    ///
    /// Returns: Returns a pointer to the next IODataQueueEntry if one is available.  Zero is returned if the queue is empty.
    ///
    /// # Safety
    ///
    /// `data_queue` must be a valid pointer.
    pub fn IODataQueuePeek(data_queue: *mut IODataQueueMemory) -> *mut IODataQueueEntry;
}

extern "C-unwind" {
    /// Dequeues the next available entry on the queue and copies it into the given data pointer.
    ///
    /// This function will dequeue the next available entry on the queue.  If a data pointer is provided, it will copy the data into the memory region if there is enough space available as specified in the dataSize parameter.  If no data pointer is provided, it will simply move the head value past the current entry.
    ///
    /// Parameter `dataQueue`: The IODataQueueMemory region mapped from the kernel.
    ///
    /// Parameter `data`: A pointer to the data memory region in which to copy the next entry data on the queue.  If this parameter is 0 (NULL), it will simply move to the next entry.
    ///
    /// Parameter `dataSize`: A pointer to the size of the data parameter.  On return, this contains the size of the actual entry data - even if the original size was not large enough.
    ///
    /// Returns: Returns kIOReturnSuccess on success.  Other return values possible are: kIOReturnUnderrun - queue is empty, kIOReturnBadArgument - no dataQueue or no dataSize, kIOReturnNoSpace - dataSize is too small for entry.
    ///
    /// # Safety
    ///
    /// - `data_queue` must be a valid pointer.
    /// - `data` must be a valid pointer.
    /// - `data_size` must be a valid pointer.
    pub fn IODataQueueDequeue(
        data_queue: *mut IODataQueueMemory,
        data: *mut c_void,
        data_size: *mut u32,
    ) -> IOReturn;
}

extern "C-unwind" {
    /// Wait for an incoming dataAvailable message on the given notifyPort.
    ///
    /// This method will simply wait for an incoming message on the given notifyPort.  Once it is received, the return from mach_msg() is returned.
    ///
    /// Parameter `dataQueue`: The IODataQueueMemory region mapped from the kernel.
    ///
    /// Parameter `notificationPort`: Mach port on which to listen for incoming messages.
    ///
    /// Returns: Returns kIOReturnSuccess on success.  Returns kIOReturnBadArgument if either dataQueue is 0 (NULL) or notifyPort is MACH_PORT_NULL.  Returns the result of the mach_msg() listen call on the given port.
    ///
    /// # Safety
    ///
    /// `data_queue` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IODataQueueWaitForAvailableData(
        data_queue: *mut IODataQueueMemory,
        notification_port: libc::mach_port_t,
    ) -> IOReturn;
}

/// Allocates and returns a new mach port able to receive data available notifications from an IODataQueue.
///
/// This port is intended to be passed down into the kernel and into an IODataQueue to allow it to send the appropriate notification.  The returned mach port is allocated with a queue limit of one message.  This allows only one mach message to be queued up at a time.  The IODataQueue code is written with the restriction in mind and will only queue up a message if no messages alread have been sent.
///
/// Returns: Returns a newly allocated mach port on success.  On failure, it returns MACH_PORT_NULL.
#[cfg(feature = "libc")]
#[inline]
pub extern "C-unwind" fn IODataQueueAllocateNotificationPort() -> libc::mach_port_t {
    extern "C-unwind" {
        fn IODataQueueAllocateNotificationPort() -> libc::mach_port_t;
    }
    unsafe { IODataQueueAllocateNotificationPort() }
}

extern "C-unwind" {
    /// Enqueues a new entry on the queue.
    ///
    /// This method adds a new data entry of dataSize to the queue.  It sets the size parameter of the entry pointed to by the tail value and copies the memory pointed to by the data parameter in place in the queue.  Once that is done, it moves the tail to the next available location.  When attempting to add a new entry towards the end of the queue and there isn't enough space at the end, it wraps back to the beginning.
    /// <br>
    /// If the queue is empty when a new entry is added, the port specified in IODataQueueSetNotificationPort will be used to send a message to the client process that data is now available.
    /// <br>
    /// <b>
    /// Please note that using this method without mapped memory create from an IOSharedDataQueue will result in undefined behavior.
    /// </b>
    ///
    /// Parameter `dataQueue`: The IODataQueueMemory region mapped from the kernel created from an IOSharedDataQueue.
    ///
    /// Parameter `data`: Pointer to the data to be added to the queue.
    ///
    /// Parameter `dataSize`: Size of the data pointed to by data.
    ///
    /// Returns: Returns kIOReturnSuccess on success.  Other return values possible are: kIOReturnOverrun - queue is full.
    ///
    /// # Safety
    ///
    /// - `data_queue` must be a valid pointer.
    /// - `data` must be a valid pointer.
    pub fn IODataQueueEnqueue(
        data_queue: *mut IODataQueueMemory,
        data: *mut c_void,
        data_size: u32,
    ) -> IOReturn;
}

extern "C-unwind" {
    /// Creates a simple mach message targeting the mach port specified in port.
    ///
    /// This message is sent when data is added to an empty queue.  It is to notify another user process that new data has become available.
    /// <b>
    /// Please note that using this method without mapped memory create from an IOSharedDataQueue will result in undefined behavior.
    /// </b>
    ///
    /// Parameter `dataQueue`: The IODataQueueMemory region mapped from the kernel created from an IOSharedDataQueue.
    ///
    /// Parameter `notifyPort`: The mach port to target with the notification message.
    ///
    /// Returns: Returns kIOReturnSuccess on success.  Returns kIOReturnBadArgument if either dataQueue is 0 (NULL).
    ///
    /// # Safety
    ///
    /// `data_queue` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IODataQueueSetNotificationPort(
        data_queue: *mut IODataQueueMemory,
        notify_port: libc::mach_port_t,
    ) -> IOReturn;
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iocfplugininterfacestruct?language=objc)
#[cfg(feature = "libc")]
#[repr(C)]
#[allow(unpredictable_function_pointer_comparisons)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IOCFPlugInInterfaceStruct {
    pub(crate) _reserved: *mut c_void,
    pub QueryInterface:
        Option<unsafe extern "C-unwind" fn(*mut c_void, REFIID, *mut LPVOID) -> HRESULT>,
    pub AddRef: Option<unsafe extern "C-unwind" fn(*mut c_void) -> ULONG>,
    pub Release: Option<unsafe extern "C-unwind" fn(*mut c_void) -> ULONG>,
    pub version: u16,
    pub revision: u16,
    pub Probe: Option<
        unsafe extern "C-unwind" fn(
            *mut c_void,
            *const CFDictionary,
            io_service_t,
            *mut i32,
        ) -> IOReturn,
    >,
    pub Start: Option<
        unsafe extern "C-unwind" fn(*mut c_void, *const CFDictionary, io_service_t) -> IOReturn,
    >,
    pub Stop: Option<unsafe extern "C-unwind" fn(*mut c_void) -> IOReturn>,
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl Encode for IOCFPlugInInterfaceStruct {
    const ENCODING: Encoding = Encoding::Struct("IOCFPlugInInterfaceStruct", &[
        <*mut c_void>::ENCODING,
        <Option<unsafe extern "C-unwind" fn(*mut c_void,REFIID,*mut LPVOID,) -> HRESULT>>::ENCODING,
        <Option<unsafe extern "C-unwind" fn(*mut c_void,) -> ULONG>>::ENCODING,
        <Option<unsafe extern "C-unwind" fn(*mut c_void,) -> ULONG>>::ENCODING,
        <u16>::ENCODING,
        <u16>::ENCODING,
        <Option<unsafe extern "C-unwind" fn(*mut c_void,*const CFDictionary,io_service_t,*mut i32,) -> IOReturn>>::ENCODING,
        <Option<unsafe extern "C-unwind" fn(*mut c_void,*const CFDictionary,io_service_t,) -> IOReturn>>::ENCODING,
        <Option<unsafe extern "C-unwind" fn(*mut c_void,) -> IOReturn>>::ENCODING,
    ]);
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl RefEncode for IOCFPlugInInterfaceStruct {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iocfplugininterface?language=objc)
#[cfg(feature = "libc")]
pub type IOCFPlugInInterface = IOCFPlugInInterfaceStruct;

extern "C-unwind" {
    /// # Safety
    ///
    /// - `plugin_type` might not allow `None`.
    /// - `interface_type` might not allow `None`.
    /// - `the_interface` must be a valid pointer.
    /// - `the_score` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IOCreatePlugInInterfaceForService(
        service: io_service_t,
        plugin_type: Option<&CFUUID>,
        interface_type: Option<&CFUUID>,
        the_interface: *mut *mut *mut IOCFPlugInInterface,
        the_score: *mut i32,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    /// # Safety
    ///
    /// `interface` must be a valid pointer.
    #[cfg(feature = "libc")]
    pub fn IODestroyPlugInInterface(
        interface: *mut *mut IOCFPlugInInterface,
    ) -> libc::kern_return_t;
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocfserializetobinary?language=objc)
pub const kIOCFSerializeToBinary: c_uint = 0x00000001;

/// # Safety
///
/// - `object` should be of the correct type.
/// - `object` might not allow `None`.
#[inline]
pub unsafe extern "C-unwind" fn IOCFSerialize(
    object: Option<&CFType>,
    options: CFOptionFlags,
) -> Option<CFRetained<CFData>> {
    extern "C-unwind" {
        fn IOCFSerialize(
            object: Option<&CFType>,
            options: CFOptionFlags,
        ) -> Option<NonNull<CFData>>;
    }
    let ret = unsafe { IOCFSerialize(object, options) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// # Safety
///
/// - `alloc` might not allow `None`.
/// - `url` might not allow `None`.
/// - `property` might not allow `None`.
/// - `error_code` must be a valid pointer.
#[inline]
pub unsafe extern "C-unwind" fn IOURLCreatePropertyFromResource(
    alloc: Option<&CFAllocator>,
    url: Option<&CFURL>,
    property: Option<&CFString>,
    error_code: *mut i32,
) -> Option<CFRetained<CFType>> {
    extern "C-unwind" {
        fn IOURLCreatePropertyFromResource(
            alloc: Option<&CFAllocator>,
            url: Option<&CFURL>,
            property: Option<&CFString>,
            error_code: *mut i32,
        ) -> Option<NonNull<CFType>>;
    }
    let ret = unsafe { IOURLCreatePropertyFromResource(alloc, url, property, error_code) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// # Safety
///
/// - `alloc` might not allow `None`.
/// - `url` might not allow `None`.
/// - `resource_data` must be a valid pointer.
/// - `properties` must be a valid pointer.
/// - `desired_properties` generic must be of the correct type.
/// - `desired_properties` might not allow `None`.
/// - `error_code` must be a valid pointer.
#[inline]
pub unsafe extern "C-unwind" fn IOURLCreateDataAndPropertiesFromResource(
    alloc: Option<&CFAllocator>,
    url: Option<&CFURL>,
    resource_data: *mut *const CFData,
    properties: *mut *const CFDictionary,
    desired_properties: Option<&CFArray>,
    error_code: *mut i32,
) -> bool {
    extern "C-unwind" {
        fn IOURLCreateDataAndPropertiesFromResource(
            alloc: Option<&CFAllocator>,
            url: Option<&CFURL>,
            resource_data: *mut *const CFData,
            properties: *mut *const CFDictionary,
            desired_properties: Option<&CFArray>,
            error_code: *mut i32,
        ) -> Boolean;
    }
    let ret = unsafe {
        IOURLCreateDataAndPropertiesFromResource(
            alloc,
            url,
            resource_data,
            properties,
            desired_properties,
            error_code,
        )
    };
    ret != 0
}

/// # Safety
///
/// - `url` might not allow `None`.
/// - `data_to_write` might not allow `None`.
/// - `properties_to_write` generics must be of the correct type.
/// - `properties_to_write` might not allow `None`.
/// - `error_code` must be a valid pointer.
#[inline]
pub unsafe extern "C-unwind" fn IOURLWriteDataAndPropertiesToResource(
    url: Option<&CFURL>,
    data_to_write: Option<&CFData>,
    properties_to_write: Option<&CFDictionary>,
    error_code: *mut i32,
) -> bool {
    extern "C-unwind" {
        fn IOURLWriteDataAndPropertiesToResource(
            url: Option<&CFURL>,
            data_to_write: Option<&CFData>,
            properties_to_write: Option<&CFDictionary>,
            error_code: *mut i32,
        ) -> Boolean;
    }
    let ret = unsafe {
        IOURLWriteDataAndPropertiesToResource(url, data_to_write, properties_to_write, error_code)
    };
    ret != 0
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iourlerror?language=objc)
#[repr(transparent)]
#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
pub struct IOURLError(pub c_int);
impl IOURLError {
    #[doc(alias = "kIOURLUnknownError")]
    pub const UnknownError: Self = Self(-10);
    #[doc(alias = "kIOURLUnknownSchemeError")]
    pub const UnknownSchemeError: Self = Self(-11);
    #[doc(alias = "kIOURLResourceNotFoundError")]
    pub const ResourceNotFoundError: Self = Self(-12);
    #[doc(alias = "kIOURLResourceAccessViolationError")]
    pub const ResourceAccessViolationError: Self = Self(-13);
    #[doc(alias = "kIOURLRemoteHostUnavailableError")]
    pub const RemoteHostUnavailableError: Self = Self(-14);
    #[doc(alias = "kIOURLImproperArgumentsError")]
    pub const ImproperArgumentsError: Self = Self(-15);
    #[doc(alias = "kIOURLUnknownPropertyKeyError")]
    pub const UnknownPropertyKeyError: Self = Self(-16);
    #[doc(alias = "kIOURLPropertyKeyUnavailableError")]
    pub const PropertyKeyUnavailableError: Self = Self(-17);
    #[doc(alias = "kIOURLTimeoutError")]
    pub const TimeoutError: Self = Self(-18);
}

#[cfg(feature = "objc2")]
unsafe impl Encode for IOURLError {
    const ENCODING: Encoding = c_int::ENCODING;
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IOURLError {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogadddrivers?language=objc)
pub const kIOCatalogAddDrivers: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogadddriversnomatch?language=objc)
pub const kIOCatalogAddDriversNoMatch: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogremovedrivers?language=objc)
pub const kIOCatalogRemoveDrivers: c_uint = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogremovedriversnomatch?language=objc)
pub const kIOCatalogRemoveDriversNoMatch: c_uint = 4;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogstartmatching__removed?language=objc)
pub const kIOCatalogStartMatching__Removed: c_uint = 5;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogremovekernellinker__removed?language=objc)
pub const kIOCatalogRemoveKernelLinker__Removed: c_uint = 6;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogkextdactive?language=objc)
pub const kIOCatalogKextdActive: c_uint = 7;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogkextdfinishedlaunching?language=objc)
pub const kIOCatalogKextdFinishedLaunching: c_uint = 8;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogresetdrivers?language=objc)
pub const kIOCatalogResetDrivers: c_uint = 9;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogresetdriversnomatch?language=objc)
pub const kIOCatalogResetDriversNoMatch: c_uint = 10;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocataloggetcontents?language=objc)
pub const kIOCatalogGetContents: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocataloggetmoduledemandlist?language=objc)
pub const kIOCatalogGetModuleDemandList: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocataloggetcachemisslist?language=objc)
pub const kIOCatalogGetCacheMissList: c_uint = 3;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocataloggetrommkextlist?language=objc)
pub const kIOCatalogGetROMMkextList: c_uint = 4;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogresetdefault?language=objc)
pub const kIOCatalogResetDefault: c_uint = 1;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogmoduleunload?language=objc)
pub const kIOCatalogModuleUnload: c_uint = 1;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogmoduleterminate?language=objc)
pub const kIOCatalogModuleTerminate: c_uint = 2;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiocatalogserviceterminate?language=objc)
pub const kIOCatalogServiceTerminate: c_uint = 3;

/// # Safety
///
/// - `buffer` must be a valid pointer.
/// - `allocator` might not allow `None`.
/// - `error_string` must be a valid pointer.
#[inline]
pub unsafe extern "C-unwind" fn IOCFUnserialize(
    buffer: *const c_char,
    allocator: Option<&CFAllocator>,
    options: CFOptionFlags,
    error_string: *mut *const CFString,
) -> Option<CFRetained<CFType>> {
    extern "C-unwind" {
        fn IOCFUnserialize(
            buffer: *const c_char,
            allocator: Option<&CFAllocator>,
            options: CFOptionFlags,
            error_string: *mut *const CFString,
        ) -> Option<NonNull<CFType>>;
    }
    let ret = unsafe { IOCFUnserialize(buffer, allocator, options, error_string) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// # Safety
///
/// - `buffer` must be a valid pointer.
/// - `allocator` might not allow `None`.
/// - `error_string` must be a valid pointer.
#[inline]
pub unsafe extern "C-unwind" fn IOCFUnserializeBinary(
    buffer: *const c_char,
    buffer_size: usize,
    allocator: Option<&CFAllocator>,
    options: CFOptionFlags,
    error_string: *mut *const CFString,
) -> Option<CFRetained<CFType>> {
    extern "C-unwind" {
        fn IOCFUnserializeBinary(
            buffer: *const c_char,
            buffer_size: usize,
            allocator: Option<&CFAllocator>,
            options: CFOptionFlags,
            error_string: *mut *const CFString,
        ) -> Option<NonNull<CFType>>;
    }
    let ret =
        unsafe { IOCFUnserializeBinary(buffer, buffer_size, allocator, options, error_string) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// # Safety
///
/// - `buffer` must be a valid pointer.
/// - `allocator` might not allow `None`.
/// - `error_string` must be a valid pointer.
#[inline]
pub unsafe extern "C-unwind" fn IOCFUnserializeWithSize(
    buffer: *const c_char,
    buffer_size: usize,
    allocator: Option<&CFAllocator>,
    options: CFOptionFlags,
    error_string: *mut *const CFString,
) -> Option<CFRetained<CFType>> {
    extern "C-unwind" {
        fn IOCFUnserializeWithSize(
            buffer: *const c_char,
            buffer_size: usize,
            allocator: Option<&CFAllocator>,
            options: CFOptionFlags,
            error_string: *mut *const CFString,
        ) -> Option<NonNull<CFType>>;
    }
    let ret =
        unsafe { IOCFUnserializeWithSize(buffer, buffer_size, allocator, options, error_string) };
    ret.map(|ret| unsafe { CFRetained::from_raw(ret) })
}

/// IOMessage.h
///
/// Defines message type constants for several IOKit messaging API's.
///
/// See also [Apple's documentation](https://developer.apple.com/documentation/iokit/iomessage?language=objc)
pub type IOMessage = u32;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/osobjectref?language=objc)
pub type OSObjectRef = u64;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcversion190615?language=objc)
pub const kIORPCVersion190615: c_uint = 0x4da2b68c;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcversion190615reply?language=objc)
pub const kIORPCVersion190615Reply: c_uint = 0x4da2b68d;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessageremote?language=objc)
pub const kIORPCMessageRemote: c_uint = 0x00000001;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessagelocalhost?language=objc)
pub const kIORPCMessageLocalHost: c_uint = 0x00000002;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessagekernel?language=objc)
pub const kIORPCMessageKernel: c_uint = 0x00000004;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessageoneway?language=objc)
pub const kIORPCMessageOneway: c_uint = 0x00000008;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessageobjectrefs?language=objc)
pub const kIORPCMessageObjectRefs: c_uint = 0x00000010;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessageonqueue?language=objc)
pub const kIORPCMessageOnqueue: c_uint = 0x00000020;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessageerror?language=objc)
pub const kIORPCMessageError: c_uint = 0x00000040;
/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kiorpcmessagesimplereply?language=objc)
pub const kIORPCMessageSimpleReply: c_uint = 0x00000080;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iorpcmessage?language=objc)
#[repr(C, packed(4))]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IORPCMessage {
    pub msgid: u64,
    pub flags: u64,
    pub objectRefs: u64,
    pub objects: [OSObjectRef; 0],
}

#[cfg(feature = "objc2")]
unsafe impl Encode for IORPCMessage {
    const ENCODING: Encoding = Encoding::Struct(
        "IORPCMessage",
        &[
            <u64>::ENCODING,
            <u64>::ENCODING,
            <u64>::ENCODING,
            <[OSObjectRef; 0]>::ENCODING,
        ],
    );
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for IORPCMessage {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/iorpcmessageerrorreturncontent?language=objc)
#[cfg(feature = "libc")]
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct IORPCMessageErrorReturnContent {
    pub hdr: IORPCMessage,
    pub result: libc::kern_return_t,
    pub pad: u32,
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl Encode for IORPCMessageErrorReturnContent {
    const ENCODING: Encoding = Encoding::Struct(
        "IORPCMessageErrorReturnContent",
        &[
            <IORPCMessage>::ENCODING,
            <libc::kern_return_t>::ENCODING,
            <u32>::ENCODING,
        ],
    );
}

#[cfg(all(feature = "libc", feature = "objc2"))]
unsafe impl RefEncode for IORPCMessageErrorReturnContent {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/kosclasscanremote?language=objc)
pub const kOSClassCanRemote: c_uint = 0x00000001;

/// [Apple's documentation](https://developer.apple.com/documentation/iokit/osclassdescription?language=objc)
#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq)]
pub struct OSClassDescription {
    pub descriptionSize: u32,
    pub name: [c_char; 96],
    pub superName: [c_char; 96],
    pub methodOptionsSize: u32,
    pub methodOptionsOffset: u32,
    pub metaMethodOptionsSize: u32,
    pub metaMethodOptionsOffset: u32,
    pub queueNamesSize: u32,
    pub queueNamesOffset: u32,
    pub methodNamesSize: u32,
    pub methodNamesOffset: u32,
    pub metaMethodNamesSize: u32,
    pub metaMethodNamesOffset: u32,
    pub flags: u64,
    pub resv1: [u64; 8],
    pub methodOptions: [u64; 0],
    pub metaMethodOptions: [u64; 0],
    pub dispatchNames: [c_char; 0],
    pub methodNames: [c_char; 0],
    pub metaMethodNames: [c_char; 0],
}

#[cfg(feature = "objc2")]
unsafe impl Encode for OSClassDescription {
    const ENCODING: Encoding = Encoding::Struct(
        "OSClassDescription",
        &[
            <u32>::ENCODING,
            <[c_char; 96]>::ENCODING,
            <[c_char; 96]>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u32>::ENCODING,
            <u64>::ENCODING,
            <[u64; 8]>::ENCODING,
            <[u64; 0]>::ENCODING,
            <[u64; 0]>::ENCODING,
            <[c_char; 0]>::ENCODING,
            <[c_char; 0]>::ENCODING,
            <[c_char; 0]>::ENCODING,
        ],
    );
}

#[cfg(feature = "objc2")]
unsafe impl RefEncode for OSClassDescription {
    const ENCODING_REF: Encoding = Encoding::Pointer(&Self::ENCODING);
}

#[cfg(feature = "libc")]
#[deprecated = "renamed to `IONotificationPort::create`"]
#[inline]
pub extern "C-unwind" fn IONotificationPortCreate(
    main_port: libc::mach_port_t,
) -> IONotificationPortRef {
    extern "C-unwind" {
        fn IONotificationPortCreate(main_port: libc::mach_port_t) -> IONotificationPortRef;
    }
    unsafe { IONotificationPortCreate(main_port) }
}

extern "C-unwind" {
    #[deprecated = "renamed to `IONotificationPort::destroy`"]
    pub fn IONotificationPortDestroy(notify: IONotificationPortRef);
}

#[deprecated = "renamed to `IONotificationPort::run_loop_source`"]
#[inline]
pub unsafe extern "C-unwind" fn IONotificationPortGetRunLoopSource(
    notify: IONotificationPortRef,
) -> Option<CFRetained<CFRunLoopSource>> {
    extern "C-unwind" {
        fn IONotificationPortGetRunLoopSource(
            notify: IONotificationPortRef,
        ) -> Option<NonNull<CFRunLoopSource>>;
    }
    let ret = unsafe { IONotificationPortGetRunLoopSource(notify) };
    ret.map(|ret| unsafe { CFRetained::retain(ret) })
}

extern "C-unwind" {
    #[cfg(feature = "libc")]
    #[deprecated = "renamed to `IONotificationPort::mach_port`"]
    pub fn IONotificationPortGetMachPort(notify: IONotificationPortRef) -> libc::mach_port_t;
}

extern "C-unwind" {
    #[cfg(feature = "libc")]
    #[deprecated = "renamed to `IONotificationPort::set_importance_receiver`"]
    pub fn IONotificationPortSetImportanceReceiver(
        notify: IONotificationPortRef,
    ) -> libc::kern_return_t;
}

extern "C-unwind" {
    #[cfg(feature = "dispatch2")]
    #[deprecated = "renamed to `IONotificationPort::set_dispatch_queue`"]
    pub fn IONotificationPortSetDispatchQueue(
        notify: IONotificationPortRef,
        queue: Option<&DispatchQueue>,
    );
}
